/*
Java Hummer Project
File: HummerI.java
Date: 4-9-99
Author: Chad Ward
Description:  Connects to a Hummer server and will start up,
              stop, and get basic information about the status
              of the Hummer servers (Config, Peer, Message)


 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.client;

import java.util.Enumeration;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Hashtable;

import java.net.Socket;
import java.net.UnknownHostException;
import java.net.InetAddress;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.StreamCorruptedException;
import java.io.OptionalDataException;

import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.SignatureException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.server.Hummer;
import edu.uidaho.hummer.dataobject.PersonalConfig;
import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.socket.HummerClientSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;

/**
Connects to a Hummer server and will start up,
stop, and get basic information about the status
of the Hummer servers (Config, Peer, Message).  This is
done by logging into the Hummer server with the admin account.
<br><b>Note:</b></br>
For this method to work, the user account must have already been 
initialized(previously logged in), and the user account on the
target Hummer must be allowed to start and stop Hummer servers.
@author Chad Ward
*/
public final class HummerI {

	/**
	Number of choices in the menu.
	*/
  private final int _number_of_choices = 11;

  private Properties _constants;

	/**
	Table of access values for the current machine that the user is connected
	to.
	*/
  private Hashtable _access_table;

	/**
	Address of the Hummer that is being logged into.
	*/
  private InetAddress _hummer_address;

	/**
	PersonalConfig that will be used to login to the Hummer system.  The alias must also
	be the filename of the HummerKeyStore.
	*/
  private PersonalConfig _personal_config;

	/**
	HummerSocket used for communication with the server.  This needs to be a class
	field, because it is used in both startHummerI and adminMenu
	*/
  private HummerClientSocket _hummer_client_socket = null;

	/**
	Key store of the user logging into the Hummer server.
	*/
  private HummerKeyStore _key_store;

/*
====================================================================================
Main method
*/

	/**
	Main is the method which starts the whole thing going.

	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li> Main creates a new Hummer
	  <li> Main starts the new Hummer
	</ul>
	@param argv[] The name/address of the Hummer to connect to.
	*/
  public static void main(String argv[]) {
    String usage = "Usage:\n\tjava edu.uidaho.hummer.client.HummerI <username> <name|address> <password> \n" +
                   "\tPassword is only if the account has not been initialized yet.";
    if(argv.length < 2 || argv.length > 3) {
      System.err.println(usage);
      return;
    }
    System.out.println("\n\n======================================================================\n" +
                       "Welcome to the Hummer system - HummerI\n" +
                       " Copyright (C) 1997-1998 The Hummer " +
                       "Project - hummer@cs.uidaho.edu\n" +
                       "----------------------------------------------------------------------");
    byte[] password = null;
    if(argv.length == 3) {
      password = argv[2].getBytes();
    }
    PersonalConfig personal_config = new PersonalConfig(argv[0], password);
    InetAddress hummer_address = null;
    try {
      hummer_address = InetAddress.getByName(argv[1]);
    }
    catch(UnknownHostException e) {
      System.err.println("HummerI:main: Could not connect to the given host (" + argv[1] + ").");
      return;
    }
    String key_store_filename = null;

    HummerI hummer_interface = new HummerI(personal_config, hummer_address);
    hummer_interface.startHummerI();
  }


/*
====================================================================================
constructor
*/
	/**
	Sets the _hummer_properties value.
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>get Properties object from StaticMethods.getHummerProperties()
	</ul>
      @see edu.uidaho.hummer.util.StaticMethods#getHummerProperties()
	*/
  public HummerI(PersonalConfig personal_config, InetAddress hummer_address) {
    this._personal_config = personal_config;
    this._hummer_address = hummer_address;
    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch(FileNotFoundException e) {
      System.err.println("HummerI:constructor: Could not find Hummer.properties file: " + e);
    }
    catch(IOException e) {
      System.err.println("HummerI:constructor: " + e);
    }
  }


/*
====================================================================================
methods
*/

	/**
	Attempts to connect to Hummer server before taking input on what
	the user would like to do with the server.
	*/
  public boolean startHummerI() {
    Socket socket = null;
    Response input;
    Integer port = null;
    try {
      _key_store = HummerKeyStore.getInstance();
      System.out.print("HummerI: Now loading key store...");
      try {
        _key_store.load(false, _personal_config.getAlias());
        System.out.println("loaded.");
      }
      catch(UnknownHostException e) {
        System.out.println("HummerI:main: Could not get HummerKeyStore filename: " + e);
        return false;
      }
      catch(FileNotFoundException e) {
        System.out.println("HummerI:main: Could not get HummerKeyStore filename: " + e);
        return false;
      }
      catch(IOException e) {
        System.out.println("HummerI:main: Could not get HummerKeyStore filename: " + e);
        return false;
      }
      port = (new Integer(_constants.getProperty("port.HUMMER_PORT")));
    }
    catch(KeyStoreException e) {
      System.out.println("HummerI:start: Could not load HummerKeyStore: " + e);
      return false;
    }
    catch(ClassNotFoundException e) {
      System.out.println("HummerI:start: Could not load HummerKeyStore: " + e);
      return false;
    }

		/*
		Have connected and validated with server, now find out what
		the user wants to do.
		*/
    int choice;
    try {
		/**
		The last value in the menu should always be the choice to
		end transmission.
		*/
      do {
        System.out.print("HummerI: Connecting to " +
                         _hummer_address.getHostName() + "...");
        socket = new Socket(_hummer_address, port.intValue());
        System.out.println("connected.");
        _hummer_client_socket = new HummerClientSocket(_personal_config, socket, _key_store);
        System.out.print("HummerI: Validating...");
        _hummer_client_socket.validation(null);
        _access_table = _key_store.getHummerCertificate(_hummer_address.getHostAddress()).getAccessTable();
        System.out.println("validated.");
        _key_store.store(false, _personal_config.getAlias());
        input = _hummer_client_socket.readResponseObject();
        String[] choice_list = (String[])input._message;
        choice = this.getUserChoice(choice_list);
        while(choice != 0) {
          Response output = new Response();
          output._message = new Integer(choice);
          _hummer_client_socket.writeResponseObject(output);
          input = _hummer_client_socket.readResponseObject();
          if(input._type == Response._EOT) {
            _hummer_client_socket.close();
          }
          System.out.println("\nServer Name\tRunning\t\tStatus\n" +
                             "-----------------------------------------------------------------------");
          Object[] server_info;
          for(Enumeration enum = input._data.elements(); enum.hasMoreElements();) {
            server_info = (Object[])enum.nextElement();
            if(server_info == null && server_info.length != 3) {
              System.out.println("HummerI: Server has sent invalid information. Disconnecting.");
              choice = 0;
              break;
            }
            System.out.println(server_info[0].toString() + "\t" + server_info[1] +
                               "\t\t" + server_info[2]);
          }
          System.out.println("\n");

          if(choice != 0) {
      		/*
      		If the server has sent invalid information, then the system should
      		disconnect without taking information from the user again.
      		*/
            choice = this.getUserChoice(choice_list);
          }
        }
        _hummer_client_socket.endOfTransmission(null);
        _hummer_client_socket.close();
      } while((_hummer_address = this.getNextConnection()) != null);
    }
    catch(HummerSocketException e) {
      System.err.println("HummerI:start: Error while connecting to Hummer server:\n" + e);
      return false;
    }
    catch(IOException e) {
      System.err.println("HummerI:start: Error while connecting to Hummer server:\n" + e);
      System.err.println("\n*******************************************************" +
                         "\nYou must first start the Hummer.  Use:" +
                         "\n\tjava edu.uidaho.hummer.server.Hummer <options>" +
                         "\n*******************************************************");
      return false;
    }
    catch(UnknownAliasException e) {
      System.err.println("HummerI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("HummerI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(KeyStoreException e) {
      System.out.println("HummerI:start: Could not store HummerKeyStore: " + e);
      return false;
    }
    catch(CertificateException e) {
      System.err.println("HummerI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(NoSuchAlgorithmException e) {
      System.err.println("HummerI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(UnrecoverableKeyException e) {
      System.err.println("HummerI:start: Error while validating to Hummer server:\n" + e);
      return false;
    }
    System.out.println("\n\nThank you for supporting your local Java addicts.\n");
    return true;
  }

	/**
	Reads what server (if any) that the user would like to connect to next.
	@returns The InetAddress of the next system to connect to.  Null is returned
		if the user does not wish to connect to another server.
	*/
  private InetAddress getNextConnection() {
    String response = null;
    InetAddress address = null;
    BufferedReader in_stream = new BufferedReader(new InputStreamReader(System.in));
    while(address == null) {
      System.out.print("\n\nDo you wish to connect to another machine? (y/n) ");
      try {
        response = in_stream.readLine().toLowerCase();
        while(!response.equals("n") && !response.equals("y")) {
          System.out.print("Do you wish to connect to another machine? (y/n) ");
          response = in_stream.readLine();
        }
        if(response.equals("n")) {
          return null;
        }
        System.out.print("\n\nPlease enter the target host's address: ");
        response = in_stream.readLine();
        try {
          address = InetAddress.getByName(response);
        }
        catch(UnknownHostException e) {
          System.out.println("HummerI: Could not find the specified host.");
        }
      }
      catch(IOException e) {
        System.err.println("HummerI:getNextConnection: " +
                           "Error while reading from standard input: " + e);
        return null;
      }
    }

	/**
	Will user be logging into an initialized account or new account?
	*/
    String new_account = "invalid";
    try {
      while(!response.equals("y") && !response.equals("n")) {
        System.out.print("\n\nIs this the first time to log into " + address + "? (y/n) ");
        response = in_stream.readLine().toLowerCase();
      }
      if(response.equals("y")) {
        System.out.print("Please enter a password for the account: ");
        response = in_stream.readLine();
        _personal_config = new PersonalConfig(_personal_config.getAlias(), response.getBytes());
      }
    }
    catch(IOException e) {
      System.err.println("HummerI:getNextConnection: " +
                         "Error while reading from standard input: " + e);
      return null;
    }

    System.out.println("======================================================\n");
    return address;
  }

	/**
	Reads in the user's choice from STDIN of what 
	option they would like Hummer to execute.

	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>Read in option from STDIN.  All options, except 0 (End transmission) are
	  	sent by the server.
	</ul>
	@return The integer value representing the selection.
	*/
  private int getUserChoice(String[] choice_list) throws IOException {
    int value = -1;
    String input;
    boolean first_time = true;
    BufferedReader in_stream = new BufferedReader(new InputStreamReader(System.in));

    while(value < 0 || value > choice_list.length) {
      if(first_time) {
        System.out.println("======================================================\n" +
                           "Connected to " + _hummer_address + ".\n" +
                           "Please make a choice.\n" +
                           "------------------------------------------------------\n");
        first_time = false;
      }
      else {
        System.out.println("======================================================\n" +
                           "Connected to " + _hummer_address + ".\n" +
                           "\nThat is not a valid choice:\n" +
                           "Please make a choice.\n" +
                           "------------------------------------------------------");
      }
      if(_personal_config.getAlias().equals("admin")) {
        System.out.println("A - Account administration");
      }
      if(_personal_config.getAlias().equals("admin") ||
         ((Boolean)_access_table.get("Control_Tools")).booleanValue()) {
        System.out.println("T - Tool administration");
      }
      System.out.println("0 - End Transmission");
      for(int i = 0; i < choice_list.length; i++) {
        System.out.println((i + 1) + " - " + choice_list[i]);
      }
      System.out.print("---------------------------------------\nChoice - ");
      input = in_stream.readLine().trim();
      if(input.toLowerCase().equals("a")) {
        try {
          adminMenu();
        }
        catch(HummerSocketException e) {
          System.err.println("HummerI: Error during admininistration:" + e);
        }
        value = edu.uidaho.hummer.server.Hummer._DISPLAY_STATUS;
      }
      else if(input.toLowerCase().equals("t")) {
//        try {
          ToolI tool_interface = new ToolI(_personal_config, _hummer_address);
          tool_interface.startToolI();    
//        }
//        catch(HummerSocketException e) {
//          System.err.println("HummerI: Error during admininistration:" + e);
//        }
        value = edu.uidaho.hummer.server.Hummer._DISPLAY_STATUS; 
      }
      else {
        try {
          value = (new Integer(input)).intValue();
        }
        catch (NumberFormatException e) {}
      }
      System.out.println("---------------------------------------");
    }
    return value;
  }

	/**
	Method for handling the administration of user accounts on the remote
	Hummer server.  Presents a menu of choices, and sends requests based
	upon the choice of the user.
	*/
  private void adminMenu() throws IOException, HummerSocketException {
    int choice = -1;
    final int BACK = 0,
              LIST = 1,
              ALIAS_ACCESS = 2,
              ADD_USER = 3,
              ADD_SERVER = 4,
              ADD_PEER = 5,
              REMOVE_ALIAS = 6,
              RESET_ALIAS = 7;
    String input;
    String[] choice_list = {"0 - Back to main menu",
                            "1 - List users",
                            "2 - List user access values (2 <alias>)",
                            "3 - Add user (3 <user_alias> <password>)",
                            "4 - Add server (4 <server_alias> <is_trusted-y|n> <password>)",
                            "5 - Add peer (5 <peer_alias> <password>",
                            "6 - Delete alias (6 <username>)",
                            "7 - Reset alias (7 <username> <new_password>)"
                           };
    BufferedReader in_stream = new BufferedReader(new InputStreamReader(System.in));
    System.out.println("\n");
    StringTokenizer tokenizer = null;
    while(choice != BACK) {
      while(choice < 0 || choice > choice_list.length) {
        System.out.println("======================================================\n" +
                           "Administration Menu for " + _hummer_address + "(admin access only)\n" +
                           "Please make a choice.\n" +
                           "------------------------------------------------------\n");
        for(int i = 0; i < choice_list.length; i++) {
          System.out.println(choice_list[i]);
        }
        System.out.print("------------------------------------------------------\n" +
                         "Choice - ");
        input = in_stream.readLine().trim();
        tokenizer = new StringTokenizer(input);
        try {
          choice = (new Integer(tokenizer.nextToken())).intValue();
        }
        catch(NumberFormatException e) {}
        if(choice == BACK) {
          System.out.print("\n");
        }
        else {
          System.out.println("\n==================================================");
        }
      }	// while
      if(choice == BACK) {
        return;
      }

      String alias, password;
      boolean trusted;
      switch(choice) {
        case LIST:
          Vector alias_list = _hummer_client_socket.getAliasList();
          Enumeration enum = alias_list.elements();
          System.out.println(HummerKeyStore.statisticsToString((Object[])enum.nextElement(), 30, 20) +
                             "\n----------------------------------------------------------------" +
                             "-----------------------------------------------");

          while(enum.hasMoreElements()) {
            System.out.println(HummerKeyStore.statisticsToString((Object[])enum.nextElement(), 30, 20));
          }
          break;
        case ALIAS_ACCESS:
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ALIAS_ACCESS + " <alias>");
            break;
          }
          else {
            alias = tokenizer.nextToken();
            Hashtable access_table = _hummer_client_socket.getAccessTable(alias);
            System.out.println("\tAccess Name\tValue\n" +
                               "-----------------------------------------------------------");
            String current_key;
            for(enum = access_table.keys(); enum.hasMoreElements();) {
              current_key = enum.nextElement().toString();
              System.out.println("\t" + current_key + "\t" +
                                 access_table.get(current_key).toString());
            }
          }
          break;
        case ADD_USER:
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_USER +
                               " <user_alias> <password>");
            break;
          }
          alias = tokenizer.nextToken();
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_USER +
                               " <user_alias> <password>");
            break;
          }
          password = tokenizer.nextToken();
          _hummer_client_socket.addUserAlias(alias, password.getBytes());
          System.out.println("\nHave made request to server.  Use alias list for results.");
          break;
        case ADD_SERVER:
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_SERVER +
                               "<server_alias> <is_trusted-y|n> <password>)");
            break;
          }
          alias = tokenizer.nextToken();
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_SERVER +
                               "<server_alias> <is_trusted-y|n> <password>)");
            break;
          }
          input = tokenizer.nextToken().toLowerCase();
          if(!input.equals("y") && !input.equals("n")) {
            System.out.println("Invalid format.  Should be " + ADD_SERVER +
                               "<server_alias> <is_trusted-y|n> <password>)");
            break;
          }
          trusted = input.equals("y");
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_USER +
                               " <user_alias> <password>");
          }
          password = tokenizer.nextToken();
          _hummer_client_socket.addServerAlias(alias, trusted, password.getBytes());
          System.out.println("\nHave made request to server.  Use alias list for results.");
          break;
        case ADD_PEER:
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_PEER +
                               " <peer_alias> <password>");
            break;
          }
          alias = tokenizer.nextToken();
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + ADD_PEER +
                               " <peer_alias> <password>");
            break;
          }
          password = tokenizer.nextToken();
          _hummer_client_socket.addPeerAlias(alias, password.getBytes());
          System.out.println("\nHave made request to server.  Use alias list for results.");

          break;
        case REMOVE_ALIAS:
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + REMOVE_ALIAS +
                               " <alias>");
            break;
          }
          alias = tokenizer.nextToken();
          _hummer_client_socket.deleteAlias(alias);
          System.out.println("\nHave made request to server.  Use alias list for results.");
          break;

        case RESET_ALIAS:
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + RESET_ALIAS +
                               " <alias> <new_password>");
            break;
          }
          alias = tokenizer.nextToken();
          if(!tokenizer.hasMoreElements()) {
            System.out.println("Invalid format.  Should be " + RESET_ALIAS +
                               " <alias> <new_password>");
            break;
          }
          password = tokenizer.nextToken();
          _hummer_client_socket.resetAlias(alias, password.getBytes());
          System.out.println("\nHave made request to server.  Use alias list for results.");
          break;
      }
      choice = -1;
      System.out.print("\n");
    }
  }
}
