/*
PeerI.java
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke

Coded by: Joel Marlow and Matt McDonald 
Adapted from: PeerIMain.c and PeerIMain.h
Description: This file acts as a client within peer moderation.
  It is used to connect to PeerServ in order to subscribe, unsubscribe, 
  and send messages to the Peer Server (or moderator) running somewhere
  on the internet.

 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
package edu.uidaho.hummer.client;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;
import java.util.Calendar;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerClientSocket;
import edu.uidaho.hummer.dataobject.*;



/**
This class acts as a client within peer moderation.  It is used to connect
to PeerServ in order to subscribe, unsubscribe, and send messages to the
Peer Server (or moderator) running somewhere on the internet.

@author Joel Marlow, Jesse C. McConnell, Matt McDonald
 */
public class PeerI{


  // global vars for peer communication
  /**
  System Properties object that when created contains the System.getProperties and gets the properties defined in Hummer.properties.
  */
  private Properties _constants = new Properties(System.getProperties());

  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _url = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  private String _password = new String();
  /**
  int representing the port number the MessagingServer should be running on.
  The value of this int is set in the Hummer.properties file and should only be
  set via the _constants Properties object.
  */
  private int port; 
  /**
  Basic Statement object.  Probably need not be a class variable. FIX
  */
  //++Needed for authentication
  HummerKeyStore _key_store;
  //++Needed for authentication
  PersonalConfig _personal;

//  private Statement statement = null;

/**
Creates the PeerI object and sets up all of the variables that are from the
.properties file. 
*/
  public PeerI() {
    String hum_name = "";
    String driver = "";
    String address = new String();

  //++This try and catch is needed for authentication
    try {
      address = InetAddress.getLocalHost().getHostAddress();
      //    address = "admin";
      //      System.out.println(address);
      _key_store = HummerKeyStore.getInstance();
      _key_store.load(true, address);
    }
    catch (Exception e) {
      System.err.println("ConfigI: " + e);
    }
    //++Needed for authentication
    _personal = new PersonalConfig(address, null);


    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch (FileNotFoundException fnfe) {
      System.err.println("PeerI: Hummer.properties file not found.");
      System.exit(0);
    }
    catch (IOException ioe) {
      System.err.println("PeerI: File Hummer.properties not valid.");
      System.exit(0);
    }
    catch (NumberFormatException ne) {
      System.err.println("PeerI: Bad port.PEER_PORT property.");
      System.exit(0);
    }

    try {
      address = InetAddress.getLocalHost().getHostAddress();
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
        _url = _constants.getProperty("jdbc.url." + address) + nametoken.nextToken();
      }
      else {
        _url = _constants.getProperty("jdbc.url");
      }

      if (_constants.containsKey("jdbc.driver." + address)) {
        driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
        driver = _constants.getProperty("jdbc.driver");
      }

      if (_constants.containsKey("jdbc.user." + address)) {
        _user = _constants.getProperty("jdbc.user." + address);
      }
      else {
        _user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
        _password = _constants.getProperty("jdbc.password." + address);
      }
      else {
        _password = _constants.getProperty("jdbc.password");
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("PeerI:" + uhe);
    }
    try {
      Class.forName(driver);
    } 
    catch (ClassNotFoundException C) { 
      System.err.println ("PeerI:" + C);
    }
  
  }


  /**
  This is the main function of the peer interface. It handles the input
  passed into it via command line parameters. If there are no command line
  arguments then it prints out the instructions on the usage of PeerI.

  @param args This is what stores the command line arguments.
  */

  public static void main(String args[]){

    PeerI peer = new PeerI();
    
    if(args.length > 0){
      if (!peer.processParameters(args)) {
        peer.printInstructions();
	System.exit(0);
      }
    } 
    else{
      peer.printInstructions();        // Print the instructions 
      System.exit(0);
    }
  }


  /**
  This prints the instructions on the usage for PeerI. 
  */
  public void printInstructions(){
    
    System.out.println("usage: PeerI c \"<peer group>\" \"<message>\"\n" +
		       "             s \"<peer group>\"" + 
		       " \"<moderator address>\"\n" +
		       "             u \"<peer group>\"\n\n" + 
		       "NOTE: To communicate a message, you must be a " + 
		       "member of the <peer group>");
  }


  /**
  This method parses the command line parameters and places the data in the
  message object. This function also reports errors in the command line so
  the instructions can be printed. It also sets up any of the checks that
  will tell the program whether or not it has the correct parameters.


  @param args This is what stores the command line arguments.
  @return Boolean representing the success of the parameter parsing.
  */

  public boolean processParameters(String args[]){
    
    int i = 0;

    switch (args[i].charAt(0)){
      
    case 'c':
      return sendMessageToGroup(args[++i], args[++i]);
    case 's':
      try {
	return subscribeToGroup(args[++i], InetAddress.getByName(args[++i]));
      }
      catch (UnknownHostException uhe) {
	System.err.println("PeerI:processParameters:" + uhe);
	return false;
      }
    case 'u':
      return unsubscribeFromGroup(args[++i]);
    default:
      System.out.println("PeerI:processParamters:Unknown option: " + args[i]);
      return false;
    }
  }


  /**
  This method returns the Moderator address associated with a peer group name.
  If the database call fails then null is returned.
  <p>
  Database Call:
  <pre>
  SELECT * FROM pgroups WHERE pgroup = '" + peer_grp +"';

  </pre>  
  @param peer_grp This String is the name of the peer group that the user 
  wants to be added to.
  @return An InetAddress representing the address for a peer group, or null.
  */
  public InetAddress getModeratorAddress(String peer_grp) {

    
    try {

      Connection db_connection = DriverManager.getConnection(
					       _url, _user, _password);
      Statement statement = db_connection.createStatement();
      
      String query = "SELECT * FROM pgroups WHERE pgroup = '" + peer_grp +"';";
      
      ResultSet result_set = statement.executeQuery(query);
      result_set.next();

      try {
	String temp = result_set.getString("mod_addr");
	InetAddress addr = InetAddress.getByName(temp);
	return addr;
      }
      catch (NullPointerException npe) {
	System.err.println("PeerI:getModeratorAddress:" + npe);
      }

       statement.close();
      db_connection.close();
      return null;
    } 
    catch (SQLException sqle) { 
      System.err.println ("PeerI.getModeratorAddress" + sqle); 
      return null;
    }
    catch(UnknownHostException uhe){
      System.err.println("PeerI:getModeratorAddress:" + uhe);
      return null;
    }
  }

 


  /**
  This method opens a connection to mod_addr and sends a request Peer object 
  for subscription to the peer group.  Upon success, the server will
  send the Peer object back with all of the items 
  relevant to the peer group we are seeking to subscribe to.  The
  subscription will update the tables "peers", "pgroups", and "levels" in our
  database.

  <p>
  Database Call:
  <pre>
   1 - INSERT INTO peers VALUES ( '" + new_pgroup + "', '" + new_mem_addr +
       "', " + new_peer_sub + ", " + new_peer_mod + ");
   2* - INSERT INTO levels VALUES ('" + new_mem_addr + "','',1,1,1,0);
   3 - INSERT INTO pgroups VALUES ( '" + new_pgroup + "', '" + new_mod_addr
       + "', '" + new_email + "');
   4* - DELETE FROM levels WHERE start_addr='" + peer_name + "';
   * The adding and deleting levels was commented out, because there was no
      check for it when it was adding. Add it did not need to be deleted when 
      a Hummer unsubscribed from a Peer Group.
  </pre>
  */

  public boolean subscribeToGroup(String pg_name, InetAddress mod_addr) {

    // DECLARATIONS 

    //    System.out.println(pg_name + " ... " + mod_addr);
    Socket hummer_socket;        // Socket Object
    Peer peer_obj = new Peer();
    Peer return_obj;
    //++Needed for authentication
    Response response = new Response();
    //++Needed for authentication
    HummerClientSocket dcs = null;

    //Send the subscription object on the wire
    try{
      hummer_socket = new Socket(mod_addr, Integer.parseInt(
                                 _constants.getProperty("port.PEER_PORT")));

      //Setup the subscription object
      peer_obj.action = "subscribe";
      peer_obj.peer_group = pg_name;
      dcs = new HummerClientSocket(_personal, hummer_socket, _key_store);
      //    dcs.validation("129.101.54.53"); this should be the same as below
      dcs.validation(null);

      //++Needed for authentication
      response._type = Response._OK;
      //++Needed for authentication
      response._message = peer_obj;
      // End of protocol mesage, and close out connection
      //++This try and catch is needed for authentication

      dcs.writeResponseObject(response);
      //Read in the response subscription object
      response = dcs.readResponseObject();
      return_obj = (Peer)response._message;
      dcs.close();
    }
    catch (IOException ioe) {
      System.err.println("PeerI:subscribeToGroup:" + ioe); 
      return false;
    }
    catch (Exception e) {
      System.err.println("PeerI:subscribeToGroup: " + e);
      e.printStackTrace();
      return false;
    }



      //Make sure the Date Object is the same for security reasons
      if (!peer_obj.date.equals(return_obj.date)) {
	
	System.err.println("PeerI:subscribeToGroup:Security Issue:Different" +
			   " Date Objects!");
	return false;
      }


      if (!return_obj.action.equals("subscribed")) {
	System.err.println("PeerI:subscribeToGroup:Failed to join " + pg_name);
	return false;
      }
      else {
	//System.out.println("PeerI:subscribeToGroup:Joined " + pg_name);
      }
	

      // We can subscribe, so we continue...
      // UPDATE OUR DATABASE TABLES "peers","levels", AND "pgroups" WITH 
      // PEER GROUP INFO FROM SERVER
      // Read in the number of table entries for peers in our peer group
      
      try {
	
	// Receive all of the server's "peers" and "levels" table info 
	//regarding
	// our peer group, and insert
	// it into our "peers" and "levels" table (all levels default to 1)
        Connection db_connection = DriverManager.getConnection(_url, _user, _password);
        Statement statement = db_connection.createStatement();
        String query = "";
        for(Enumeration enum = return_obj.statements.elements(); enum.hasMoreElements();) {
          query += enum.nextElement().toString();
          if(query.charAt(query.length() - 1) != ';') {
            query += ";";
          }
        }		  	   
        statement.executeUpdate(query);
        db_connection.close();
      }
      catch (SQLException sqle) { 
        System.err.println ("PeerI:subscribeToGroup:" + sqle); 
        return false;
      }
    return true;
  }


  

  /**
  This method opens a connection to the Peer Server (PeerServ) and sends a
  request for unsubscription to the server.  Upon success, the server will
  remove us from their database and we will proceed to clean up our 
  database.

  <p>
  Database Call:
  <pre>
  1 - delete from peers where pgroup = '" + _pg_name +"';
  2 - delete from pgroup where pgroup = '" + _pg_name +"';
  </pre>

  @param pg_name String value representing the peer group to unsubscribe from.
  @return Boolean value representing success of the unsubscribe action.
  */
  public boolean unsubscribeFromGroup(String pg_name) {

    Socket hummer_socket;        // Socket Object
    InetAddress moderator_addr;
    Peer peer_obj = new Peer();
    Peer return_obj;
    //++Needed for authentication
    Response response = new Response();
    //++Needed for authentication
    HummerClientSocket dcs = null;

    //Send the unsubscribe object
    try {
      
      moderator_addr = getModeratorAddress(pg_name);

      if (moderator_addr == null) {
	System.err.println("PeerI:unsubscribeFromGroup:Not member of " 
			   + pg_name);
	return false;
      }

      hummer_socket = new Socket(moderator_addr, Integer.parseInt(
                                 _constants.getProperty("port.PEER_PORT")));

      //Construct the Peer unsubscription object
      peer_obj.action = "unsubscribe";
      peer_obj.peer_group = pg_name;

      dcs = new HummerClientSocket(_personal, hummer_socket, _key_store);
      //    dcs.validation("129.101.54.53"); this should be the same as below
      dcs.validation(null);

      //++Needed for authentication
      response._type = Response._OK;
      //++Needed for authentication
      response._message = peer_obj;
      // End of protocol mesage, and close out connection
      //++This try and catch is needed for authentication

      dcs.writeResponseObject(response);
      //Read in the response subscription object
      response = dcs.readResponseObject();
      return_obj = (Peer)response._message;
      dcs.close();

      //Make sure the Date Object is the same for security reasons
      if (!peer_obj.date.equals(return_obj.date)) {
	
	System.err.println("PeerI:unsubscribeFromGroup:Security " +
			   "Issue:Different Date Objects!");
	return false;
      }

      if (!return_obj.action.equals("unsubscribed")) {
	System.err.println("PeerI:unsubscribeFromGroup:Failed to leave " 
			   + pg_name);
	return false;
      }
      else {
	//	System.out.println("PeerI:unsubscribeFromGroup:Left " + pg_name);
      }


      // We have unsubscribed, so we continue...
      // CLEAN UP OUR DATABASE (REMOVE OUR SUBSCRIPTION TO _pg_name)
      // Create a Database Object, if successful perform deletions...

      try {
	Connection db_connection = DriverManager.getConnection(
				    _url, _user, _password);
	Statement statement = db_connection.createStatement();
	
        String update = new String();
        update = "delete from peers where pgroup = '" + pg_name +"';";
        update += "delete from pgroups where pgroup = '" + pg_name +"';";
        statement.executeUpdate(update);
        statement.close();
	db_connection.close();
      }
      catch(SQLException sqle) {
        System.err.println("PeerI:unsubscribeFromGroup:" + sqle);
        return false;
      }
      return true;
    }
    catch(UnknownHostException uhe) {
      System.err.println("PeerI:unsubscribeFromGroup:" + uhe);
      return false;
    }
    catch(IOException ioe) {
      System.err.println("PeerI:unsubscribeFromGroup:" + ioe);
      return false;
    }
    catch (Exception e) {
      System.err.println("PeerI:unsubscribeFromGroup: " + e);
      return false;
    }
  }  



  /**
  This method opens a connection to a Peer Server and sends a Peer object 
  to it. 
  @param pg_name Name of the Peer Group message is to be sent to.
  @param message String to be sent to the Peer Group.
  @return Boolean representing successful sending of the Peer object.
  */
  public boolean sendMessageToGroup(String pg_name, String message) {
 
    String local_addr;
    InetAddress moderator_addr;
    Socket hummer_socket;
    Peer peer_obj = new Peer();
    //++Needed for authentication
    Response response = new Response();
    //++Needed for authentication
    HummerClientSocket dcs = null;

    try {
      
      moderator_addr = getModeratorAddress(pg_name);

      hummer_socket = new Socket(moderator_addr, Integer.parseInt(
                                 _constants.getProperty("port.PEER_PORT"))); 


      local_addr = InetAddress.getLocalHost().getHostAddress();
      
      //Fill out the Peer object
      peer_obj.action = "message";
      peer_obj.peer_group = pg_name;
      peer_obj.message.host_IP = local_addr;
      peer_obj.message.description = message;
      peer_obj.message.filename = pg_name;
      peer_obj.message.tool = "peers";
      
      dcs = new HummerClientSocket(_personal, hummer_socket, _key_store);
      //    dcs.validation("129.101.54.53"); this should be the same as below
      dcs.validation(null);

      //++Needed for authentication
      response._type = Response._EOT;
      //++Needed for authentication
      response._message = peer_obj;
      // End of protocol mesage, and close out connection
      //++This try and catch is needed for authentication

      dcs.writeResponseObject(response);
      dcs.close();
    }
    catch (IOException ioe) {
      System.err.println("PeerI:sendMessageToGroup:" + ioe);
      return false;
    }    
    catch (NumberFormatException nfe) {
      System.err.println("PeerI:sendMessageToGroup:" + nfe);
      return false;
    }
    catch (Exception e) {
      System.err.println("PeerI:sendMessageToGroup: " + e);
      return false;
    }
    return true;
  }   
}







