/*
Java Hummer Project
File: ToolI.java
Date: 4-9-99
Author: Chad Ward
Description:  Connects to a Hummer server and will start up,
              stop, and get basic information about the status
              of the Hummer servers (Config, Peer, Message)


 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.client;

import java.util.Enumeration;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Hashtable;

import java.net.Socket;
import java.net.UnknownHostException;
import java.net.InetAddress;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.StreamCorruptedException;
import java.io.OptionalDataException;

import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.SignatureException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.server.Hummer;
import edu.uidaho.hummer.dataobject.PersonalConfig;
import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.socket.HummerClientSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;

/**
Connects to a Tool server and will start up,
stop, and get basic information about the status
of the Tools on the system.  This is
done by logging into the Tool server.
<br><b>Note:</b></br>
For this method to work, the user account must have already been 
initialized(previously logged in), and the user account on the
target Hummer must be allowed to start and stop Hummer servers.
@author Chad Ward
*/
public final class ToolI {

	/**
	Number of choices in the menu.
	*/
  private final int _number_of_choices = 11;

//  private Properties _constants;

	/**
	Address of the Hummer that is being logged into.
	*/
  private InetAddress _hummer_address;

	/**
	PersonalConfig that will be used to login to the Hummer system.  The alias must also
	be the filename of the HummerKeyStore.
	*/
  private PersonalConfig _personal_config;

	/**
	HummerSocket used for communication with the server.  This needs to be a class
	field, because it is used in both startToolI and adminMenu
	*/
  private HummerClientSocket _hummer_client_socket = null;

	/**
	Key store of user.
	*/
  private HummerKeyStore _key_store;

	/**
	Socket with Tool server.
	*/
//  private Socket _socket;

/*
====================================================================================
Main method
*/

	/**
	Main is the method which starts the whole thing going.

	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li> Main creates a new Hummer
	  <li> Main starts the new Hummer
	</ul>
	@param argv[] The name/address of the Hummer to connect to.
	*/
  public static void main(String argv[]) {
    String usage = "Usage:\n\tjava edu.uidaho.hummer.client.ToolI <username> <name|address> \n" +
                   "\tPassword is only if the account has not been initialized yet.";
    if(argv.length < 2 || argv.length > 3) {
      System.err.println(usage);
      return;
    }
    System.out.println("\n\n======================================================================\n" +
                       "Welcome to the Hummer system - ToolI\n" +
                       " Copyright (C) 1997-1998 The Hummer " +
                       "Project - hummer@cs.uidaho.edu\n" +
                       "----------------------------------------------------------------------");
    byte[] password = null;
    if(argv.length == 3) {
      password = argv[2].getBytes();
    }
    PersonalConfig personal_config = new PersonalConfig(argv[0], password);
    InetAddress hummer_address = null;
    try {
      hummer_address = InetAddress.getByName(argv[1]);
    }
    catch(UnknownHostException e) {
      System.err.println("ToolI:main: Could not connect to the given host (" + argv[1] + ").");
      return;
    }

    ToolI hummer_interface = new ToolI(personal_config, hummer_address);
    hummer_interface.startToolI();
  }


/*
====================================================================================
constructor
*/
	/**
	Sets the _hummer_properties value.
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>get Properties object from StaticMethods.getHummerProperties()
	</ul>
      @see edu.uidaho.hummer.util.StaticMethods#getHummerProperties()
	*/
  public ToolI(PersonalConfig personal_config, InetAddress hummer_address) {
    this._personal_config = personal_config;
    this._hummer_address = hummer_address;
  }


/*
====================================================================================
methods
*/

	/**
	Attempts to connect to Hummer server before taking input on what
	the user would like to do with the server.
	*/
  public boolean startToolI() {
    try {
      _key_store = HummerKeyStore.getInstance();
      System.out.print("ToolI: Now loading key store...");
      try {
        _key_store.load(false, _personal_config.getAlias());
        System.out.println("loaded.");
      }
      catch(UnknownHostException e) {
        System.out.println("ToolI:main: Could not get HummerKeyStore filename: " + e);
        return false;
      }
      catch(FileNotFoundException e) {
        System.out.println("ToolI:main: Could not get HummerKeyStore filename: " + e);
        return false;
      }
      catch(IOException e) {
        System.out.println("ToolI:main: Could not get HummerKeyStore filename: " + e);
        return false;
      }
    }
    catch(KeyStoreException e) {
      System.out.println("ToolI:start: Could not load HummerKeyStore: " + e);
      return false;
    }
    catch(ClassNotFoundException e) {
      System.out.println("ToolI:start: Could not load HummerKeyStore: " + e);
      return false;
    }
    return handleRequests();
  }
  
  public boolean startToolI(HummerKeyStore key_store) {
    this._key_store = key_store;
    return handleRequests();
  }

  private boolean handleRequests() {
    Properties constants = null;
    Response input, output;
    try {
      constants = StaticMethods.getHummerProperties();
    }
    catch(FileNotFoundException e) {
      System.err.println("ToolI:handleRequests: Could not find Hummer.properties file: " + e);
    }
    catch(IOException e) {
      System.err.println("ToolI:handleRequests: " + e);
    }
		/*
		Have connected and validated with server, now find out what
		the user wants to do.
		*/
    try {
		/**
		The last value in the menu should always be the choice to
		end transmission.
		*/
      Integer port = null;
      port = (new Integer(constants.getProperty("port.TOOL_PORT")));
      System.out.print("ToolI: Connecting to " + _hummer_address.getHostName() + "...");
      Socket socket = new Socket(_hummer_address, port.intValue());
      System.out.println("connected.");
      _hummer_client_socket = new HummerClientSocket(_personal_config, socket, _key_store);
      System.out.print("ToolI: Validating...");
      _hummer_client_socket.validation(null);
      System.out.println("validated.");
      _key_store.store(false, _personal_config.getAlias());
      input = _hummer_client_socket.readResponseObject();
      String[] choice_list = (String[])input._message;
      output = this.getUserChoice(choice_list);
      while(((Integer)output._message).intValue() != 0) {
        _hummer_client_socket.writeResponseObject(output);
        input = _hummer_client_socket.readResponseObject();
        if(input._type == Response._EOT) {
          _hummer_client_socket.close();
        }
        System.out.println(toolInfoToString(input._data) + "\n");

        if(((Integer)output._message).intValue() != 0) {
      		/*
      		If the server has sent invalid information, then the system should
      		disconnect without taking information from the user again.
      		*/
          output = this.getUserChoice(choice_list);
        }
      }
      _hummer_client_socket.endOfTransmission(null);
      _hummer_client_socket.close();
    }
    catch(HummerSocketException e) {
      System.err.println("ToolI:start: Error while connecting to Hummer server:\n" + e);
      return false;
    }
    catch(IOException e) {
      System.err.println("ToolI:start: Error while connecting to Hummer server:\n" + e);
      System.err.println("\n*****************************************************" +
			 "\n The Tool Server is not running.  You must start it." +
			 "\n     Select \"Start Tool Server\" from the menu." +
			 "\n*****************************************************");
      return false;
    }
    catch(UnknownAliasException e) {
      System.err.println("ToolI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("ToolI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(KeyStoreException e) {
      System.out.println("ToolI:start: Could not store HummerKeyStore: " + e);
      return false;
    }
    catch(CertificateException e) {
      System.err.println("ToolI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(NoSuchAlgorithmException e) {
      System.err.println("ToolI:start: Error while creating HummerSocket: " + e);
      return false;
    }
    catch(UnrecoverableKeyException e) {
      System.err.println("ToolI:start: Error while validating to Hummer server:\n" + e);
      return false;
    }
    System.out.println("\nExiting ToolI.\n");
    return true;
  }

  private String toolInfoToString(Vector data) {
    String return_value = "";
    final String SPACE = "                  ";
    final int MAX_LENGTH = 15;
    String[] header;
    Object[] current_info;
    Enumeration enum = data.elements();
    if(enum.hasMoreElements()) {
      header = (String[])enum.nextElement();
      return_value = header[0];
      for(int i = 1; i < header.length; i++) {
        return_value += SPACE.substring(0, MAX_LENGTH - header[i - 1].length()) + header[i];
      }
      return_value += "\n----------------------------------------------------\n";
      int previous_length;
      while(enum.hasMoreElements()) {
        current_info = (Object[])enum.nextElement();
        return_value += current_info[0];
        for(int i = 1; i < current_info.length; i++) {
          if(current_info[i] == null) {
            current_info[i] = "null";
          }
          previous_length = current_info[i - 1].toString().length();
          return_value += SPACE.substring(0, MAX_LENGTH - previous_length) +
                          current_info[i];
        }
        return_value += "\n";
      }
    }
    return return_value;
  }

	/**
	Reads in the user's choice from STDIN of what 
	option they would like Hummer to execute.

	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>Read in option from STDIN.  All options, except 0 (End transmission) are
	  	sent by the server.
	</ul>
	@return The integer value representing the selection.
	*/
  private Response getUserChoice(String[] choice_list) throws IOException {
    int choice = -1;
    StringTokenizer tokenizer;
    Response output = null;
    boolean first_time = true;
    BufferedReader in_stream = new BufferedReader(new InputStreamReader(System.in));

    while(choice < 0 || choice > choice_list.length) {
      if(first_time) {
        System.out.println("\n======================================================\n" +
                           "ToolI - Hummer Tool Interface\n" +
                           "Connected to " + _hummer_address + ".\n" +
                           "Please make a choice.\n" +
                           "------------------------------------------------------\n");
        first_time = false;
      }
      else {
        System.out.println("\n======================================================\n" +
                           "ToolI - Hummer Tool Interface\n" +
                           "Connected to " + _hummer_address + ".\n" +
                           "\nThat is not a valid choice:\n" +
                           "Please make a choice.\n" +
                           "------------------------------------------------------");
      }
      output = new Response();
      System.out.println("0 - End Transmission");
      for(int i = 0; i < choice_list.length; i++) {
        System.out.println((i + 1) + " - " + choice_list[i]);
      }
      System.out.print("---------------------------------------\nChoice - ");
      tokenizer = new StringTokenizer(in_stream.readLine());
      try {
        choice = (new Integer(tokenizer.nextToken())).intValue();
      }
      catch (NumberFormatException e) {}
      output._message = new Integer(choice);
      while(tokenizer.hasMoreTokens()) {
        output._data.add(tokenizer.nextToken());
      }
    }
    System.out.println("---------------------------------------");
    return output;
  }
}
