/*
Java Hummer Project
File: NewUserValidator.java
Date: 2-14-99
Author: Chad Ward
Description:  Object for tracking a user's authentication and access
              rights to the system.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


package edu.uidaho.hummer.dataobject;

import java.util.Arrays;
import java.io.IOException;
import java.io.Serializable;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import edu.uidaho.hummer.util.StaticMethods;

/**
Class which holds a password, can validate the password when requested.
<h4>Class Usage and Flow:</h4>
<ul>
  <li>Constructor creates a new _password array, and uses System.arraycopy to
      copy the contents of password parameter into _password
  <li>clear method will reset _password in memory to insure that it cannot be read
      after the reference has been reset.
  <li>isValidPassword method validates a given password.
</ul>
<h4>Usage Example</h4>
Create an instance of a NewUserValidator with password equal to "password".
	<blockquote><b>
	NewUserValidator validator = new NewUserValidator("password".getBytes());
	</b></blockquote>

Get the bytes from a combination of a timestamp and a random number.
	<blockquote><b>
	byte[] random_timestamp = StaticMethods.getRandomTimestampBytes();
	</b></blockquote>

A PublicKey will probably already exist, but for this example, one will be generated.
	<blockquote><b>
	PublicKey public_key = StaticMethods.generateKeyPair().getPublic();
	</b></blockquote>

Get the MessageDigest bytes generated from the password, "random timestamp" combination,
and the encoded version of the PublicKey.  This method is equivalent to StaticMethods.getDigest.
	<blockquote><b>
	byte[] message_digest = validator.getDigest();
	</b></blockquote>

If needed, the password can then be erased from memory.
	<blockquote><b>
	validator.clear();
	</b></blockquote>
@see edu.uidaho.hummer.util.StaticMethods#getRandomTimestampBytes()
@see edu.uidaho.hummer.util.StaticMethods#generateKeyPair()
@see edu.uidaho.hummer.util.StaticMethods#getDigest(byte[], byte[],java.security.PublicKey)
@author Chad Ward
*/
public class NewUserValidator implements Serializable {
	/**
	Byte representation of password.
	@serial
	*/
  protected byte[] _password = null;

	/**
	Initializes password
	@param password Array of bytes representing the user's password
	*/
  public NewUserValidator(byte[] password) {
    if(password == null) {
      password = new byte[0];
    }
    this._password = new byte[password.length];
    System.arraycopy(password, 0, this._password, 0, password.length);
  }

	/**
	Will fill password will ' ' (SPACE) characters and then set _password reference to null.
	This should be used whenever the validator is no longer needed, so that the password
	does not stay in memory.
	*/
  public void clear() {
    Arrays.fill(_password, (byte)' ');
    _password = null;
  }
	/**
	Authenticates the given password by encoding the passphrase text and comparing
	the value with the <i>_password_authentication</i> array
	@param password Password of account
	@return True if the password for this account is valid, and false if it is invalid
	*/
  public boolean isValidPassword(byte[] check_digest_bytes, byte[] random_timestamp, PublicKey public_key) 
         throws NoSuchAlgorithmException, IOException {
    if(public_key == null) {
      throw new NullPointerException("NewUserValidator:isValidPassword:  public_key must be a non-null value");
    }
    byte[] digest = StaticMethods.getDigest(_password, random_timestamp, public_key);
    return MessageDigest.isEqual(digest, check_digest_bytes);
  }

  public byte[] getDigest(byte[] random_timestamp, PublicKey public_key)
         throws IOException, NoSuchAlgorithmException {
    return StaticMethods.getDigest(_password, random_timestamp, public_key);
  }
}