/*
Java Hummer Project
File: ClientHandler.java
Date: 11-11-98
Author: Joel Marlow & Anu Kamath
Description: This handles the connections to the Configuration Server.


 Copyright (C) 1997-1998 The Hummer Project - hummer@csds.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.uidaho.hummer.dataobject.Config;
import edu.uidaho.hummer.client.PeerI;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.*;

/**
The Client Handler class is the server behind ConfigClient.  It handles
requests made by ConfigClient to connect to the database and either
insert or delete records.

<h4>Class Usage and Flow:</h4>
<ul>
<li>Constructor creates a connection to the database using JDBC and the
java.System.Properties file "_constants".  It also grabs the socket the
client is going to be talking on.
<li>The run() method is launched because it is a Thread, and tells the
object to handle the input stream on the socket "_socket".
<li>The processRequest(Socket) method reads a database transaction 
request from the socket, and processes it if it is either an INSERT
or DELETE.
<li>Thread unbinds from sockets and stops naturally.
</ul>

@author Joel Marlow & Anu Kamath , javadoc of method usage and flow done by Zac Woodall
*/

class ClientHandler extends Thread {
  /**
  Config is the main object that is being passed between any configuration 
  client and the configuration server.
   */
  private Config _config = new Config();
  /**
  The server socket for the ConfigServer.  This may not need to be a class
  variable.
  */
  private Socket _socket;
  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _url = new String();
  /**
  Database Driver which is taken from the corresponding _constants property.
  */
  private String _driver = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  private String _password = new String();
  /**
  Boolean value used in order to tell if we were ever connected to the 
  database. 
  */
  private boolean _is_connected = false;
  /**
  System Properties object that when created contains the System.getProperties
  and gets the properties defined in Hummer.properties.
  */
  private Properties _constants;
  //++Needed for authentication
  HummerServerSocket _hummer_socket;
  //++Needed for authentication
  HummerKeyStore _key_store;


  /**
  Basic constructor instanciates a ClientHandler on a socket with 
  some properties (via a Properties file).

  
  <h4>Constructor Usage and Flow:</h4>
  <ul>
  <li> Set up the DB from the properties file:
  <ul>
  <li> Get the DB URL
  <li> Get the DB driver
  <li> Get the DB user
  <li> Get the DB password
  </ul>
  <li> Open a connection to the DB
  </ul>
  @param sock this is the socket which the server will run on.
  @param constants this is the Properties file containing configuration info.
  */

  public ClientHandler (Socket sock, Properties constants) throws IOException {
    this._socket = sock;
    _constants = constants;
    String address = new String();

  //++This try and catch is needed for authentication
    try {
    address = InetAddress.getLocalHost().getHostAddress();
    _key_store = HummerKeyStore.getInstance();
    _key_store.load(true, address);
    }
    catch (Exception e) {
      System.err.println("ClientHandler: " + e);
      e.printStackTrace();
    }

    //Initialize the database strings from the properties object.
    try {
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
        _url = _constants.getProperty("jdbc.url." + address) + nametoken.nextToken();
      }
      else {
        _url = _constants.getProperty("jdbc.url") + nametoken.nextToken();
      }

      if (_constants.containsKey("jdbc.driver." + address)) {
        _driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
        _driver = _constants.getProperty("jdbc.driver");
      }
      if (_constants.containsKey("jdbc.user." + address)) {
        _user = _constants.getProperty("jdbc.user." + address);
      }
      else {
        _user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
        _password = _constants.getProperty("jdbc.password." + address);
      }
      else {
        _password = _constants.getProperty("jdbc.password");
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("ClientHandler:" + uhe);
    }

    // Connect to DB server

    try {
      Class.forName (_driver);
      //      System.out.println ("connected");
      _is_connected = true;
    } 
    catch (ClassNotFoundException C) { 
      System.err.println ("ClientHandler:" + C);
    }
  }



  /**
  This method processes a reqest from ConfigClient with reguard 
  to it's DB.
  As of right now there is no authentication, so this is a potential 
  security flaw in Hummer.

  <H4> Method Usage and Flow </H4>
  <ul>
  <li> Create an input stream on <i>socket</i>
  <li> Use StringTokenizer to reconstruct a series of DB queries 
       sent out over that socket by ConfigClient.
  <li> Make the query asked for where the query is one of the following types:
  <ul>
  <li> accept_list
  <li> config_time
  <li> filter
  <li> forward_list
  <li> forward_peer
  <li> kill_file
  <li> levels
  <li> manager
  <li> peers
  <li> pgroups
  <li> subordinates
  </ul>
  <li> Write the current DB configuration back out over
       the socket.
  </ul>

  @param socket This is the socket that is used to connect to the client
  @return boolean value that says whether the request was 
  processed correctly.
  */

  public boolean processRequest(Socket socket) {
    ObjectInputStream in;
    ObjectOutputStream out;
    Vector temp_vector = new Vector();
    Vector store_config = new Vector();
    Enumeration main_enum; 
    Enumeration inside_enum;
    String query = new String ();
    String table = new String();
    StringTokenizer query_info;
    String token = new String();
    String address = new String();
    //++Needed for authentication
    Response response;
    boolean not_done = true;

//    System.out.println("right before try");
    try {
      while (not_done) {
        //++Needed for authentication
        //      System.out.println("has validated:" + _hummer_socket.hasValidated());
        response = _hummer_socket.readResponseObject();
        address = InetAddress.getLocalHost().getHostAddress();
        //writing out the keystore so there is no trouble
        _key_store.store(true, address);
        //System.out.println("after reading");
        //++Needed for authentication
        if (response._type == Response._EOT) {
              not_done = false;
        }

        else if (_is_connected) {    
          //++Needed for authentication
          _config = (Config)response._message;
          // Do until the end of the protocol indicator
          store_config = _config.data;
          _config.data = new Vector();
          main_enum = store_config.elements();
          while (main_enum.hasMoreElements()) {
            //System.out.println("i am running through the while loope");
            query = (String)main_enum.nextElement();
	  //          System.out.println(query);
            query_info = new StringTokenizer(query," ");
            token = query_info.nextToken();
            if (token.equals("close")) {
              not_done = false;
            }
            else if (token.equals("accept_list")) {
              acceptListQuery(query_info);
            }
            else if(token.equals("config_time")) {
              configTimeQuery(query_info);
            } 
            else if(token.equals("filter")) {
              filterQuery(query_info);
            } 
            else if(token.equals("forward_list")) {
              forwardListQuery(query_info);
            } 
            else if(token.equals("forward_peer")) {
              forwardPeerQuery(query_info);
            } 
            else if(token.equals("kill_file")) {
              killFileQuery(query_info);
            } 
            else if(token.equals("levels")) {
              levelsQuery(query_info);
            } 
            else if(token.equals("manager")) {
              managerQuery(query_info);
            } 
            else if(token.equals("peers")) {
              peersQuery(query_info);
            } 
            else if(token.equals("pgroups")) {
              pgroupsQuery(query_info);
            } 
            else if(token.equals("subordinates")) {
              subordinatesQuery(query_info);
            } 
            //the next ones are action commands
            else if(token.equals("subscribe")) {
              _config.data.addElement(String.valueOf(usingPeerI(token, query_info)));
            }
            else if(token.equals("unsubscribe")) {
              _config.data.addElement(String.valueOf(usingPeerI(token, query_info)));
            }
            else if(token.equals("message")) {
            }
            else if(token.equals("newmanager")) {
              addManager(query_info);
            }
            else if(token.equals("newsubordinate")) {
              addSubordinate(query_info);
            }
            else if(token.equals("writeoutkey")) {
              address = InetAddress.getLocalHost().getHostAddress();
              //writing out the keystore so there is no trouble
              _key_store.store(true, address);
            }
            else if(token.equals("writekey")) {
              address = InetAddress.getLocalHost().getHostAddress();
              //writing out the keystore so there is no trouble
              _key_store.store(true, address);
              not_done = false;
            }
            // if the string was anything else quit.
            else { 
              break;
            }
          } // while

          //      System.out.println("after if and while");
          if (not_done) {
            //++Needed for authentication
            response = new Response();
            //++Needed for authentication
            response._type= Response._OK;
            //++Needed for authentication
            response._message = _config;
            //++Needed for authentication
            //      System.out.println("has validated:" + _hummer_socket.hasValidated());
            _hummer_socket.writeResponseObject(response);
          }
        } // else if
      }
      //++Needed for authentication
      _hummer_socket.close();
      System.out.println("The Socket is now closed");
    } // try
    catch (IOException I) { 
      System.err.println ("ClientHandler:processRequest: " + I); 
      return false;
    }   
    //++Needed for authentication this one should look
    //++for all of the exceptions instead of the general one
    catch (Exception e) {
      //      System.out.println("second");
      System.err.println("ClientHandler:processRequest:" + e);
      e.printStackTrace();
    }
    return true;
  
  } //processRequest

  /**
  This method processes any query made against the accept_list
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean acceptListQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String filter = new String();
    String start_addr = new String();
    String end_addr = new String();
    String inh = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 2) {
        filter = query_info.nextToken();
        inh = query_info.nextToken();
        query = token + " from accept_list where filter=" + filter + " and inh=" + inh + ";";
      }
      else if (query_info.countTokens() == 1) {
        inh = query_info.nextToken();
        query = token + " from accept_list where inh=" + inh + ";";
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from accept_list;";
      }
      else 
        return false;
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 4) {
        filter = query_info.nextToken();
        start_addr = query_info.nextToken();
        end_addr = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into accept_list values (" + filter + ", " + start_addr + ", " + end_addr + ", " + inh + ");";  
        dbAccess(query, token);
      }
      else
        return false;
    }
    else {
      return false;
    }
    return true;
  }// acceptListQuery

  /**
  This method processes any query made against the config_time
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean configTimeQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      query = token + " from config_time;";
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 1) {
        String ctime = query_info.nextToken();
        query = token + " into config_time values (" + ctime + ");";  
        dbAccess(query, token);
      }
      else
        return false;
    }
    else {
      return false;
    }
    return true;
  }//configTimeQuery

  /**
  This method processes any query made against the filters
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean filterQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String expression = new String();
    String inh = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 2) {
        expression = query_info.nextToken();
        inh = query_info.nextToken();
        query = token + " from filter where expression=" + expression + " and inh=" + inh + ";";
      }
      else if (query_info.countTokens() == 1) {
        inh = query_info.nextToken();
        query = token + " from filter where inh=" + inh + ";";
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from filter;";
      }
      else 
        return false;
      //      System.out.println("sending to db query: " + query);
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 11) {
        expression = query_info.nextToken();
        String date_start = query_info.nextToken();
        String date_end = query_info.nextToken();
        String time_start = query_info.nextToken();
        String time_end = query_info.nextToken();
        String trust = query_info.nextToken();
        String coop = query_info.nextToken();
        String integ = query_info.nextToken();
        String log = query_info.nextToken();
        String alert = query_info.nextToken();
        inh = query_info.nextToken();
        query = token + " into filter values (" + expression + " , " + date_start + " , "
                                               + date_end + " , " + time_start + " , " 
                                               + time_end + " , " + trust + " , " 
                                               + coop + " , "  + integ + " , " 
                                               + log + " , " + alert + " , " 
                                               + inh +" );";  
        dbAccess(query, token);
      }
      else 
        return false;
    }
    else {
      return false;
    }
    return true;
  }//filterQuery

  /**
  This method processes any query made against the forward_list
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean forwardListQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String filter = new String();
    String inh = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 2) {
        filter = query_info.nextToken();
        inh = query_info.nextToken();
        query = token + " from forward_list where filter=" + filter + " and inh=" + inh + ";";
      }
      else if ((query_info.countTokens() == 1) && (token.equals("delete"))) {
        inh = query_info.nextToken();
        query = token + " from forward_list where inh=" + inh + ";";
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from forward_list;";
      }
      else 
        return false;
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 3) {
        filter = query_info.nextToken();
        String host = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into forward_list values (" + filter + ", " + host + ", " + inh + ");";  
        dbAccess(query, token);
      }
      else 
        return false;
    }
    else {
      return false;
    }
    return true;
  }//forwardListQuery

  /**
  This method processes any query made against the forward_peer
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean forwardPeerQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String filter = new String();
    String inh = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 2) {
        filter = query_info.nextToken();
        inh = query_info.nextToken();
        query = token + " from forward_peer where filter=" + filter + " and inh=" + inh + ";";
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from forward_peer;";
      }
      else 
        return false;
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 3) {
        filter = query_info.nextToken();
        String pgroup = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into forward_peer values (" + filter + ", " + pgroup + ", " + inh + ");";  
        dbAccess(query, token);
      }
      else 
        return false;
    }
    else {
      return false;
    }
    return true;
  }//forwardPeerQuery

  /**
  This method processes any query made against the kill_file
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean killFileQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String start_addr = new String();
    String end_addr = new String();
    String inh = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 1) {
        inh = query_info.nextToken();
        query = token + " from kill_file where inh=" + inh + ";";
      }
      else if ((query_info.countTokens() == 3) && (token.equals("delete"))) {
        start_addr = query_info.nextToken();
        end_addr = query_info.nextToken();
        inh = query_info.nextToken();
        if (end_addr.equals("'NULL'")){
          query = token + " from kill_file where start_addr =" + start_addr + " and end_addr is null and inh=" + inh + ";";
        }
        // These two lines have not been tested
        else{
          query = token + " from kill_file where start_addr =" + start_addr + " and end_addr =" + end_addr + " and inh=" + inh + ";";
        }
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from kill_file;";
      }
      else 
        return false;
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 3) {
        start_addr = query_info.nextToken();
        end_addr = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into kill_file values (" + start_addr + ", " + end_addr + ", " + inh + ");";  
        dbAccess(query, token);
      }
      else if (query_info.countTokens() == 2) {
        start_addr = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into kill_file values (" + start_addr + ", null, " + inh + ");";  
        dbAccess(query, token);
      }
      else 
        return false;
    }
    else {
      return false;
    }
    return true;
  }//killFileQuery

  /**
  This method processes any query made against the levels
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean levelsQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String start_addr = new String();
    String end_addr = new String();
    String inh = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 1) {
        inh = query_info.nextToken();
        query = token + " from levels where inh=" + inh + ";";
      }
      else if ((query_info.countTokens() == 2) && (token.equals("select *"))) {
        start_addr = query_info.nextToken();
        inh = query_info.nextToken();
        query = token + " from levels where start_addr =" + start_addr + " and inh=" + inh + ";";
      }
      else if ((query_info.countTokens() == 3) && (token.equals("delete"))) {
        start_addr = query_info.nextToken();
        end_addr = query_info.nextToken();
        inh = query_info.nextToken();
        if (end_addr.equals("'NULL'")) {
          query = token + " from levels where start_addr =" + start_addr + " and end_addr is null and inh=" + inh + ";";
        }
        // These two lines have not been tested
        else {
          query = token + " from levels where start_addr =" + start_addr + " and end_addr =" + end_addr + " and inh=" + inh + ";";
        }
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from levels;";
      }
      else 
        return false;
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 6) {
        start_addr = query_info.nextToken();
        end_addr = query_info.nextToken();
        String trust = query_info.nextToken();
        String coop = query_info.nextToken();
        String integ = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into levels values (" + start_addr + ", " + end_addr + ", " + trust + ", " + coop + ", " + integ + ", " + inh + ");";  
        dbAccess(query, token);
      }
      else if (query_info.countTokens() == 5) {
        start_addr = query_info.nextToken();
        String trust = query_info.nextToken();
        String coop = query_info.nextToken();
        String integ = query_info.nextToken();
        inh= query_info.nextToken();
        query = token + " into levels values (" + start_addr + ", null, " + trust + ", " + coop + ", " + integ + ", " + inh + ");";  
        dbAccess(query, token);
      }
      else 
        return false;
    }
    else {
      return false;
    }
    return true;
  }//levelsQuery

  /**
  This method processes any query made against the manager
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean managerQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      query = token + " from manager;";
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 1) {
        String manager = query_info.nextToken();
        query = token + " into manager values (" + manager + ");";  
        dbAccess(query, token);
      }
      else
        return false;
    }
    else {
      return false;
    }
    return true;
  }//managerQuery

  /**
  This method processes any query made against the peers
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean peersQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String next_token = new String();
    String pgroup = new String();
    String mem_addr = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 2) {
        next_token = query_info.nextToken();
        if (next_token.equals("pgroup")) {
          pgroup = query_info.nextToken();
          query = token + " from peers where pgroup =" + pgroup +" ;";
        }
        else if (next_token.equals("mem_addr")) {
          mem_addr = query_info.nextToken();
          query = token + " from peers where mem_addr =" + mem_addr +" ;";
        }
        else {
          pgroup = next_token;
          mem_addr = query_info.nextToken();
          query = token + " from peers where pgroup =" + pgroup +" and  mem_addr =" + mem_addr + ";";
        }
      }
      else if (query_info.countTokens() == 0){
        query = token + " from peers;";
//System.out.println("in peersQuery: "+ query);
      }
      else 
        return false;
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 4) {
        pgroup = query_info.nextToken();
        mem_addr = query_info.nextToken();
        String peer_sub = query_info.nextToken();
        String peer_mod = query_info.nextToken();
        query = token + " into peers values (" + pgroup +", " + mem_addr +", "+ peer_sub + ", " + peer_mod + ");";  
        dbAccess(query, token);
      }
      else
        return false;
    }
    else {
      return false;
    }
    return true;
  }//peersQuery

  /**
  This method processes any query made against the pgroups
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean pgroupsQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String mod_addr = new String();
    String pgroup = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if (query_info.countTokens() == 2) {
        pgroup = query_info.nextToken();
        mod_addr = query_info.nextToken();
        query = token + " from pgroups where pgroup=" + pgroup + " and mod_addr=" + mod_addr + ";";
      }
      else if (query_info.countTokens() == 1) {
        mod_addr = query_info.nextToken();
        query = token + " from pgroups where mod_addr=" + mod_addr + ";";
      }
      else if (query_info.countTokens() == 0){
        query = token + " from pgroups;";
      }
      else {
        return false;
      }
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 3) {
        pgroup = query_info.nextToken();
        mod_addr = query_info.nextToken();
        String email = query_info.nextToken();
        query = token + " into pgroups values (" + pgroup + ", " + mod_addr + ", " + email + ");";  
        dbAccess(query, token);
      }
      else
        return false;
    }
    else {
      return false;
    }
    return true;
  }//pgroupsQuery

  /**
  This method processes any query made against the subordinates
  table.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Parse the query to create a string version
  <li> Submit the query to the DB.
  </ul>

  @param query_info this is the query in tokenizer form sent against the DB.
  */
  public boolean subordinatesQuery(StringTokenizer query_info) {
    String token = new String();
    String query = new String();
    String subord = new String();

    token = query_info.nextToken();
    if ((token.equals("select")) || (token.equals("delete"))) {
      if (token.equals("select")){
        token = token + " *";
      }
      if ((query_info.countTokens() == 1) && (token.equals("delete"))) {
        subord = query_info.nextToken();
        query = token + " from subordinates where subord=" + subord + ";";
      }
      else if (query_info.countTokens() == 0) {
        query = token + " from subordinates;";
      }
      else {
        return false;
      }
      dbAccess(query, token);
    }
    else if (token.equals("insert")) {
      if (query_info.countTokens() == 2) {
        String hum_name = query_info.nextToken();
        subord = query_info.nextToken();
        query = token + " into subordinates values (" + hum_name + ", " + subord + ");";  
        dbAccess(query, token);
      }
      else
        return false;
    }
    else {
      return false;
    }
    return true;
  }//subordinatesQuery

  /**
  This method tries to connect to the DB with the given setup values.

  <h4> Method Usage and Flow:</h4>
  <ul>
  <li> Connect to the database.
  <li> Create a new statement.
  <li> If the query is an insert or delete:
  <ul>
  <li> execute the query.
  <li> print out the query on STDOUT.
  </ul>
  <li> Otherwise:
  <ul>
  <li> execute the query and put the result into a result set.
  <li> add the result set to the configuration (_config.data).
  <li> close the result set.
  <li> print out the query on STDOUT.
  </ul>
  </ul>

  @param query Query to be executed against the DB.
  @param token String containing type of query (insert or delete).
  */
  public void dbAccess(String query, String token) {
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;
    ResultSetMetaData table_data = null;
    int number_column = 0;
    Vector table = new Vector();
    Vector row = new Vector();

    try {
      db_connect = DriverManager.getConnection(_url, _user, _password);
      statement = db_connect.createStatement();
      if ((token.equals("delete")) ||  (token.equals("insert"))){
	//System.out.println(query);
        statement.executeUpdate(query);
      }
      else {
	//System.out.println(query);
        result_set = statement.executeQuery(query);
        table_data = result_set.getMetaData();
        number_column = table_data.getColumnCount();
        while (result_set.next()) {
          row = new Vector();
          for (int i = 1; i <= number_column; i++){
            row.addElement(result_set.getObject(i));
          }
          table.addElement(row);
        }
        _config.data.addElement(table);
        result_set.close();
      }
      statement.close();
      db_connect.close();
    }
    catch (SQLException S) { 
      System.err.println ("ClientHandler:processRequest: " + S);
    }
  } // dbAccess

  public boolean usingPeerI(String action, StringTokenizer peer_info) {
    PeerI peer = new PeerI();

    if ((peer_info.countTokens() == 2) && (action.equals("subscribe"))){
      String args[] = {"s", peer_info.nextToken(), peer_info.nextToken()};
      return peer.processParameters(args);
    }
    else if ((peer_info.countTokens() == 1) && (action.equals("unsubscribe"))){
      String args[] = {"u", peer_info.nextToken()};
      return peer.processParameters(args);
    }
    else {
      return false;
    }
  }

  /**
   Receiving  
   */
  public boolean addManager(StringTokenizer manager_info) {
    int port;
    Socket manager_socket; 
    PersonalConfig personal = null;
    Response response = new Response();
    HummerClientSocket dcs = null;
    Config config = new Config();
    String manager_name = manager_info.nextToken();
    String password = manager_info.nextToken();

    try{
      Integer value = new Integer(_constants.getProperty("port.CONFIG_PORT"));
      port = value.intValue();
      manager_socket = new Socket(manager_name, port);
      String address = InetAddress.getLocalHost().getHostAddress();
      personal = new PersonalConfig(address, password.getBytes());
      System.out.println("\n\n\ncreating client socket");
      dcs = new HummerClientSocket(personal, manager_socket, _key_store);
      dcs.validation(null);
      System.out.println("\n\n\nAdding Manager(have validated)");
      dcs.addManager();
      System.out.println("\n\n\nHave added Manager");
      config.data.addElement("writekey");
      //++Needed for authentication
      response._type = Response._OK;
      //++Needed for authentication
      response._message = config;
      dcs.writeResponseObject(response);
      //writing out the keystore so there is no trouble
      _key_store.store(true, address);
      System.out.println("Keystore is now written out");
      dcs.close();
    } 
    catch(UnknownHostException e) {
      System.err.println("ClientHandler:addManager:  " + e.getMessage());
    }
    catch(IOException e) {
      System.err.println("ClientHandler:addManager: " + e.getMessage());
    }
    catch (NumberFormatException ne) {
      System.err.println("ClientHandler:addManager: Bad port.CONFIG_PORT property.");
    }
    catch (Exception e) {
      System.err.println("ClientHandler:addManager: " + e);
      e.printStackTrace();
    }
    return true;
  }

  /**
  After reciving the newsubordinate command, this method is then looking for the 
  IP of the subordinate, the password it will be using  
   */
  public boolean addSubordinate(StringTokenizer sub_info) {
    String username = sub_info.nextToken();
    String password = sub_info.nextToken();

    try{
      String address = InetAddress.getLocalHost().getHostAddress();
      _key_store.addServerAlias(username, password.getBytes() , false, StaticMethods.getDefaultServerAccessTable());
      //writing out the keystore so there is no trouble
      _key_store.store(true, address);
    }
    catch (IOException ioe) {
      System.err.println ("ClientHandler:addSubordinate: " + ioe);
    }
    catch(Exception e) {
      System.err.println ("ClientHandler:addSubordinate: " + e);
      e.printStackTrace();
    }
    return true;
  }

  /**
     The executable code.
  */

  public void run() {
	

    try {
      String address = InetAddress.getLocalHost().getHostAddress();
      //++Needed for authentication
      _hummer_socket = new HummerServerSocket(null, _socket, _key_store);
      //++Needed for authentication
      _hummer_socket.validation(null); 
      //writing out the keystore so there is no trouble
      _key_store.store(true, address);

      // Process request in method
      processRequest(_socket);
    }
    catch (IOException ioe) {
      System.err.println ("ClientHandler:run:" + ioe);
      ioe.printStackTrace();
    }
    //++Needed for authentication this one should look
    //++for all of the exceptions instead of the general one
    catch(Exception e) {
      System.err.println ("ClientHandler:run:" + e);
      e.printStackTrace();
    }

  } //run 

} //ClientHandler 
