/*
Java Hummer Project
File: Hummer.java
Date: 12-23-98
Author: Jesse C. McConnell, Chad Ward
Description:  This is the class that will run MessagingServer, PeerServer, 
and ConfigServer.  Through this class you will be able to start and stop 
each of the Hummer Services.


 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.net.Socket;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.UnknownHostException;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.StreamCorruptedException;
import java.io.OptionalDataException;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;

import java.sql.ResultSet;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.Connection;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;

import java.security.KeyStoreException;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.server.*;
import edu.uidaho.hummer.gui.hac.Interface;
import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.dataobject.AliasInfo;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;

import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;

/**
This is the class that ties all of Hummer together.

<h4>Class Usage and Flow:</h4>
<ul>
<li>The constructor tries to read the Properties file "Hummer.properties"
from the current working dir.
<li>The method startHummer() is called from the void main right
after creation of a new Hummer. startHummer loops while Hummer is
running:
<ul>
<li>Display to the user the options Hummer supports using 
displayInstructions().
<li>Read the user's choice and start or stop the according portions
(Servers/Services) of Hummer or quit Hummer. 
</ul>
<li> Print out: "Thanks for supporting your local Java addicts." and exit.
</ul>

@author Jesse McConnell, Chad Ward
 */
public class Hummer implements Runnable {

	/** 
	MessagingServer thread that Hummer uses for starting and stopping the 
	service.
	*/
  private static MessagingServer _ms;

	/**
	ConfigServer thread that Hummer used for starting and stopping the
	service.
	*/
  private static ConfigServer _cs;

	/**
	PeerServer thread that Hummer used for starting and stopping the service.
	*/
  private static PeerServer _ps;

	/**
	ToolServer thread that Hummer used for starting and stopping the service.
	*/
  private static ToolServer _ts;

	/**
	System Properties object that when created contains the System.getProperties
	and gets the properties defined in Hummer.properties.
	*/
  private Properties _constants = new Properties(System.getProperties());

	/**
	Key store used to validate connections from clients.
	*/
  private static HummerKeyStore _key_store;

  private static Socket _client_socket;

	/**
	Options available for starting Hummer servers and operation.
	@see edu.uidaho.hummer.client.HummerI#getUserChoice(String[])
	*/
  private final String[] _choice_list = {
                                       "Start Everything",			// 1
                                       "Shutdown Everything",			// 2
                                       "Start Messaging Server",		// 3
                                       "Stop MessagingServer",		// 4
                                       "Start Configuration Server",	// 5
                                       "Stop Configuration Server",	// 6
                                       "Start Peer Server",			// 7
                                       "Stop Peer Server",			// 8
                                       "Start Tool Server",			// 9
                                       "Stop Tool Server",			// 10
                                       "Reload MessageHandler",		// 11
                                       "Current Status",				// 12
                                     };

	/**
	Menu choice for commands
	*/
  public final static int _START_EVERYTHING = 1,
                          _SHUTDOWN_EVERYTHING = 2,
                          _START_MS = 3,
                          _STOP_MS = 4,
                          _START_CS = 5,
                          _STOP_CS = 6,
                          _START_PS = 7,
                          _STOP_PS = 8,
                          _START_TS = 9,
                          _STOP_TS = 10,
                          _RELOAD_MESSAGE_HANDLER = 11,
                          _DISPLAY_STATUS = 12;

  public static void main(String[] argv) {
    String usage = "\n***************************************\n" +
                    "Invalid usage.  Usage:\n" +
                   "\tjava edu.uidaho.hummer.server.Hummer <start <all|ms|cs|ps|ts>>\n\n";
    Vector commands = new Vector();
    if(argv.length > 0) {
      if(!argv[0].equals("start")) {
        System.err.println(usage);
        return;
      }
      if(argv.length < 2) {
        System.err.println(usage);
        return;
      }
      for(int i = 1; i < argv.length; i++) {
        if(!argv[i].equals("all") && !argv[i].equals("ms") &&
           !argv[i].equals("cs") && !argv[i].equals("ps") && !argv[i].equals("ts")) {
          System.err.println(usage);
          return;
        }
        commands.add(argv[i]);
      }
    }
    Hummer hummer = new Hummer();
    hummer.startHummer(commands);
  }

	/**
	The consructor for Hummer.  It essentially sets up all of the constants for
	Hummer by reading in the "Hummer.properties" file from the local
	directory.
	*/
  public Hummer() {
    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch(UnknownHostException e) {
      System.err.println("HummerI:constructor: Error while getting local host address: " + e);
    }
    catch(FileNotFoundException e) {
      System.err.println("HummerI:constructor: Could not find Hummer.properties file: " + e);
    }
    catch(IOException e) {
      System.err.println("HummerI:constructor: " + e);
    }
  }


	/**
	This is where Hummer spends most of its time. 

	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>While loop governed by boolean value going.
	  <ul>
	    <li>Tell the user what they can do: displayInstructions()
	    <li>Find out what the user wants to do: readInstructions()
	    <li>Do what the user wants (could be one of the following):
	    <ul>
	      <li>Start Everything
	      <ul>
              <li>Create new MessagingServer, ConfigServer, PeerServer
	        <li>Start MessagingServer
	        <li>Start ConfigServer
	        <li>Start PeerServer
	      </ul>
	      <li>Start MessagingServer: create a new MessagingServer, and start it.
	      <li>Stop MessagingServer: killMessagingServer()
	      <li>Start ConfigServer: create a new ConfigServer, and start it.
	      <li>Stop ConfigServer: killConfigServer
	      <li>Start PeerServer: create a new PeerServer, and start it.
	      <li>Stop PeerServer: killPeerServer()
	      <li>Current Status: displayStatus()
	      <li>Shutdown Everything: clean up and exit.
	    </ul>
	  </ul>
	</ul>  
	*/
  public void startHummer(Vector commands) {
    String local_host_name = null;
    try {
    	 local_host_name = InetAddress.getLocalHost().getHostName() + ".";
    	 local_host_name = local_host_name.substring(0, local_host_name.indexOf("."));
    }
    catch(UnknownHostException e){
		/*
		If the local host name cannot be found, just set the name to unknown
		*/
      local_host_name = "unknown";
    }

    System.out.println("\n\n======================================================================\n" +
                       "Welcome to the Hummer system - Hummer server on " + local_host_name + ".\n" +
                       " Copyright (C) 1997-1998 The Hummer " +
                       "Project - hummer@cs.uidaho.edu\n" +
                       "----------------------------------------------------------------------");

    int choice = -1;
    int port = (new Integer(_constants.getProperty("port.HUMMER_PORT"))).intValue();
    

    ServerSocket server_socket = null;
    try {
      server_socket = new ServerSocket(port);
    }
    catch(IOException e) {
      System.err.println("Hummer:startHummer: Error while opening socket: " + e);
    }
    boolean file_not_found = false;
    if(_key_store == null) {
      System.out.print("Hummer: Loading key store...");
      System.out.flush();
      try {
        _key_store = HummerKeyStore.getInstance();
        _key_store.load(true, null);
      }
      catch(KeyStoreException e) {
        System.err.println("Hummer:startHummer: Error while creating/loading HummerKeyStore: " + e);
        return;
      }
      catch(ClassNotFoundException e) {
        System.err.println("Hummer:startHummer: Error while loading key store: " + e);
        return;
      }
      catch(StreamCorruptedException e) {
        System.err.println("Hummer:startHummer: Error while loading key store: " + e);
        return;
      }
      catch(UnknownHostException e) {
        System.err.println("Hummer:startHummer: Error while loading key store: " + e);
        return;
      }
      catch(FileNotFoundException e) {
        file_not_found = true;
      }
      catch(OptionalDataException e) {
        System.err.println("Hummer:startHummer: Error while loading key store: " + e);
        return;
      }
      catch(IOException e) {
        System.err.println("Hummer:startHummer: Error while loading key store: " + e);
        return;
      }
      if(file_not_found) {
        System.out.print("HummerKeyStore not found for this server.\n" +
                           "Please enter password for admin account: ");
        byte[] password = null;
        try {
          java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
          password = in.readLine().trim().getBytes();
          _key_store.load(password);
          _key_store.store(true, null);
        }
        catch(UnloadedKeyStoreException e) {}
        catch(KeyStoreException e) {
          System.err.println("Hummer:startHummer: Error while creating/loading HummerKeyStore: " + e);
          return;
        }
        catch(ClassNotFoundException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(CertificateException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(NoSuchAlgorithmException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(java.security.NoSuchProviderException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(StreamCorruptedException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(UnknownHostException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(OptionalDataException e) {
          System.err.println("Hummer:startHummer: Error while loading key store: " + e);
          return;
        }
        catch(IOException e) {
          System.err.println("Hummer: Error while reading from standard input: " + e);
        }
      }
      System.out.println("loaded.\n" +
                         "Hummer: Waiting for connections from clients." +
                         "\n********************************************\n");

    }


		/*
		The creation of the server objects must be done here and not the
		constructor, because the constructor is called each time
		that a connection with a client is made.
		*/
    _ms = new MessagingServer(_constants);
    _cs = new ConfigServer(_constants);
    _ps = new PeerServer(_constants);
    _ts = new ToolServer(_constants);
    if(commands.contains("all")) {
      startMS();
      startCS();
      startPS();
      startTS();
    }
    else {
      if(commands.contains("ms")) {
        startMS();
      }
      if(commands.contains("cs")) {
        startCS();
      }
      if(commands.contains("ps")) {
        startPS();
      }
      if(commands.contains("ts")) {
        startTS();
      }
    }
    showStatus();

    commands.removeAllElements();

    HummerServerSocket hummer_server_socket;

    while(true) {
      try {
        _client_socket = server_socket.accept();
        System.out.print("Hummer: Have made connection with client.  " +
                           "Will now validate client identity...");
        System.out.flush();
      }
      catch(IOException e) {
        System.err.println("Hummer:startHummer:Error while opening new socket for client: " + e);
        return;
      }
      Thread client_thread = new Thread(new Hummer());
      client_thread.start();
    }
  }

  public void run() {
    try {
      int choice = -1;
      HummerServerSocket hummer_server_socket = new HummerServerSocket(null, _client_socket, _key_store);
      hummer_server_socket.validation(null);
      _key_store.store(true, null);
      System.out.println(". Validated: \"" + hummer_server_socket.getCurrentPartnerAlias() + "\".");
      if(_key_store.getAliasType(hummer_server_socket.getCurrentPartnerAlias()) != AliasInfo._USER_ALIAS_TYPE) {
        hummer_server_socket.throwException("Hummer:startHummer: " +
                                            "Only user accounts have access to control Hummer servers.", null, 0);
      }
      Hashtable client_access_table = hummer_server_socket.getCurrentClientAccessTable();
      Boolean can_connect = (Boolean)(client_access_table.get("Control_Servers"));
      if(!hummer_server_socket.getCurrentPartnerAlias().equals("admin") && !can_connect.booleanValue()) {
        hummer_server_socket.throwException("Hummer:startHummer: " +
                                            "Alias does not have access to control Hummer servers.", null, 0);
      }
      
      Response output = new Response();
      output._message = _choice_list;
		/*
		Since everything went ok, send an _OK message back to the client.
		*/
      hummer_server_socket.writeResponseObject(output);
      Response input = hummer_server_socket.readResponseObject();
      _key_store.store(true, null);
      while(input._type != Response._EOT) {

        choice = ((Integer)input._message).intValue();
		/******************************************
		Parse the choice made by the user
		******************************************/
        switch (choice) {

			/***********************
			Start all of the servers
			***********************/
          case _START_EVERYTHING:
            startMS();
            startCS();
            startPS();
            startTS();
            output._data.add(msStatus());
            output._data.add(csStatus());
            output._data.add(psStatus());
            output._data.add(tsStatus());
            break;

			/***********************
			Start messaging server
			***********************/
        case _START_MS:
          startMS();
          output._data.add(msStatus());
          break;

			/***********************
			Kill messaging server
			***********************/
        case _STOP_MS:
          stopMS();
          output._data.add(msStatus());
          break;


			/***********************
			Start config server
			***********************/
        case _START_CS:
          startCS();
          output._data.add(csStatus());
          break;

			/***********************
			Kill config server
			***********************/
        case _STOP_CS:
          stopCS();
          output._data.add(csStatus());
          break;

			/***********************
			Start peer server
			***********************/
        case _START_PS:
          startPS();
          output._data.add(psStatus());
          break;

			/***********************
			Kill peer server
			***********************/
        case _STOP_PS:
          stopPS();
          output._data.add(psStatus());
          break;


			/***********************
			Start tool server
			***********************/
        case _START_TS:
          startTS();
          output._data.add(tsStatus());
          break;

			/***********************
			Kill tool server
			***********************/
        case _STOP_TS:
          stopTS();
          output._data.add(tsStatus());
          break;


			/***********************
			Reload message handler
			***********************/
        case _RELOAD_MESSAGE_HANDLER:
          if (_ms.isRunning()) {
            _ms.setReloadHandler();
          }
          output._data.add(msStatus());
          break;

			/***********************
			Display status
			***********************/
        case _DISPLAY_STATUS:
          output._data.add(msStatus());
          output._data.add(csStatus());
          output._data.add(psStatus());
          break;
      
			/***********************
			Stop all of the servers
			***********************/
        case _SHUTDOWN_EVERYTHING:
          stopMS();
          stopCS();
          stopPS();
          output._data.add(msStatus());
          output._data.add(csStatus());
          output._data.add(psStatus());
        }
        
		/******************************************
		Send back the response to the user.
		******************************************/
        hummer_server_socket.writeResponseObject(output);

		/*****************************************
		Read the next choice made by the user
		*****************************************/
        input = hummer_server_socket.readResponseObject();
        _key_store.store(true, null);
        output = new Response();
      }
      System.out.println("Hummer: User \"" + hummer_server_socket.getCurrentPartnerAlias() +
                         "\" has disconnected.");
    }
    catch(HummerSocketException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(IOException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(NoSuchAlgorithmException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(KeyStoreException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(UnknownAliasException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(UnrecoverableKeyException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
    catch(CertificateException e) {
      System.err.println("Hummer:startHummer: Error while connecting to client: " + e);
      return;
    }
  }

	/**
	Displays the current status of all the servers on the system.
	*/
  private void showStatus() {
    String[][] statuses = new String[4][];
    statuses[0] = msStatus();
    statuses[1] = csStatus();
    statuses[2] = psStatus();
    statuses[3] = tsStatus();
    System.out.println("\nServer Name\tRunning\t\tStatus\n" +
                       "-----------------------------------------------------------------------");
    Object[] server_info;
    for(int i = 0; i < statuses.length; i++) {
      System.out.println(statuses[i][0].toString() + "\t" + statuses[i][1] + "\t\t" + statuses[i][2]);
    }
    System.out.println("-----------------------------------------------------------------------");
  }

	/**
	Calls MessagingServer to start itself
	*/
  private void startMS() {
    _ms.setRunning(true);
  }

	/**
	If config server is not already going, and then returns the results (started or not)
	*/
  private void startCS() {
    _cs.setRunning(true);
  }

	/**
	If peer server is not already going, and then returns the results (started or not)
	*/
  private void startPS() {
    _ps.setRunning(true);
  }

	/**
	If tool server is not already going, and then returns the results (started or not)
	*/
  private void startTS() {
    _ts.setRunning(true);
  }


  /**
  This method does what you might expect... it stops the MessagingServer

  <h4>Method Usage and Flow:</h4>
  <ul>
  <li> Tell the MessagingServer to kill itself: _ms.killMessagingServer();
  <li> Connect to the port it is on, so it will figure
  out that it needs to exit gracefully (stop blocking).
  </ul>
  */
  private void stopMS() throws UnknownHostException, IOException {
    if(_ms.isRunning()) {
      try {
        _ms.setRunning(false);
        Socket socket;
        InetAddress host_address = InetAddress.getLocalHost();
        Integer value = new Integer(_constants.getProperty("port.MESSAGE_PORT"));
        socket = new Socket(host_address, value.intValue());
        socket.close();
      }
      catch (NumberFormatException ne) {
        System.err.println("Hummer:stopMS:Bad port.MESSAGE_PORT property.");
      }
    }
  }

  /**
  This method does what you might expect... it stops the ConfigServer

  <h4>Method Usage and Flow:</h4>
  <ul>
  <li> Tell the ConfigServer to kill itself: _cs.killConfigServer();
  <li> Connect to the port it is on, so it will figure
  out that it needs to exit gracefully (stop blocking).
  </ul>
  */
  private void stopCS() throws UnknownHostException, IOException {
    if(_cs.isRunning()) {
      _cs.setRunning(false);
      try {
        Socket socket;
        InetAddress host_address = InetAddress.getLocalHost();
        Integer value = new Integer(_constants.getProperty("port.CONFIG_PORT"));
        socket = new Socket(host_address, value.intValue());
        socket.close();
      }
      catch (NumberFormatException ne) {
        System.err.println("Hummer:stopCS:Bad port.CONFIG_PORT property.");
      }
    }
  }


  /**
  This method does what you might expect... it stops the PeerServer

  <h4>Method Usage and Flow:</h4>
  <ul>
  <li> Tell the PeerServer to kill itself: _ps.killPeerServer();
  <li> Connect to the port it is on, so it will figure
  out that it needs to exit gracefully (stop blocking).
  </ul>
  */
  private void stopPS() throws UnknownHostException, IOException {
    if(_ps.isRunning()) {
      _ps.setRunning(false);
      try {
        Socket socket;
        InetAddress host_address = InetAddress.getLocalHost();
        Integer value = new Integer(_constants.getProperty("port.PEER_PORT"));
        socket = new Socket(host_address, value.intValue());
        socket.close();
      }
      catch (NumberFormatException ne) {
        System.err.println("Hummer:stopPS:Bad port.PEER_PORT property.");
      }
    }
  }



  /**
  This method does what you might expect... it stops the ConfigServer

  <h4>Method Usage and Flow:</h4>
  <ul>
  <li> Tell the ConfigServer to kill itself: _cs.killConfigServer();
  <li> Connect to the port it is on, so it will figure
  out that it needs to exit gracefully (stop blocking).
  </ul>
  */
  private void stopTS() throws UnknownHostException, IOException {
    if(_ts.isRunning()) {
      _ts.setRunning(false);
      try {
        Socket socket;
        InetAddress host_address = InetAddress.getLocalHost();
        Integer value = new Integer(_constants.getProperty("port.TOOL_PORT"));
        socket = new Socket(host_address, value.intValue());
        socket.close();
      }
      catch (NumberFormatException ne) {
        System.err.println("Hummer:stopTS:Bad port.CONFIG_PORT property.");
      }
    }
  }


  private String[] msStatus() {
    String[] return_value = new String[3];
    return_value[0] = "Messaging";
    if(_ms.isRunning()) {
      return_value[1] = "yes";
    }
    else {
    	  return_value[1] = "no";
    }
    if((return_value[2] = _ms.getStatus()) == null) {
      return_value[2] = "Normal";
    }
    return return_value;
  }


  private String[] csStatus() {
    String[] return_value = new String[3];
    return_value[0] = "Config   ";
    if(_cs.isRunning()) {
      return_value[1] = "yes";
    }
    else {
      return_value[1] = "no";
    }
    if((return_value[2] = _cs.getStatus()) == null) {
      return_value[2] = "Normal";
    }
    return return_value;
  }

  private String[] psStatus() {
    String[] return_value = new String[3];
    return_value[0] = "Peer     ";
    if(_ps.isRunning()) {
      return_value[1] = "yes";
    }
    else {
      return_value[1] = "no";
    }
    if((return_value[2] = _ps.getStatus()) == null) {
      return_value[2] = "Normal";
    }
    return return_value;
  }


  private String[] tsStatus() {
    String[] return_value = new String[3];
    return_value[0] = "Tool     ";
    if(_ts.isRunning()) {
      return_value[1] = "yes";
    }
    else {
      return_value[1] = "no";
    }
    if((return_value[2] = _ts.getStatus()) == null) {
      return_value[2] = "Normal";
    }
    return return_value;
  }
}
