/*
Java Hummer Project
File: LevelCon.java
Date: 11-23-98
Author: Jesse C. McConnell
Description:  This is the implementation of the level container object.
              This object stores and organizes all the level objects after
              they are created from the contents of the configuration
              files.

Adapted from LevelCon.C and LevelCon.h from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.server;

import java.util.Vector;
import java.net.InetAddress;
import java.util.Enumeration;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.ConvertIP;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.*;


/**
This is the implementation of the level container object. This object
stores and organizes all the level objects after they are created from
the contents of the configuration files.

@author Jesse C. McConnell
*/
public class LevelCon {

  /**
  Vector which should only contain Level Objects.
  */
  private Vector _levels;
  /**
  The default trust level for all levels.  The initial value for this variable
  is set in the Hummer.properties file.
  */
  private int _default_trust_level;
  /**
  The default cooperation level for all levels.  The initial value for this
  variable is set in the Hummer.properties file.
  */
  private int _default_coop_level;
  /**
  The default integrity level for all levels.  The initial value for this
  variable is set in the Hummer.properties file.
  */
  private int _default_integ_level;

  /**
  Initialize the Vector of levels.
  */
  public LevelCon() {
    _levels = new Vector();
  }

  /**
  Add a Level to the Vector of Levels.

  @param new_level A Level Object that is to be added to the Vector.
  */
  public void addLevel(Level new_level) {
    _levels.addElement(new_level);
  }

  /**
  This function sets the defualt trust level. This trust level is
  assigned to any message from a host that is not found in the
  trust container.

  @param level Integer representing the default trust level.
  */
  public void setDefaultTrust(int level) {
     _default_trust_level = level;
  }

  /**
  This function sets the defualt cooperation level. This cooperation
  level is assigned to any message from a host that is not found in
  the cooperation container.

  @param level Integer representing the default cooperation level.
  */
  public void setDefaultCoop(int level) {
    _default_coop_level = level;
  }

  /**
  This function sets the defualt integrity level. This integrity
  level is assigned to any message from a host that is not found in
  the integerity container.

  @param level Integer representing the default integrity level.
  */
  public void setDefaultInteg(int level) {
    _default_integ_level = level;
  }

  /**
  This function takes an IP address as a parameter and returns the
  trust associated with that address. Default is returned if the
  host ip is not found.

  @param addr InetAddress to check the trust level on.
  @return The trust value for addr, or the default trust level.
  */
  public int getTrust(InetAddress addr) {

    int trust;

    //start with default trust value.
    trust = _default_trust_level;

    Enumeration enum = _levels.elements();

    while (enum.hasMoreElements()) {
      Level instance = (Level)enum.nextElement();
      //Check to see if it is an explicit value.  In this case the end_host will
      //be zero, otherwise it is a range of IP's
      if (instance.explicit) {
        if (addr.equals(instance.start_host_ip)) {
          trust = (int)instance.trust_level;
          break;
        }
      }
      else {
        ConvertIP converter = new ConvertIP();
        Long start = converter.convertIP(instance.start_host_ip);
        Long end = converter.convertIP(instance.end_host_ip);
        Long to_test = converter.convertIP(addr);

        if ((start.longValue() <= to_test.longValue()) &&
            (end.longValue() >= to_test.longValue())) {

          trust = (int)instance.trust_level;
          break;
        }
      }
    }

    return trust;
  }

  /**
  This function takes an IP address as a parameter and returns the
  cooperation associated with that address. Default is returned if the
  host ip is not found.

  @param addr InetAddress to check the cooperation level on.
  @return The cooperation value for addr, or the default cooperation level.
  */
  public int getCoop(InetAddress addr) {

    int coop;

    //start with default trust value.
    coop = _default_coop_level;

    Enumeration enum = _levels.elements();

    while (enum.hasMoreElements()) {
      Level instance = (Level)enum.nextElement();
      //Check to see if it is an explicit value.  In this case the end_host will
      //be zero, otherwise it is a range of IP's
      if (instance.explicit) {
        if (addr.equals(instance.start_host_ip)) {
          coop = (int)instance.coop_level;
          break;
        }
      }
      else {
        ConvertIP converter = new ConvertIP();
        Long start = converter.convertIP(instance.start_host_ip);
        Long end = converter.convertIP(instance.end_host_ip);
        Long to_test = converter.convertIP(addr);

        if ((start.longValue() <= to_test.longValue()) &&
            (end.longValue() >= to_test.longValue())) {

          coop = (int)instance.coop_level;
          break;
        }
      }
    }

    return coop;
  }


  /**
  This function takes an IP address as a parameter and returns the
  integrity associated with that address. Default is returned if the
  host ip is not found.

  @param addr The InetAddress to check the integrity level on.
  @return The integrity level for addr, or the default integrity level.
  */
  public int getInteg(InetAddress addr) {

    int integ;

    //start with default trust value.
    integ = _default_integ_level;

    Enumeration enum = _levels.elements();

    while (enum.hasMoreElements()) {
      Level instance = (Level)enum.nextElement();
      //Check to see if it is an explicit value.  In this case the end_host will
      //be zero, otherwise it is a range of IP's
      if (instance.explicit) {
        if (addr.equals(instance.start_host_ip)) {
          integ = (int)instance.integ_level;
          break;
        }
      }
      else {
        ConvertIP converter = new ConvertIP();
        Long start = converter.convertIP(instance.start_host_ip);
        Long end = converter.convertIP(instance.end_host_ip);
        Long to_test = converter.convertIP(addr);

        if ((start.longValue() <= to_test.longValue()) &&
            (end.longValue() >= to_test.longValue())) {

          integ = (int)instance.integ_level;
          break;
        }
      }
    }

    return integ;
  }

};



