
/*
Java Hummer Project
File: MessageHandler.java
Date: 11-17-98
Author: Jesse C. McConnell
Description: This is the bulk of the Messaging Server.  It is passed a message
             object and deals with it.

Adapted from MessageHandler.C, MessageHandler.h and ConfigLoad.C from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.net.ConnectException;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.PrintWriter;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.OptionalDataException;
import java.io.FileNotFoundException;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.sql.ResultSet;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.Connection;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.ConvertIP;
import edu.uidaho.hummer.util.DateTime;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.*;


/**
This is the class that takes care of the message, matching it to filters,
 forwarding the message around and adding it to the database.

@author Jesse C. McConnell
@see Message
@see MessageFilterCon
@see LevelCon
*/
public class MessageHandler {

  /**
  Properties object that is passed in during the construction of this object.
  */
  private Properties _constants;
  /**
  Container of all of the Message_Filters of the local running Hummer.
  */
  private MessageFilterCon _filters;
  /**
  Vector should take only HostList pairings representing hosts which Messages
  should not be accepted from.
  */
  private Vector _host_kill_list;
  /**
  Container for all of the Level objects that are created during the call to
  loadLevels().
  */
  private LevelCon _levels;
  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _sql_url = new String();
  /**
  Database Driver which is taken from the corresponding _constants property.
  */
  private String _sql_driver = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _sql_user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  private String _sql_password = new String();


  /**
  Constructor for MessageHandler.  Passed in is the Hummer systems Properties
  object.  This properties object is created from the System properties and
  those defined in the Hummer.properties file.  This constructor reads the
  database fields from the defined Properties.
  */
  public MessageHandler(Properties constants) {

    _constants = constants;
    //Initialize the database strings from the properties object.

    try {
      String address = InetAddress.getLocalHost().getHostAddress();
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
	_sql_url = _constants.getProperty(
		   "jdbc.url." + address) + nametoken.nextToken();
      }
      else {
	_sql_url = _constants.getProperty("jdbc.url") + nametoken.nextToken();
      }
      
      if (_constants.containsKey("jdbc.driver." + address)) {
	_sql_driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
	_sql_driver = _constants.getProperty("jdbc.driver");
      }
      
      if (_constants.containsKey("jdbc.user." + address)) {
	_sql_user = _constants.getProperty("jdbc.user." + address);
      }
      else {
	_sql_user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
	_sql_password = _constants.getProperty("jdbc.password." + address);
      }
      else {
	_sql_password = _constants.getProperty("jdbc.password");
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("MessagingServer:startServer:" + uhe);
    }
    
    try {
      Class.forName(_sql_driver);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("MessageHandler:" + cnfe);
      System.exit(0);
    }

    _host_kill_list = new Vector();
    _levels = new LevelCon();
  }



  /**
  This function is where all the message handling takes place. It
  uses the message filters that are matched with the message for processing.
  This is where all of the matching of the message object and the filters
  takes place, outside of a regexp match to determine if the filter applys in
  the first place.<br>

  Currently, the only matching fields that are functional are the ones
  dealing with Trust, Cooperation, and Integrity.<br>

  Still be done are filter matching dealing with dates, times, and host kill
  lists.<br>
  <p>
  Database Call:
  <pre>
  INSERT INTO log VALUES (" + message_trust + "," +
                          message_coop + "," + message_integ + ", '" +
                          dtconverter.makeDate(new_message.date) + "', '" +
                          dtconverter.makeTime(new_message.time) + "', '" +
                          new_message.description + "', '" + new_message.host_IP +
                          "', '" + new_message.filename + "', '" +
                          new_message.tool + "');
  </pre>
  @param new_message A completed Message Object to match to filters and route
  to the appropiate locations.
  @param addr InetAddress representing the host of the MessagingServer the
  message was sent from.  This will the local host until such time RMI is
  incorporated into Hummer.
  @return A boolean value, true if the message was handled properly, false if not.
  */
  public boolean handleMessage(Message new_message, InetAddress addr, HummerKeyStore key_store) {

    int message_trust;
    int message_coop;
    int message_integ;
    int alerted = 0;
    Vector filter_list; //list of filters
    Connection db_connection;
    Statement db_statement;
    SimpleDateFormat date_format = new SimpleDateFormat("MM-dd-yyyy");
    SimpleDateFormat time_format = new SimpleDateFormat("hh:mm:ss");
    //++Needed for authentication
    PersonalConfig personal;
    //++Needed for authentication
    Response response = new Response();
    //++Needed for authentication
    HummerClientSocket hummer_client_socket = null;
    String address = new String();

    //If addr is in the kills file, then we are done.
    if (this.inKills(addr)) {
      return true;
    }


    try {
      address = InetAddress.getLocalHost().getHostAddress();
      //++Needed for authentication
      personal = new PersonalConfig(address, null);
    }
    catch(UnknownHostException e) {
        System.err.println("ConfigI:closeServer:Host not found:  " + e);
        return false;
    }

    //Get Levels for message.
    message_trust = _levels.getTrust(addr);
    message_coop = _levels.getCoop(addr);
    message_integ = _levels.getInteg(addr);
    
    //Add levels to Message Object.  This may be an issue later, 
    //other sites find out my levels for possibly other hosts.
    new_message.trust = String.valueOf(message_trust);
    new_message.cooperation = String.valueOf(message_coop);
    new_message.integrity = String.valueOf(message_integ);

    //Match against regex's we have and get Vector of Filters.
    filter_list = _filters.matchExp(new_message.description);

    Enumeration enum = filter_list.elements();

    while (enum.hasMoreElements()){


      MessageFilter instance = (MessageFilter)enum.nextElement();


		/*
		This part is a bit funky.  The jist of it is to search
		thru the filters and see if it fails on any of them.  The
		moment it does it stops working with that filter and
		moves to a new filter.
		ASSUMPTION: The trust and other values are based on the
		1 is low and 5 is high scale.
		*/


      try {
		/*
		message_trust is the trust this machine has for the addr from
		the message.  If it is greater then the trust_level from the
		instance then skip this message (for this matching filter)
		*/
  //System.out.println("Trust:" + message_trust + "|" + instance.trust_level);
        if (!(message_trust > Integer.parseInt(instance.trust_level))) {
          continue;
        }
      }
      catch (NumberFormatException e) {
        System.err.println("MessageHandler:handleMessage:FilterCheck1:" + e);
        continue;
      }

      try {
        if (!(message_coop > Integer.parseInt(instance.coop_level))) {
          continue;
        }
      }
      catch (NumberFormatException e) {
      System.err.println("MessageHandler:handleMessage:FilterCheck2:" + e);
        continue;
      }


      try {
        if (!(message_integ > Integer.parseInt(instance.integ_level))) {
          continue;
        }
      }
      catch (NumberFormatException e) {
      System.err.println("MessageHandler:handleMessage:FilterCheck3:" + e);
        continue;
      }



      if ((instance.start_datetime.getTime() != 0) && 
	  (instance.end_datetime.getTime() != 0)) {
	
	/* If the Message is before or after the start and end time of the
	   MessageFilter then we continue by breaking out of this loop and
	   going to the next MessageFilter that matchs.
	*/
	if ((instance.start_datetime.before(new_message.date_time))
	    || (instance.end_datetime.after(new_message.date_time))) {
	  continue;
	}
      }

      
      Enumeration host_list = instance.from_host_list.elements();
      boolean flag = false;  //local tester to see if the message is in the
                             //allowable host list.

      if (host_list.hasMoreElements()) {
        while (host_list.hasMoreElements()) {
          HostList host = (HostList)host_list.nextElement();

          if (host.explicit) {
            if (addr.equals(host.start_host_ip)) {
              flag = true;
            }
          }
          else {
            ConvertIP converter = new ConvertIP();
            Long start = converter.convertIP(host.start_host_ip);
            Long end = converter.convertIP(host.end_host_ip);
            Long to_test = converter.convertIP(addr);

            if ((start.longValue() <= to_test.longValue()) &&
                (end.longValue() >= to_test.longValue())) {
              flag = true;
            }
          }
        }
      }
      else {
        flag = true;
      }

      if (!flag) {
        continue;
      }
      
      //Made it through the filter section...


      if (instance.log) {
	try {
	  db_connection = DriverManager.getConnection(_sql_url, _sql_user, _sql_password);
	  db_statement = db_connection.createStatement();
	  
	  
	  

	  DateTime dtconverter = new DateTime();
	  String log_update = "INSERT INTO log VALUES (" + 
	    message_trust + "," +
	    message_coop + "," + message_integ + ", '" +
	    date_format.format(new_message.date_time) + "', '" +
	    time_format.format(new_message.date_time) + "', '" +
	    new_message.description + "', '" + new_message.host_IP +
	    "', '" + new_message.filename + "', '" +
	    new_message.tool + "');";
	  
	  //System.out.println("LOG: " + log_update);
	  
	  
	  int eu = db_statement.executeUpdate(log_update);
	  db_statement.close();
	  db_connection.close();
	}
	catch (SQLException S) {
	  System.err.println("MessageHandler:handleMessage:" + S);
	}
      }

      //Ok.  We have gone through the lengthy process of commiting this message
      //to the database.  We should now proceed to alerting the console window.
      //Yes, in case you were wondering, we are still in that bleeding while 
      //loop.  Remove the alerted ! = 1 statement if you want to be alerted 
      //for every filter that matches the message.
      if (instance.alert && (alerted != 1)) {
        Socket to_alert;

        try {
          to_alert = new Socket(InetAddress.getLocalHost(),
		              Integer.parseInt(_constants.getProperty(
		              "port.ALERT_INTERFACE_PORT")));
          hummer_client_socket = new HummerClientSocket(personal, to_alert, key_store);
          hummer_client_socket.validation(null);
          //++Needed for authentication
          response._type = Response._OK;
          //++Needed for authentication
          response._message = new_message;

          hummer_client_socket.writeResponseObject(response);
          hummer_client_socket.close();
        }
        catch (ConnectException e) {
//          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (UnknownHostException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (OptionalDataException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (FileNotFoundException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (IOException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (KeyStoreException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (NoSuchAlgorithmException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (UnloadedKeyStoreException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (HummerSocketException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (UnknownAliasException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (CertificateException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
        catch (UnrecoverableKeyException e) {
          System.err.println("MessageHandler:handleMessage: Cannot send message to console:" + e);
        }
      }
      
      
      //Ok.  Now we are done informing the interface of what we have recieved.
      //Onwards to forwarding the message out to other hosts.

      Enumeration forward_list = instance.forward_host_list.elements();
      while (forward_list.hasMoreElements()) {
        HostList host = (HostList)forward_list.nextElement();
        Socket to_forward;

        try {
          to_forward = new Socket(host.start_host_ip,
                                                Integer.parseInt(_constants.getProperty(
                                                "port.MESSAGE_PORT")));

          hummer_client_socket = new HummerClientSocket(personal, to_forward, key_store);
          hummer_client_socket.validation(null);
          //++Needed for authentication
          response._type = Response._OK;
          //++Needed for authentication
          response._message = new_message;

          hummer_client_socket.writeResponseObject(response);
          hummer_client_socket.close();
        }
        catch (ConnectException ce) {
          System.err.println("MessageHandler:handleMessage:Not a Problem:" + ce);
        }
        catch (IOException ioe) {
          System.err.println("MessageHandler:handleMessage: " + ioe);
        }
        catch (Exception e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }

      }
      
      //We have now taken care of the host that should be forwwarded this message.
      //Now we send it to the members of the Peer group.
      Enumeration peer_list = instance.forward_peer_list.elements();
      while (peer_list.hasMoreElements()) {
        PeerList host = (PeerList)peer_list.nextElement();
        Socket to_peer;
        Peer peer_message = new Peer();
	
        date_format = new SimpleDateFormat("yyyyMMdd");
        time_format = new SimpleDateFormat("hhmmss");
	
        try {
          to_peer = new Socket(host.modhost_ip, Integer.parseInt(
			       _constants.getProperty("port.PEER_PORT")));
          peer_message.action = "message";
          peer_message.peer_group = host.group_name;
          peer_message.message = new_message;

	  //String msg = "m|" + host.group_name + "|" +
	  //  String.valueOf(new_message.message_age.intValue() + 1) +
	  //  "|" + new_message.description
	  //  + "|" + date_format.format(new_message.date_time) + "|" + 
	  //  time_format.format(new_message.date_time) + "|" +
	  //  new_message.host_IP + "|" + new_message.filename + "|" +
	  //  new_message.tool;
	  

          hummer_client_socket = new HummerClientSocket(personal, to_peer, key_store);
          hummer_client_socket.validation(null);
          //++Needed for authentication
          response._type = Response._OK;
          //++Needed for authentication
          response._message = peer_message;

          hummer_client_socket.writeResponseObject(response);
          hummer_client_socket.close();
        }
        catch (ConnectException ce) {
          System.err.println("MessageHandler:handleMessage:Not a Problem:" + ce);
        }
        catch (IOException ioe) {
          System.err.println("MessageHandler:handleMessage: " + ioe);
        }
        catch (UnloadedKeyStoreException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
        catch (HummerSocketException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
        catch (NoSuchAlgorithmException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
        catch (KeyStoreException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
        catch (CertificateException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
        catch (UnknownAliasException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
        catch (UnrecoverableKeyException e) {
          System.err.println("MessageHandler:handleMessage: " + e);
        }
      }
    }
    //Wow...That was a royal pain...
    return true;
  }



  /**
  A call to this method returns a boolean value representing whether or not
  the passed in IP address is in the kill list.  If a true is returned then a
  message should not be processed, instead it should be thrown away.

  @param addr InetAddress representing the IP of a machine to check whether it
  is contained in the _host_kill_list.
  @return True if addr is in the _host_kill_list, false if it is not.
  */
  private boolean inKills(InetAddress addr) {

    Enumeration enum = _host_kill_list.elements();
    boolean truth = false;
    ConvertIP converter = new ConvertIP();

    while (enum.hasMoreElements()) {
      HostList instance = (HostList)enum.nextElement();
      //Check to see if it is an explicit value.  In this case the end_host will
      //be zero, otherwise it is a range of IP's
      if (instance.explicit) {
        if (addr.equals(instance.start_host_ip)) {
          truth = true;
          break;
        }
      }
      else {
        Long start = converter.convertIP(instance.start_host_ip);
        Long end = converter.convertIP(instance.end_host_ip);
        Long to_test = converter.convertIP(addr);

        if ((start.longValue() <= to_test.longValue()) &&
            (end.longValue() >= to_test.longValue())) {

          truth = true;
          break;
        }
      }
    }

    return truth;
  }

  /**
  When called this method calls the private methods loadMessageFilters(),
  loadLevel(), and loadKills().  All combined, this call initializes the
  MessageHandler.  This can be called again to refresh the Filter Objects, etc.

  @return Boolean representing if all the configuration calls were successful.
  */
  public boolean loadConfigFiles() {

    if (!loadMessageFilters()) {
      System.err.println("MessageHandler:loadConfigFiles: MessageFilter issue.");
      return false;
    }
    if (!loadLevel()) {
      System.err.println("MessageHandler:loadConfigFiles: Level issue.");
      return false;
    }
    if (!loadKills()) {
      System.err.println("MessageHandler:loadConfigFiles: Kill issue.");
      return false;
    }
    return true;
  }

  /**
  A call to this method will return the manager of the local running Hummer.
  <p>
  Database Call:<br>
  <pre>
  SELECT * from manager;
  </pre>

  @return A string representing the address of the manager for the local running
  Hummer.  This should be converted to an InetAddress.
  */
  private String getManager() {
    String manager;
    Connection db_connection;
    Statement db_statement;

    // Connect to DB server
    try {
      db_connection = DriverManager.getConnection(_sql_url, _sql_user, _sql_password);
      db_statement = db_connection.createStatement();

      String query = "SELECT * FROM manager;";
      
     
      ResultSet rs = db_statement.executeQuery("query");
      rs.next();
      manager = rs.getString("manager");

      db_statement.close();
      db_connection.close();
      
      return manager;
    }
    catch (SQLException S) {
      System.err.println("MessageHandler:getManager:" + S);
    }
    //If the query doesn't work then return a -1
    return String.valueOf("-1");
  }

  /**
  A call to this method will return all of the Subordinates to the local
  running Hummer.
  <p>
  Database Call:
  <pre>
  SELECT * from subordinates;
  </pre>

  @return A Vector of HostList's representing all of the Subordinates of the
  local running Hummer.
  */
  private Vector getSubordinates() {
    HostList subordinate = new HostList();
    Vector the_subordinates = new Vector();
    Connection db_connection;
    Statement db_statement;
    String sub;

    // Connect to DB server
    try {
      db_connection = DriverManager.getConnection(_sql_url, _sql_user, _sql_password);
      db_statement = db_connection.createStatement();

      String query = "SELECT * FROM subordinates;";
      
   
      ResultSet rs = db_statement.executeQuery("query");
      while (rs.next()) {
        sub = rs.getString("subord");
        subordinate.start_host_ip = InetAddress.getByName(sub);
        subordinate.explicit = true;
        the_subordinates.addElement(subordinate);
      }
      
      db_statement.close();
      db_connection.close();

      return the_subordinates;
    }
    catch (SQLException S) {
      System.err.println("MessageHandler:getSubordinates:" + S);
    }
    catch (UnknownHostException e) {
      System.err.println("MessageHandler:getSubordinates:We have Issues:" + e);
    }
    //If the query doesn't work then return a -1
    return new Vector();
  }


  /**
  This function will load the kill file container similar to loading the message
  filter container.
  <p>
  Database Call:
  <pre>
  SELECT * from kill_file;
  </pre>

  @return boolean value representing whether or not the loadKills() was
  successful.
  */
  private boolean loadKills() {

    String kill_start_addr;
    String kill_end_addr;
    Connection db_connection;
    Statement db_statement;
    int inherit;
    ResultSet rs;


    try {
      db_connection = DriverManager.getConnection(_sql_url, _sql_user, _sql_password);
      db_statement = db_connection.createStatement();
    
      String query = "SELECT * FROM kill_file;";
      
      rs = db_statement.executeQuery(query);

      db_statement.close();
      db_connection.close();
    }
    catch (SQLException S) {
      System.err.println("MessageHandler:loadKills:" + S);
      return false;
    }

    try {
      while(rs.next()){
        kill_start_addr = rs.getString("start_addr");
        kill_end_addr = rs.getString("end_addr");
        inherit = rs.getInt("inherit");

	HostList temp = new HostList();
	temp.start_host_ip = InetAddress.getByName(kill_start_addr);
	temp.explicit = true;
	_host_kill_list.addElement(temp);
      }
    }
    catch (SQLException S) {
      System.err.println("MessageHandler:loadKills:" + S);
      return false;
    }
    catch (UnknownHostException e) {
      System.err.println("MessageHandler:loadKills:" + e);
      return false;
    }
    return true;
  }

  /**
  This function loads the level container after it calls the database for the
  values.  Starts out with the default values specified in the _constants file.
  <p>
  Database Call:
  <pre>
  SELECT * from levels;
  </pre>

  @return Boolean value representing the success of loading the Levels.
  */
  private boolean loadLevel() {

    String level_start_addr;
    String level_end_addr;
    Connection db_connection;
    Statement db_statement;
    int inherit;

    //Set the default levels.
    try {
      _levels.setDefaultTrust(Integer.parseInt(_constants.getProperty(
                             "level.DEFAULT_TRUST")));
      _levels.setDefaultCoop(Integer.parseInt(_constants.getProperty(
                             "level.DEFAULT_COOP")));
      _levels.setDefaultInteg(Integer.parseInt(_constants.getProperty(
                             "level.DEFAULT_INTEG")));
    }
    catch (NumberFormatException nfe) {
      System.err.println("MessageHandler:loadLevel:" + nfe);
    }

    ResultSet rs;

    try {
      db_connection = DriverManager.getConnection(_sql_url, _sql_user, 
						  _sql_password);
      db_statement = db_connection.createStatement();

      String query = "SELECT * FROM levels;";
      
      rs = db_statement.executeQuery(query);

      db_statement.close();
      db_connection.close();
    }
    catch (SQLException S) {
      System.err.println("MessageHandler:loadLevel:" + S);
      return false;
    }

    try {
      while(rs.next()){
        level_start_addr = rs.getString("start_addr");
        level_end_addr = rs.getString("end_addr");

	Level temp = new Level();
	temp.start_host_ip = InetAddress.getByName(level_start_addr);
	temp.explicit = true;
	temp.trust_level = (char)rs.getInt("trust");
	temp.coop_level = (char)rs.getInt("coop");
	temp.integ_level = (char)rs.getInt("integ");
	_levels.addLevel(temp);
      }
    }
    catch (SQLException sqle) {
      System.err.println("MessageHandler:loadLevel:" + sqle);
      return false;
    }
    catch (UnknownHostException uhe) {
      System.err.println("MessageHandler:loadLevel:" + uhe);
      return false;
    }
    return true;
  }


  /**
  The loadMessageFilters Section. Basic flow of this method is to make the
  following queries and then take all that information and package it all
  up into a Message Filter object.

  <p>
  Database Calls:
  <pre>
  1 - SELECT * FROM filter;
  2 - SELECT * FROM accept_list WHERE filter = %s;
  3 - SELECT pgroups.mod_addr, pgroups.pgroup FROM forward_peer, pgroups WHERE
             forward_peer.filter = %s AND forward_peer.pgroup = pgroups.pgroup;
  4 - SELECT * FROM forward_list WHERE filter = %s;
  </pre>

  @return Boolean representing the success of the loadMessageFilters() call.
  */
  private boolean loadMessageFilters() {
    _filters = new MessageFilterCon();
    String start_addr_string;
    String end_addr_string;
    Connection db_connection;
    Statement db_statement;
    ResultSet rs_filter;
    ResultSet rs_accept;
    ResultSet rs_peer;
    ResultSet rs_forward;

    try {
      db_connection = DriverManager.getConnection(_sql_url, _sql_user, _sql_password);
      db_statement = db_connection.createStatement();
 
      String query = "SELECT * FROM filter;";
      
      rs_filter = db_statement.executeQuery(query);

    }
    catch (SQLException S) {
      System.err.println("MessageHandler:loadMessageFilters:" + S);
      return false;
    }

    try {
      while (rs_filter.next()) {

        //This the instance of a filter that we are filling up.
        //One of these for each filter.
        MessageFilter temp = new MessageFilter();
       
        //Fill up the most basic parts of a MessageFilter.
        temp.regexp = rs_filter.getString("expression");
	
	java.sql.Date tmp_date1;
	java.sql.Date tmp_date2;
	java.sql.Time tmp_time1;
	java.sql.Time tmp_time2;
       
	try {
	  tmp_date1 = rs_filter.getDate("date_start");
	  if (tmp_date1 == null) {
	    tmp_date1 = new java.sql.Date(0);
	  }
	}
	catch (NullPointerException npe) {
	  tmp_date1 = new java.sql.Date(0);
	}

	try {
	  tmp_date2 = rs_filter.getDate("date_end");
	  if (tmp_date2 == null) {
	    tmp_date2 = new java.sql.Date(0);
	  }
	}
	catch (NullPointerException npe) {
	  tmp_date2 = new java.sql.Date(0);
	}
	
	try {
	  tmp_time1 = rs_filter.getTime("time_start");
	  if (tmp_time1 == null) {
	    tmp_time1 = new java.sql.Time(0);
	  }
	}
	catch (NullPointerException npe) {
	  tmp_time1 = new java.sql.Time(0);
	}
	
	try {
	  tmp_time2 = rs_filter.getTime("time_end");
	  if (tmp_time2 == null) {
	    tmp_time2 = new java.sql.Time(0);
	  }
	}
	catch (NullPointerException npe) {
	  tmp_time2 = new java.sql.Time(0);
	}
	

	/* I am fearful of these next two commands.  Right now the logic seems
	   sound because the java.sql.Time/Date zero out each other so the
	   two added together outta be the corrent date and time.
	*/
       
	temp.start_datetime = new Date((tmp_date1.getTime() + tmp_time1.getTime()));
	
	temp.end_datetime = new Date ((tmp_date2.getTime() + tmp_time2.getTime()));

        temp.trust_level = String.valueOf(rs_filter.getInt("trust"));
        temp.coop_level = String.valueOf(rs_filter.getInt("coop"));
        temp.integ_level = String.valueOf(rs_filter.getInt("integ"));


        int b = rs_filter.getInt("log");
	temp.log = (b != 0);
        int a = rs_filter.getInt("alert");
	temp.alert = (a != 0);

        temp.from_host_list = new Vector();
        temp.forward_peer_list = new Vector();
        temp.forward_host_list = new Vector();



        /*
        Now we want to see if there is an accept list that needs to be
        put into the message filter object.  All this for one element of the
        MessageFilter Object.
        */

        String query2 = "SELECT * FROM accept_list WHERE filter = '" +
                        temp.regexp + "';";
        try {
          rs_accept = db_statement.executeQuery(query2);
          while(rs_accept.next()) {
            String start_host = rs_accept.getString("start_addr");

            try {

	      HostList temp_list = new HostList();
	      temp_list.start_host_ip = InetAddress.getByName(start_host);
	      temp_list.explicit = true;
	      temp.from_host_list.addElement(temp_list);
            }
            catch (UnknownHostException e) {
              System.err.println("MessageHandler:loadMessageFilters:" + e);
	      return false;
            }
          }
        }//Accept Try clause
        catch (SQLException sqle) {
          System.err.println("MessageHandler:loadMessageFilters:" + sqle);
	  return false;
        }


        /*
	  We are done with processing the from_host_list for the temp 
	  MessageFilter Now we will fill out the forward_peer_list Vector
        */

        String query3 = "SELECT pgroups.mod_addr, pgroups.pgroup FROM forward_peer," +
                        "pgroups WHERE forward_peer.filter = '" + temp.regexp +
                        "' AND forward_peer.pgroup = pgroups.pgroup;";
        try {
          rs_peer = db_statement.executeQuery(query3);
          while(rs_peer.next()) {
            PeerList peer = new PeerList();
            peer.modhost_ip = InetAddress.getByName(rs_peer.getString("mod_addr"));
            peer.group_name = rs_peer.getString("pgroup");
            temp.forward_peer_list.addElement(peer);
          }
        }
        catch (SQLException S) {
          System.err.println("MessageHandler:loadMessageFilters: " + S);
	  return false;
        }


        /*
        Now we want to fill out the forward_host_list Vector in the MessageFilter
        */
        String query4 = "SELECT * FROM forward_list WHERE filter = '" +
    temp.regexp + "';";

        try {
          rs_forward = db_statement.executeQuery(query4);

          while(rs_forward.next()) {

            String start_host = rs_forward.getString("host");

            try {
	      HostList temp_list = new HostList();
	      temp_list.start_host_ip = InetAddress.getByName(start_host);
	      temp_list.explicit = true;
	      temp.forward_host_list.addElement(temp_list);
            }
            catch (UnknownHostException e) {
              System.err.println("MessageHandler:loadMessageFilters:" + e);
	      return false;
            }
          }
	}//Accept Try clause
	catch (SQLException sqle) {
          System.err.println("MessageHandler:loadMessageFilters:" + sqle);
	  return false;
	}
	
        //After all of that work, add the filter into the 
	//MessageFilterContainer..
        _filters.addFilter(temp);
      }

      db_statement.close();
      db_connection.close();

    }
    catch (UnknownHostException uhe) {
      System.err.println("MessageHandler:loadMessageFilters:" + uhe);
      return false;
    }
    catch (SQLException sqle) {
      System.err.println("MessageHandler:loadMessageFilters:" + sqle);
      return false;
    }
    
    return true;
  }
};