/*
Java Hummer Project
File: MessagingServer.java
Date: 11-11-98
Author: Jesse C. McConnell
Description: This is the implementation of the messaging server.
             The server waits for clients to connect and sends them
             to a MessageHandler which processes the client.  This
             class has been converted to a thread class to be started
             by another class.

Adapted from Server.h and Server.C from the Hummer Project.

 Copyright (C) 1997-1999 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.ResultSet;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.Connection;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.*;

/**
The MessagingServer class is the Thread whose purpose is message processing in 
Hummer.

<h4>Class Usage and Flow:</h4>
<ul>
<li>Constructor creates ServerSocket with the port.MESSAGE_PORT property from 
the _constants object.
<li>The run() method is launched because it is a Thread.
<li>The startServer() method is run from the Constructor.
<li>Loops in startServer() on boolean variable _running.
<ul>
<li>Accepts connections from clients and hands off the client socket to the 
MessageHandler class via the processRequest() method.
<li>When killMessagingServer() is called, loop ends after next connection.
</ul>
<li>Thread unbinds from sockets and stops naturally.
</ul>

@author Jesse C. McConnell
*/

public final class MessagingServer implements Runnable {

  /**
  The port number that the MessagingServer will run on.
  */
  private int _port;

  /**
  Boolean value representing whether or not the MessagingServer should listen
  for additional connections.
  */
  private boolean _running = false;

  /**
  Thread used to wrap around this Runnable object.  This is done
  rather than having Messaging server extends Thread because once
  a thread completes the run() method it is considered dead...forever.
  So a new thread must be created each time that the server is started up
  again.
  */
  private Thread _wrapper_thread;

  /**
  Information about the current state of messaging server.  This will
  usually hold error information.  If MessagingServer is in a normal state,
  this will be null.
  */
  private String _status = null;

  /**
  Boolean value representing whether or not the MessageHandler needs to be
  reloaded.  This should be set to true when the filters change to force the
  new filter objects to be created.
  */
  private boolean _reload_handler = true;
  /**
  The server socket for the MessagingServer.  This may not need to be a class
  variable.
  */
  private ServerSocket _server_socket;
  /**
  The instance of the MessageHandler for this MessagingServer.
  */
  private MessageHandler _msg_handler;
  /**
  System Properties object that when created contains the System.getProperties
  and gets the properties defined in Hummer.properties.
  */
  private Properties _constants = new Properties(System.getProperties());


  /**
  Creates the MessagingServer using the default port specified in the
  Hummer properties file.  The properties files is passed in during 
  construction of this object.

  @param constants A Hummer properties file being passed in which removes
  the extra construction time for Properties objects.
   */

  public MessagingServer(Properties constants) {

    _constants = constants;

    try{
      _port = (new Integer(_constants.getProperty("port.MESSAGE_PORT"))).intValue();
    }
    catch (NumberFormatException ne) {
      System.err.println("MessagingServer: Bad port.MESSAGE_PORT property.");
    }
  }




  /**
  This function reads in the incoming message, parses it into a
  message object and sends it to the message handler.

  <h4>Method Usage and Flow:</h4>
  <ul>
  <li>Takes in a socket and reads a Message object off of the wire.
  <li>Increase the age of the Message object.
  <li>Checks the Message age against a constant specified in the Properties
  file.
  <li>Call the MessageHandler.handleMessage(Message, rmt_addr) method to 
  deal with the Message object.
  </ul>

  @param in_sock The accepted socket from the connected client that will
  be sending a serialized Message object.
  */
  private void processRequest(Socket in_sock) {
    
    Message tmp_msg = new Message();
    InetAddress rmt_addr;
    //++Needed for authentication
    HummerServerSocket hummer_socket;
    //++Needed for authentication
    HummerKeyStore key_store;
    //++Needed for authentication
    Response response;
    String address = new String();

    //++This try and catch is needed for authentication
    try {
      address = InetAddress.getLocalHost().getHostAddress();
      key_store = HummerKeyStore.getInstance();
      key_store.load(true, address);
      //++Needed for authentication
      hummer_socket = new HummerServerSocket(null, in_sock, key_store);
      //++Needed for authentication
      hummer_socket.validation(null); 
      //++Needed for authentication
      //      System.out.println("has validated:" + _hummer_socket.hasValidated());
      response = hummer_socket.readResponseObject();
      if (response._type == Response._EOT) {
        try {
        key_store = HummerKeyStore.getInstance();
        key_store.load(true, address);
        }
        catch (Exception e) {
          _status = "Error: " + e;
          System.err.println("MessagingServer:ProcessRequest: " + e);
        }
      }
      hummer_socket.close();

      //++Needed for authentication
      tmp_msg = (Message)response._message;
      
      //Increase the Message Age
      int i = tmp_msg.message_age.intValue() + 1;
      tmp_msg.message_age.valueOf(String.valueOf(i));
      if (tmp_msg.message_age.intValue() <
        Integer.parseInt(_constants.getProperty("const.MAX_MESSAGE_AGE"))) {
	
        rmt_addr = in_sock.getInetAddress();
        _msg_handler.handleMessage(tmp_msg, rmt_addr, key_store);
	
      }
    }
    catch (IOException ioe) {
      _status = "MessagingServer:processRequest:" + ioe;
      System.err.println(_status);
      return;
    }
    catch (NumberFormatException nfe) {
      _status = "MessagingServer:processRequest:" + nfe;
      System.err.println(_status);
      return;
    }
    //++Needed for authentication this one should look
    //++for all of the exceptions instead of the general one
    catch (Exception e) {
      _status = "MessagingServer:processRequest:" + e;
      System.err.println(_status);
      return;
    }
    _status = null;
    
  }
  
  /**
  Deletes the old message handler if it exists and create a new
  one with the updated config files.
  */
  private void loadHandler() {

    _msg_handler = new MessageHandler(_constants);
    _msg_handler.loadConfigFiles();
    _reload_handler = false;
  }


  /**
  Set the reload handler flag to true. This function will force the message
  handler to be reloaded.
  */
  public void setReloadHandler() {
    _reload_handler = true;
    _status = "Reloading handler.";
  }

  /**
  Returns the value of _running
  */
  public boolean isRunning() { return _running; }

  /**
  Sets the _running boolean value to false so the loop will exit.
  */
  public void setRunning(boolean running) {
    if(running) {
      if(!this._running) {
//        System.out.println("Will now start messaging server..");
        this._running = true;
        _wrapper_thread = new Thread(this);
        _wrapper_thread.start();
      }
    }
    else {
      this._running = running;
    }
  }


	/**
	Returns the current value of _status.
	*/
  public String getStatus() { return _status; }

  /**
  The method Thread.start() should not be called.  Please use
  setRunning(true).
  Upon starting, this method will also set _status to null.
  */
  public void run() {

    _reload_handler = true;
    _status = null;
    try {
      _server_socket = new ServerSocket(_port);
    }
    catch(IOException e) {
      _status = "Could not create ServerSocket: " + e;
      System.err.println("MessagingServer:run: " + _status);
      _running = false;
    }

    try {
      while (_running) {
        Socket client_socket = null;
		/*
		wait to accept socket connections
		*/
 //       System.out.println("MessagingServer waiting for clients...");
        client_socket = _server_socket.accept();

        if (_reload_handler) {
          loadHandler();
          _status = null;
        }
		/*
		Process the request into an message object.
		*/
        if (_running) {
	     this.processRequest(client_socket);
        }
        else {
          client_socket.close();
        }
      }
      _server_socket.close();
    }
    catch(java.net.SocketException e) {
      _status = "Socket is still pausing after closing.  Please wait.";
      System.err.println("MessagingServer:run: " + e);
    }
    catch (IOException ioe) {
      _status = "Error:" + ioe;
      System.err.println("MessagingServer:run: " + ioe);
    }
    catch (NullPointerException n) {
      _status = "Error: " + n;
      System.err.println("MessagingServer:run: " + n);
    }
    _running = false;
  }
}
