/**
PeerServer.java
Joel Marlow
Fall 1998
Hummingbird Project - Dr. Frincke
Coded by: Joel Marlow
Adapted from: PeerServer.C and PeerServer.h
Description: This is the implementation of the peer moderation server.
             The server waits for peers to connect (subscribe), grants
	     or denies a subscription, and moves to a message handling
	     state if appropriate.

 Copyright (C) 1997-1999 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*****************************************************************/


package edu.uidaho.hummer.server;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;
import java.util.StringTokenizer;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.*;


/**
The PeerServer class is the main access point for peer subscribing, peer unsubscribing
and peer messaging processing in Hummer. It connects to the socket specified by the 
constructor and waits for messages. When it receives a connection it uses the procRequest
method. Which will figure out what to do with the information received from the socket.   

@author Joel Marlow, Jesse C. McConnel
*/

public class PeerServer implements Runnable {
  /**
  The server socket for the PeerServer.  This may not need to be a class
  variable.
  */
  private ServerSocket _server_socket;
  /**
  int representing the port number the MessagingServer should be running on.
  The value of this int is set in the Hummer.properties file and should only be
  set via the _constants Properties object.
  */
  private int _port;
  /**
  Boolean value representing whether or not the MessagingServer should listen
  for additional connections.
  */
  private boolean _running = false;

  /**
  Thread used to wrap around this Runnable object.  This is done
  rather than having Messaging server extends Thread because once
  a thread completes the run() method it is considered dead...forever.
  So a new thread must be created each time that the server is started up
  again.
  */
  private Thread _wrapper_thread;

  /**
  Information about the current state of messaging server.  This will
  usually hold error information.  If MessagingServer is in a normal state,
  this will be null.
  */
  private String _status = null;

  /**
  Boolean value representing whether or not information needs to be forwarded
  to Peers
  */
  private boolean _push_peer_info = false;
  /**
  System Properties object that when created contains the System.getProperties
  and gets the properties defined in Hummer.properties.
  */
  private Properties _constants;;
  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _sql_url = new String();
  /**
  Database Driver which is taken from the corresponding _constants property.
  */
  private String _sql_driver = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _sql_user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  private String _sql_pwd = new String();
  //++Needed for authentication
  HummerServerSocket _hummer_socket;
  //++Needed for authentication
  HummerKeyStore _key_store;

 
  /**
  Creates the PeerServer using the default port specified in the
  Hummer properties file.  The properties files is passed in during 
  construction of this object.

  @param constants A Hummer properties file being passed in which removes
  the extra construction time for Properties objects.
  */
  public PeerServer(Properties constants) {
    
    _constants = constants;
    String address = new String();

  //++This try and catch is needed for authentication
    try {
      address = InetAddress.getLocalHost().getHostAddress();
      _key_store = HummerKeyStore.getInstance();
      _key_store.load(true, address);
    }
    catch (Exception e) {
      _status = "Error: " + e;
      System.err.println("PeerServer: " + e);
    }

    try {
      address = InetAddress.getLocalHost().getHostAddress();
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
        _sql_url = _constants.getProperty("jdbc.url." + address) + nametoken.nextToken();
      }
      else {
        _sql_url = _constants.getProperty("jdbc.url") + nametoken.nextToken();
      }
      
      if (_constants.containsKey("jdbc.driver." + address)) {
        _sql_driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
        _sql_driver = _constants.getProperty("jdbc.driver");
      }
      
      if (_constants.containsKey("jdbc.user." + address)) {
        _sql_user = _constants.getProperty("jdbc.user." + address);
      }
      else {
        _sql_user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
        _sql_pwd = _constants.getProperty("jdbc.password." + address);
      }
      else {
        _sql_pwd = _constants.getProperty("jdbc.password");
      }
    }
    catch (UnknownHostException uhe) {
      _status = "Error: " + uhe;
      System.err.println("PeerServer: " + uhe);
    }
    
    try {
      Class.forName(_sql_driver);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("PeerServer:" + cnfe);
      _status = "Error: " + cnfe;
    }
    try{
      Integer value = new Integer(_constants.getProperty("port.PEER_PORT"));
      _port = value.intValue();
    }
    catch (NumberFormatException ne) {
      System.err.println("PeerServer: Bad port.PEER_PORT property.");
      _status = "Error: " + ne;
    }

  }



  /**
  Creates the PeerServer using the default port specified in the
  Hummer properties file.  It first initializes this Properties object and
  then grabs the port the PeerServer is to bind to.
  */
  public PeerServer() {


    try {
      _constants = StaticMethods.getHummerProperties();
      Integer value = new Integer(_constants.getProperty("port.PEER_PORT"));
      _port = value.intValue();
     
     }
    catch (FileNotFoundException fnfe) {
      System.err.println("PeerServer: Hummer.properties file not found: " + fnfe);
      _status = "Error: Hummer.properties file not found.";
    }
    catch (IOException ioe) {
      System.err.println("PeerServer: File Hummer.properties not valid.");
      _status = "Error: File Hummer.properties not valid.";
    }
    catch (NumberFormatException ne) {
      System.err.println("PeerServer: Bad port.PEER_PORT property.");
      _status = "Error: Bad port.PEER_PORT property.";
    }
  }


/**
Creates the MessagingServer using the port specified in the call.

@deprecated Replaced by MessagingServer().  Do not use this constructor.
*/

  public void PeerServer(int new_port) {
    _port = new_port;
    // Create the sockets and initialize.
  }


  /**
  This is the place the PeerServer spends its time until it receives a 
  connection. After initializing the server socket the execution moves into
  a period of looping, waiting for socket connections.  Access to this loop
  is governed by a boolean value that can be set to end the loop by calling
  the method killMessagingServer().  This is where the thread starts.
  *
  public void startServer() {
  } //startServer


  /**
  This method is called after the peers are updated to reset the 
  _push_peer_info variable.
  */ 
  public void pushToPeers() {
    _push_peer_info = false;  
  }

  /**
  This method sets the stage for forcing the rest of the Peers in a group
  to update their information.     
  */
  public void setPushPeer() {
    _push_peer_info = true;
  }


  /**
  This function receives a connection from PeerI, a peer seeking to
  subscribe, unsubscribe or send a message to a peer. It grants/  denies
  the action based on table info in the "peers" table.
  <p>
  Database Call:
  <pre>
  SELECT * FROM peers WHERE pgroup = '" + pg_name +"' AND mem_addr = '"+
   peer_name +"';
  </pre>

  @param peer_addr The accepted socket from the connected client that will
  be used to send information to the client.
  @return Boolean value representing successful completion of the request.
  */
  public boolean processRequest(Socket peer_addr) {
    
    Peer peer_obj;
    String peer_name;

    //Old Variables
    Socket s = null;
    String query  = new String();
    String message  = new String();
    String pg_name  = new String();
    String peer_dir  = new String();
    int success;
    String get_input = new String();
    boolean found_group = false;
    int state;
    Message peer_message = new Message();
    String in_peer = new String("");
    String address = null;
    //++Needed for authentication
    Response response;

    try {

      peer_name = (peer_addr.getInetAddress()).getHostAddress();
      //      System.out.println(peer_name);    
      //++Needed for authentication
      //      System.out.println("has validated:" + _hummer_socket.hasValidated());
      response = _hummer_socket.readResponseObject();
      if (response._type == Response._EOT) {
        try {
          address = InetAddress.getLocalHost().getHostAddress();
          _key_store = HummerKeyStore.getInstance();
          _key_store.load(true, address);
          _hummer_socket.close();
        }
        catch (Exception e) {
          _status = "Error: " + e;
          System.err.println("PeerServer:ProcessRequest: " + e);
        }
      }
      else {
        //++Needed for authentication
        //Read in the Peer Object off of the stream
        peer_obj = (Peer)response._message;

        //Increase Message Age
        int i = peer_obj.message.message_age.intValue() + 1;
        peer_obj.message.message_age.valueOf(String.valueOf(i));

      
        //This next section is getting the info from the database
        Connection db_connection = DriverManager.getConnection(
				  _sql_url, _sql_user, _sql_pwd);
        Statement statement = db_connection.createStatement();
          
         //        System.out.println(peer_obj.peer_group);
        // See if peer group name AND peer address are in our table
        query = "SELECT * FROM peers WHERE pgroup = '" + peer_obj.peer_group 
	+ "' AND mem_addr = '"+ peer_name +"';";

        statement.executeQuery(query);

        ResultSet result_set = statement.getResultSet();

        if (result_set.next()) {
          //Peer group name exists in table with peer address, continue 
          // connection and update peer
          in_peer = result_set.getString("pgroup");
        }
        result_set.close();
        statement.close();
        db_connection.close();
      
        if (!in_peer.equals("")) {
          //if peer wishes to subscribe, do this
          if (peer_obj.action.equals("subscribe")) {
            subscribePeer(peer_addr, peer_obj);
          }
          // if peer wishes to unsubscribe, do this
          else if (peer_obj.action.equals("unsubscribe")) {
            unsubscribePeer(peer_addr, peer_obj);
          }
          // if peer wishes to send a message, do this
          else if (peer_obj.action.equals("message")) {  
            processMessage(peer_obj);
          }
          else {
            //++Needed for authentication
            _hummer_socket.close();
            return false;
          }
       }  //if (in_peer != null) 
       else {
         peer_obj.action = "failed";
         //++Needed for authentication
         response = new Response();
         //++Needed for authentication
         response._type= Response._OK;
         //++Needed for authentication
         response._message = peer_obj;
         //++Needed for authentication
         //      System.out.println("has validated:" + _hummer_socket.hasValidated());
         _hummer_socket.writeResponseObject(response);
       }
       // done processing requests and closing out the sockets there may be a couple more spots
       // that these sockets need to be closed.
       //++Needed for authentication
       _hummer_socket.close();
       return true;   
      }
    }
    catch (SQLException se) {
      System.err.println("PeerServer:processRequest:" + se);
      _status = "Error: " + se;
      return false;
    }
    catch (IOException ioe) { 
      System.err.println("PeerServer:processRequest:"+ ioe); 
      _status = "Error: " + ioe;
    } 
    catch (Exception e) {
      //      System.out.println("second");
      System.err.println("PeerServer:processRequest:" + e);
      _status = "Error: " + e;
      //      e.printStackTrace();        
    }
    return true;
  }


  /**
  This method allows for the subscription to a peergroup moderated by this
  hummer.

  <p>
  Database Call:
  <pre>
   1 - UPDATE peers SET peer_sub = 1, peer_mod=0 WHERE pgroup = '" +
       pg_name + "' AND mem_addr ='" + peer_name + "';
   2 - SELECT * FROM peers WHERE pgroup='" + pg_name + "';
   3 - SELECT * FROM pgroups WHERE pgroup='" + pg_name + "';
  </pre>

  @param peer_addr The accepted socket from the connected client that will
  be used to send information to the client.
  @param peer_obj This is the Peer object that has been sent to the PeerServer.
  */
  public boolean subscribePeer(Socket peer_addr, Peer peer_obj) {
    String update = new String();
    String new_pgroup = new String();
    String new_mem_addr = new String();
    String new_peer_sub = new String();
    String new_peer_mod = new String();
    String new_mod_addr = new String();
    String new_email = new String();
    String peer_host;
    //++Needed for authentication
    Response response;

    try {
	
      peer_host = (peer_addr.getInetAddress()).getHostAddress();
      
      //      System.out.println(peer_host);

      //This next section is getting the info from the database
      Connection db_connection = DriverManager.getConnection(
                                  _sql_url, _sql_user, _sql_pwd);
      Statement statement = db_connection.createStatement();
      
      // Update our table to indicate that this peer is subscribed to our group
      update = "UPDATE peers SET peer_sub = 1, peer_mod=0 WHERE pgroup = '" 
	+ peer_obj.peer_group + "' AND mem_addr ='" + peer_host + "';"; 

      statement.executeUpdate(update);

      statement.close();

      // Get the peers with our subscriber's group in common, and send 
      //  them to the peer
      String query = "SELECT * FROM peers WHERE pgroup='" + 
	peer_obj.peer_group + "';"; 

      // Perform the query, quit if select doesn't go through
      statement = db_connection.createStatement();
      statement.executeQuery(query);
      ResultSet result_set = statement.getResultSet();
      
      while (result_set.next()) {
	//new_pgroup = result_set.getString("pgroup");
	new_mem_addr = result_set.getString("mem_addr");
	new_peer_sub = result_set.getString("peer_sub");
	new_peer_mod = result_set.getString("peer_mod");
	
	// For every row returned create an SQL statement that will insert
	// the row on the server's side DB (for the peers and levels tables)
	peer_obj.statements.addElement("INSERT INTO peers VALUES ( '" + 
				       peer_obj.peer_group + "', '" + 
				       new_mem_addr + 
				       "', " + new_peer_sub + ", " + 
				       new_peer_mod + ");");
	//!! This inserting into the levels was taken out and put into Hui
	// if it is put back in then we need to make sure it checks to see
	// if the level is already in the database before inserting it on the
	// client side. --Joel
	//        peer_obj.statements.addElement("INSERT INTO levels VALUES ('" + 
	//				       new_mem_addr + "','',1,1,1,0);");
      }

      // Get the peers with our subscriber's group in common, and send 
      //  them to the peer
      query = "SELECT * FROM pgroups WHERE pgroup='" + peer_obj.peer_group + 
	"';"; 
      
      // Perform the query, quit if select doesn't go through
      statement.executeQuery(query);
      result_set = statement.getResultSet();
      
      // Tell the peer how many times items it must read into its database 
      //in order to add the peers from the moderator's table     
      if (result_set.next()) {
	new_pgroup = result_set.getString("pgroup");
	new_mod_addr = result_set.getString("mod_addr");
	new_email = result_set.getString("email");
	
	// For every row returned create an SQL statement that will insert
	// the row on the server's side DB (for the peers and levels tables)
	peer_obj.statements.addElement("INSERT INTO pgroups VALUES ( '" + 
				       new_pgroup + "', '" + new_mod_addr + 
				       "', '" + new_email + "');");
	//!! This deleting from the levels was taken out because we need to have 
	// the client check things on its side.  I dont think we want the
	// levels to be deleted here anyway (it is sending this info back
	// to PeerI) Also I am not sure why it was deleting the host of the 
	// peer group from the levels database, perhaps this was suppose
	// to be insert--Joel
	//	peer_obj.statements.addElement("DELETE FROM levels WHERE start_addr='" 
	//				       + peer_host + "';"); 
      }
	
      try {
        //Setup the peer object for retransmission
        peer_obj.action = "subscribed";
        //++Needed for authentication
        response = new Response();
        //++Needed for authentication
        response._type= Response._OK;
        //++Needed for authentication
        response._message = peer_obj;
        //++Needed for authentication
        //      System.out.println("has validated:" + _hummer_socket.hasValidated());
        _hummer_socket.writeResponseObject(response);
        //++Needed for authentication
        _hummer_socket.close();
      }
      catch (IOException ioe) {
        System.err.println ("PeerServer:subscribePeer:" + ioe); 
        _status = "Error: " + ioe;
        return false;
      }
      //++Needed for authentication this one should look
      //++for all of the exceptions instead of the general one
      catch (Exception e) {
        //      System.out.println("second");
        System.err.println("PeerServer:subscribePeer:" + e);
        _status = "Error: " + e;
        //      e.printStackTrace();
      }
      
      //closing the database connection
      result_set.close();
      statement.close();
      db_connection.close();
      return true;
    }
    catch (SQLException se) {
      System.err.println("PeerServer:subscribePeer:" + se);
      _status = "Error: " + se;
      return false;
    }
  }
  

  /**
  This method allows for the unsubscription from a peergroup moderated by
  the this hummer.

  <p>
  Database Call:
  <pre>
  Delete FROM peers Where pgroup = '" + pg_name + "' AND mem_addr = '" +
  peer_name + "';
  </pre>

  @param peer_addr The accepted socket from the connected client that will
  be used to send information to the client.
  @param peer_obj This is the Peer object that has been sent to the PeerServer.
  @return Boolean value representing success of the unsubscribe process.
  */
  public boolean unsubscribePeer(Socket peer_addr, Peer peer_obj) {

    String update = new String();
    int success = 0;
    String peer_name;
    //++Needed for authentication
    Response response;

    try {

      peer_name = (peer_addr.getInetAddress()).getHostAddress();      

      //This next section is getting the info from the database
      Connection db_connection = DriverManager.getConnection(
				  _sql_url, _sql_user, _sql_pwd);
      Statement statement = db_connection.createStatement();
      

      update = "Delete FROM peers Where pgroup = '" + peer_obj.peer_group + 
	"' AND mem_addr = '" + peer_name + "';";
      
      success = statement.executeUpdate(update);
      statement.close();
      db_connection.close();

      if (success == 1) {

	try {
	  //Configure the initial unsubscribe peer object
	  peer_obj.action = "unsubscribed";
	  //++Needed for authentication
	  response = new Response();
	  //++Needed for authentication
	  response._type= Response._OK;
	  //++Needed for authentication
	  response._message = peer_obj;
	  //++Needed for authentication
	  //      System.out.println("has validated:" + _hummer_socket.hasValidated());
	  _hummer_socket.writeResponseObject(response);
	  //++Needed for authentication
	  _hummer_socket.close();
	  return true;
	}
	catch (IOException ioe) {
	  System.err.println ("PeerServer.peerUnsubscribe:" + ioe); 
      _status = "Error: " + ioe;
	  return false;
	}
	//++Needed for authentication this one should look
	//++for all of the exceptions instead of the general one
	catch (Exception e) {
	  //      System.out.println("second");
	  System.err.println("DummyServer:processRequest:" + e);
	  //      e.printStackTrace();
      _status = "Error: " + e;
	  return false;
	}
      }
      else {
	try {
	  
	  //Configure the initial unsubscribe peer object
	  peer_obj.action = "error";
	  //++Needed for authentication
	  response = new Response();
	  //++Needed for authentication
	  response._type= Response._OK;
	  //++Needed for authentication
	  response._message = peer_obj;
	  //++Needed for authentication
	  //      System.out.println("has validated:" + _hummer_socket.hasValidated());
	  _hummer_socket.writeResponseObject(response);
	  //++Needed for authentication
	  _hummer_socket.close();

	  return false;
	}
	catch (IOException ioe) {
	  System.err.println ("PeerServer:unsubscribePeer:" + ioe); 
      _status = "Error: " + ioe;
	  return false;
	}
	//++Needed for authentication this one should look
	//++for all of the exceptions instead of the general one
	catch (Exception e) {
	  //      System.out.println("second");
	  System.err.println("DummyServer:processRequest:" + e);
	  //      e.printStackTrace();
      _status = "Error: " + e;
	  return false;
	}
      }
    }
    catch (SQLException se) {
      System.err.println("PeerServer:unsubscribePeer:" + se);
      _status = "Error: " + se;
      return false;
    }
  }    

  


  /**
  This method allows messages to be sent to the members of the peer group.
  The message request can come from PeerI or the MessageHandler.

  <p>
  Database Call:
  <pre>
   SELECT mem_addr FROM peers WHERE pgroup = '" + pg_name + "' and peer_sub
   = 1;
  </pre>
  @param peer_obj This is the Peer object that has been sent to the PeerServer.
  @return Boolean representing the successful completion of the processing.
  */
  public boolean processMessage(Peer peer_obj) {

    int state = 0;
    String new_mod_addr = new String();
    String out_message = new String();
    String query = new String();
    Vector mod_list = new Vector();
    Enumeration enum;
    String send_IP = new String();
    //++Needed for authentication
    PersonalConfig personal;
    //++Needed for authentication
    Response response = new Response();
    //++Needed for authentication
    HummerClientSocket dcs = null;
    String address = new String();
     
    // Message aging still a problem this is not an ideal world here.
    // Someday we might want to use a Message Digest on Hummer Messages
    // and keep a list to check against.
    Integer max_age = new Integer(_constants.getProperty(
				  "const.MAX_MESSAGE_AGE"));

    if (peer_obj.message.message_age.intValue() < (max_age.intValue())) {

      try {
	
	//This next section is getting the info from the database
	Connection db_connection = DriverManager.getConnection(
                                    _sql_url, _sql_user, _sql_pwd);
	Statement statement = db_connection.createStatement();
	
	// Get all addresses of subscribed hosts, perform query, 
	// die if unsuccessful
	query = "SELECT mem_addr FROM peers WHERE pgroup = '" + 
	  peer_obj.peer_group + "' and peer_sub = 1;";
	
	// Perform the query, quit if select doesn't go through
	statement.executeQuery(query);
	ResultSet result_set = statement.getResultSet();
	
	while (result_set.next()) {
	  new_mod_addr = result_set.getString("mem_addr");
	  mod_list.addElement(new_mod_addr);
	}
	result_set.close();
	statement.close();
	db_connection.close();
	enum = mod_list.elements();        
	
	//Forward message to those interested.
	while (enum.hasMoreElements()) {
	  
	  try {
	    Socket to_peer;
	    
	    InetAddress peer_addr = InetAddress.getByName(
				     (String)enum.nextElement());
	    	  
	    to_peer = new Socket(peer_addr, Integer.parseInt(
					    _constants.getProperty(
					    "port.MESSAGE_PORT")));
	    
	    address = InetAddress.getLocalHost().getHostAddress();
	    //++Needed for authentication
	    personal = new PersonalConfig(address, null);
	    dcs = new HummerClientSocket(personal, to_peer, _key_store);
	    dcs.validation(null);
	    //++Needed for authentication
	    response._type = Response._OK;
	    //++Needed for authentication
	    response._message = peer_obj.message;
	    
	    dcs.writeResponseObject(response);
	    dcs.close();
	  }
	  catch (ConnectException ce) {
	    System.err.println("PeerServer:peerMessage:Not a Problem:" + ce);
        _status = "Error: " + ce;
	  }
	  catch (IOException ioe) {
	    System.err.println("PeerServer:peerMessage: " + ioe);
         _status = "Error: " + ioe;
	  }
	  catch (Exception e) {
	    System.err.println("PeerServer:peerMessage: " + e);
         _status = "Error: " + e;
	  }
	  
	  
	}
      }
      catch (SQLException se) {
        System.err.println("PeerServer:peerMessage:" + se);
        _status = "Error: " + se;
        return false;
      }  
      
      return true;
    }

    return true;
  }

  /**
  Returns the value of _running
  */
  public boolean isRunning() { return _running; }

  /**
  Sets the _running boolean value to false so the loop will exit.
  */
  public void setRunning(boolean running) {
    if(running) {
      if(!this._running) {
        this._running = true;
        _wrapper_thread = new Thread(this);
        _wrapper_thread.start();
      }
    }
    else {
      this._running = running;
    }
  }


	/**
	Returns the current value of _status.
	*/
  public String getStatus() { return _status; }



  /**
  The method startServer() is called when the thread is started by the
  call to the extended Thread.start().
  */
  public void run() {

    _status = null;

    try {
      _server_socket = new ServerSocket(_port);
    }
    catch (IOException e) {
      _status = "Error: " + e;
      System.err.println("PeerServer:run:Could not create the server:" + e);
      _running = false;
    }

    String peer_addr = new String();  
    
    pushToPeers();

    try {
      while (_running) { 
        Socket client_socket = null;
		// wait to accept socket connections
//        System.out.println ("PeerServer waiting for clients...");
        client_socket = _server_socket.accept();
        
        String address = InetAddress.getLocalHost().getHostAddress();
		//++Needed for authentication
        _hummer_socket = new HummerServerSocket(null, client_socket, _key_store);
		//++Needed for authentication
        _hummer_socket.validation(null); 
	
		//Checks to see if we are still to be running and the state of the
		//server hasn't changed.	
        if (_running) {
          this.processRequest(client_socket);
        }

        client_socket.close();


		// Has the alarm go off to send config to peers.
        if (_push_peer_info) {
		// Call Push Peer Info method
          pushToPeers();
        }
      } //while  
      _server_socket.close();
    }
    catch(java.net.SocketException e) {
      _status = "Socket is still pausing after closing.  Please wait.";
      System.err.println("PeerServer:run: " + e);
      _running = false;
    }
    catch (IOException ioe) {
      _status = "Error: " + ioe;
      System.err.println ("PeerServer:startServer:" + ioe);
      _running = false;
    }
    catch (NullPointerException n) {
      _status = "Error: " + n;
      System.err.println("PeerServer:run:" + n);
      _running = false;
    }
		//++Needed for authentication this one should look
		//++for all of the exceptions instead of the general one
    catch(Exception e) {
      _status = "Error: " + e;
      System.err.println("PeerServer:run:" + e);
      e.printStackTrace();
      _running = false;
    }
  }
} 
