/*
Java Hummer Project
File: ToolHandler.java
Date: 4-19-99
Author: Chad Ward
Description:  Client handler for handling the requests of
              starting and stopping tools on the system.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.io.OptionalDataException;
import java.io.IOException;
import java.util.Hashtable;
import java.net.Socket;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.tool.HummerTool;
import edu.uidaho.hummer.tool.ToolGroup;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;

/**
Client handler for handling the requests of
starting and stopping tools on the system.
@author Chad Ward
*/
public final class ToolHandler implements Runnable {
	/**
	HummerSocket used for communication with the client.
	*/
  private HummerServerSocket _hummer_socket;

	/**
	Group of tools that are currently running on the system.
	*/
  private ToolGroup _tool_group;

	/**
	Information about the current state of messaging server.  This will
	usually hold error information.  If MessagingServer is in a normal state,
	this will be null.
	*/
  private String _status = null;

	/**
	True if the current client is running, false otherwise.
	*/
  private boolean _running = false;

	/**
	Index + 1 into _OPTION_STRINGS
	*/
  public final int _START_ALL = 1,
                   _STOP_ALL = 2,
                   _START_SINGLE = 3,
                   _STOP_SINGLE = 4,
                   _STATUS_ALL = 5;


	/**
	Array of option explanations for tool manipulation
	*/
  public final String[] _OPTION_STRINGS = {
                                             "Start all Tools",
                                             "Stop all Tools",
                                             "Start 1 Tool (eg. " + _START_SINGLE + " bdf)",
                                             "Stop 1 Tool (eg. " + _STOP_SINGLE + " bdf)",
                                             "Tool Status"
                                           };
	/**
	Constructor which creates a HummerServerSocket with the socket and key store, and
	sets the local ToolGroup variable.
	*/
  public ToolHandler(ToolGroup tool_group, HummerKeyStore key_store, Socket socket) {
    if(tool_group == null) {
      System.out.println("ToolHandler:init: WARNING: ToolGroup object recieved is null.");
    }
    try {
      this._hummer_socket = new HummerServerSocket(null, socket, key_store);
    }
    catch(HummerSocketException e) {
      System.err.println("ToolHandler:constructor: Error while creating HummerSocket: " + e);
      return;
    }
    catch(KeyStoreException e) {
      System.err.println("ToolHandler:constructor: Error while creating HummerSocket: " + e);
      return;
    }
    catch(IOException e) {
      System.err.println("ToolHandler:constructor: Error while creating HummerSocket: " + e);
      return;
    }
    catch(NoSuchAlgorithmException e) {
      System.err.println("ToolHandler:constructor: Error while creating HummerSocket: " + e);
      return;
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("ToolHandler:constructor: Error while creating HummerSocket: " + e);
      return;
    }
    this._tool_group = tool_group;
  }

	/**
	Returns the value of _running.
	@see ToolServer#_running
	*/
  public boolean isRunning() {  return _running; }

	/**
	Sets the value of _running to false (stopping all processing).
	*/
  public void stop() {
    this._running = false;
  }
  
  public void run() {
    try {
      Response input, output;
      System.out.print("ToolHandler: Validating user...");
      _hummer_socket.validation(null);
      System.out.println(_hummer_socket.getCurrentPartnerAlias() + ".");
      Hashtable client_access_table = null;
      Boolean can_control_tools = null;
      if(!_hummer_socket.getCurrentPartnerAlias().equals("admin")) {
        client_access_table = _hummer_socket.getCurrentClientAccessTable();
        can_control_tools = (Boolean)client_access_table.get("Control_Tools");
      }
      if(!_hummer_socket.getCurrentPartnerAlias().equals("admin") &&
         !can_control_tools.booleanValue()) {
        HummerSocketException exception;
        String error_message = "ToolHandler:run: Alias does not have access to control " +
                               "tools on this system.";
        exception = new HummerSocketException(error_message, null, 0);
        System.err.println(error_message);
        _hummer_socket.writeException(exception);
        _running = false;
        return;
      }
      output = new Response();
      output._message = _OPTION_STRINGS;
      _hummer_socket.writeResponseObject(output);
      input = _hummer_socket.readResponseObject();
      while(input._type != Response._EOT) {
        output = getSelectionResponse(input);
        _hummer_socket.writeResponseObject(output);
        input = _hummer_socket.readResponseObject();
      }
    }
    catch(KeyStoreException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(OptionalDataException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(IOException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(CertificateException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(UnknownAliasException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(UnloadedKeyStoreException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(UnrecoverableKeyException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    catch(HummerSocketException e) {
      _status = "Error: " + e;
      System.err.println("ToolHandler:run: " + e);
    }
    try {
      _hummer_socket.close();
    }
    catch(IOException e) {
      _status = "Error while closing socket: " + e;
      System.err.println("ToolHandler:run: Could not close socket:" + e);
    }
  }

	/**
	Parses the input recieved from a client
	*/
  public final Response getSelectionResponse(Response input) throws HummerSocketException {
    int selection = ((Integer)input._message).intValue();
    Response output = new Response();
    if(_tool_group == null) {
      System.err.println("ToolHandler:getSelectionrResponse: " +
                         "ToolGroup object has not been initialized.");
      System.exit(-1);
    }
    switch(selection) {
      case _START_ALL:
        _tool_group.setToolsRunning(true);
        output._data = _tool_group.getToolStatus();
        break;
      case _STOP_ALL:
        _tool_group.setToolsRunning(false);
        output._data = _tool_group.getToolStatus();
        break;
      case _START_SINGLE:
        _tool_group.setToolRunning(true, input._data.get(0).toString());
        output._data = _tool_group.getToolStatus();
        break;
      case _STOP_SINGLE:
        _tool_group.setToolRunning(true, input._data.get(0).toString());
        output._data = _tool_group.getToolStatus();
        break;
      case _STATUS_ALL:
        output._data = _tool_group.getToolStatus();
        break;
      default:
        _hummer_socket.throwException("Invalid selection sent from client",
                                      input, HummerSocketException._INVALID_COMMAND);
    }
    return output;
  }
}