/*
Java Hummer Project
File: HummerServerSocketTest.java
Date: 3-7-99
Author: Chad Ward
Description:  Tests HummerServerSocket class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.util.socket;

import java.security.*;
import java.util.*;
import java.io.*;
import java.net.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.DefaultKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;
import edu.uidaho.hummer.test.Test;

/**
Description:  Tests HummerServerSocket class. <Br>
Most of the members of this class are static because there will be a thread
for the server and a thread for the client, but the information between the two
must be the same (socket information and key store information primarily). <br>
	The order of testing for validation method is:
	<ol type="I">
	  <li> Remote Account
	  <ol type="1">
	    <li> Null Response object
	    <li> Null Response._data object
	    <li> Response._data.size() is too small (must be at least 4)
	    <li> Response._data.size() is too large with non-null signer_alias (Must be exactly 4)
	    <li> _data[0] is null (protocol name)
	    <li> _data[1] is null (client alias)
	    <li> _data[3] is null (client HummerCertificate)
	    <li> Invalid protocol
	    <li> Unknown signer_alias
	    <li> Expired HummerCertificate
	    <li> Invalid certificate with valid signer_alias
	    <li> Valid certificate, but for untrusted server
	    <li> Valid information
	  </ol>
	  <li> Local Account
	  <ol type="1">
	    <li> Valid information
	  </ol>
	  <li> Update Server KeyPair
	  <ol type="1">
	    <li> Remote Account
	    <li> Remote Account (Certificate Signer update KeyPair)
	    <li> Local Account
	  </ol>
	</ol>
	The order of testing for newAccountValidation is:
	<ol type="1">
	  <li> Response._data.size() is too small (must be at exactly 6)
	  <li> Response._data.size() is too large (must be at exactly 6)
	  <li> Null PublicKey for client
	  <li> Null message digest
	  <li> Null random value
	  <li> Null timestamp
	  <li> Unknown alias
	  <li> Invalid message digest
	</ol>
	The order of testing for writeResponseObject is:
	<ol type="1">
	  <li> Attempt to write object before validation has been called
	  <li> Normal operation (check for validity of data sent back)
	</ol>
	The order of testing for readResponseObject is:
	<ol type="1">
	  <li> Invalid signature.
	  <li> Valid signature. Wrong id.
	  <li> Valid signature. _type is invalid 
	  <li> Valid signature. _type is valid (no commands)
	</ol>
	@see edu.uidaho.hummer.util.socket.HummerServerSocket
	@author Chad Ward
*/
public class HummerServerSocketTest extends Test implements Runnable {

	/**
	A common InetAddress that will be used for communication during this test.
	It is static because multiple instances of this class will be created.
	The server is the first instance, and a second instance is created and placed
	in a thread to represent a client.
	*/
  protected static InetAddress _inet_address;
	/**
	A common port number that will be used for communication during this test.
	It is static because multiple instances of this class will be created.
	The server is the first instance, and a second instance is created and placed
	in a thread to represent a client.
	*/
  protected static int _port_number;

	/**
	A common set of signature keys will be required to do any sort of testsing,
	so this object must be static.
	*/
  protected static DefaultKeyStore _key_store;

	/**
	Hashtable of access rights for an alias.
	*/
  protected static Hashtable _local_user_access_table = new Hashtable(),
                             _new_local_user_access_table = new Hashtable(),
                             _remote_user_access_table = new Hashtable(),
                             _untrusted_server_access_table = new Hashtable(),
                             _trusted_server_access_table = new Hashtable();

	/**
	Signatures keys
	*/
  protected static KeyPair _local_user_key_pair, _new_local_user_key_pair, _remote_user_key_pair,
                           _untrusted_server_key_pair, _trusted_server_key_pair,
                           _key_store_key_pair1, _key_store_key_pair2;

	/**
	Signature object used for signing or verifying data.
	*/
  protected static Signature _trusted_server_verify, _trusted_server_sign,
                             _untrusted_server_verify, _untrusted_server_sign,
                             _key_store_verify1, _key_store_sign1,
                             _key_store_verify2, _key_store_sign2,
                             _local_user_sign, _remote_user_sign;

  protected long _input_message_id = StaticMethods.getUniqueTimestamp();


	/**
	Certificate granted by a local server alias for local_user.
	*/
  protected static HummerCertificate _local_user_certificate;

	/**
	Certificate granted by a trusted server alias for remote_user.
	*/
  protected static HummerCertificate _trusted_remote_user_certificate;

	/**
	Certificate granted by an untrusted server alias for remote_user.
	*/
  protected static HummerCertificate _untrusted_remote_user_certificate;

	/**
	Response object sent to the server during testing of readResponseObject
	*/
  protected static Response _client_message;

	/**
	String representing the password
	*/
  protected final String _new_local_user_pass = "new_local_user_pass",
                         _untrusted_server_pass = "untrusted_server_pass",
                         _trusted_server_pass = "trusted_server_pass",
                         _local_user_pass = "local_user_pass",
                         _remote_user_pass = "remote_user_pass";

	/**
	Alias of an account that is used for testing.
	*/
  protected String _untrusted_server_alias = "untrusted_server", 
                   _trusted_server_alias = "trusted_server",
                   _local_user_alias = "local_user",
                   _new_local_user_alias = "new_local_user",
                   _remote_user_alias = "remote_user";

	/**
	Allows testing for HummerServerSocket to be run from the command line.
	If arguments are sent, the first is a filename for standard out, and the second is
	the filename for error out (one, two or no parameters will work).
	*/
  public static void main(String[] argv) throws Exception {
    PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new PrintStream(new FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new PrintStream(new FileOutputStream(argv[1]));
    }
    Test test_object = new HummerServerSocketTest(std_out, err_out);
    Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }
	/**
	A seperate constructor was needed so that a client thread would be started,
	and the default is still needed for an uninitialized Test, so this is added.
	*/
  public HummerServerSocketTest() { super(); }

	/**
	Constructor to specify where to place output messages
	*/
  public HummerServerSocketTest(PrintStream std_out, PrintStream err_out) {
    super(std_out, err_out);
  }

	/**
	A constructor is needed to start up a client thread.
	*/
  public HummerServerSocketTest(boolean is_server, PrintStream std_out, PrintStream err_out) {
    super(std_out, err_out);
  }

  protected void setTestClassName() {
    this._test_class_name = "HummerServerSocket";
  }

/****************************************************************************************
 ******************************************
 Begin test definition
 ******************************************
 ***************************************************************************************/


	/**
	Defines the tests run.  Five methods are tested: Constructor (1), validation, newAccountValidation,
	writeResponseObject, and readResponseObject. <br>
	*/
  protected void testDefinition() throws Exception {
    Thread client_thread = new Thread(new HummerServerSocketTest(false, _standard_out, _error_out));
    ServerSocket server_socket = new ServerSocket(0);
    Socket client_handler_socket;
    _port_number = server_socket.getLocalPort();
    _inet_address = InetAddress.getLocalHost();
    PersonalConfig personal_config = new PersonalConfig(_inet_address.getHostAddress(), null);

		/* *********************************
		Constructor testing
		************************************/
    newMethod("Constructor");
    setStaticVariables();

    client_thread.start();

    HummerServerSocket hummer_socket = null;
    client_handler_socket = server_socket.accept();
    hummer_socket = (HummerServerSocket)(runConstructorTest("Acceptable parameters",
                                                     null, false, personal_config,
                                                      client_handler_socket, _key_store)[2]);

    if(hummer_socket == null) {
      return;
    }

    testValidationServer(server_socket, hummer_socket, personal_config);
    testNewAccountValidationServer(server_socket, personal_config);
    testWriteResponseObjectServer(server_socket, personal_config);
    testReadResponseObjectServer(server_socket, personal_config);

    testCommandsServer(server_socket, personal_config);
		/*
		Wait for the client to finish before finsishing up...after
		this method, statistics are tallied.
		*/
    client_thread.join();
  }

	/**
	A thread run to be a client on a socket sending some mostly obnoxious stuff across.
	*/
  public void run() {
    try {

      testValidationClient();
      testNewAccountValidationClient();
      testWriteResponseObjectClient();
      testReadResponseObjectClient();

      testCommandsClient();
    }
    catch(Exception e) {
      System.out.println("HummerServerSocketTest:run(Client):");
      e.printStackTrace();
      System.exit(-1);
    }
  }


/*******************************************************************************************/
/*******************************************************************************************/
	/**
	Tests the validation method
	*/
  private void testValidationClient() throws Exception {
      HummerSignedObject signed_input;
      Response input, output = new Response();
      output._id = _input_message_id;
      KeyPair server_key_pair = _key_store.getPersonalKeyPair();
      HummerCertificate trusted_certificate = _trusted_remote_user_certificate,
                        untrusted_certificate = _untrusted_remote_user_certificate;
      input = null;
      output = null;
      ObjectInputStream in_stream;
      ObjectOutputStream out_stream;
      Socket socket = null;
      if(_inet_address != null) {
        socket = new Socket(_inet_address, _port_number);
      }
      else {
        errorPrintln("DefaulServerSocketTest:run (client): " +
                           "InetAddress.getLocalHost() is returning a null value!");
        throw new NullPointerException();
      }

		/*
		Send a null Response object
		*/
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(null);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());

      runSimpleTest("Client:Response object recieved after sending null Response to server",
                    "Reponse._type is _EXCEPTION, exception type is _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Send Response object with null _data Vector
		*/

      output = new Response();
      output._id = _input_message_id;
      output._data = null;

      socket = new Socket(_inet_address, _port_number);

      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending null _data vector",
                    "Reponse._type is _EXCEPTION and exceptions type equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION && 
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));
      
		/*
		Send Response object with _data Vector with not enough elements (minus HummerCertificate)
		*/
      output = new Response();
      output._type = Response._OK;
      output._id = _input_message_id;
      output._message = null;
      output._data.add("default");
      output._data.add(_remote_user_alias);
      output._data.add(_trusted_server_alias);
		/*
		Have not added the certificate, so the server should recognize that the vector is not
		the correct size before continuing.
		*/
      socket = new Socket(_inet_address, _port_number);

      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending _data " +
                    "vector that is too short(missing HummerCertificate)",
                    "Reponse._type is _EXCEPTION and exception type is _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));


		/*
		Send Response object with _data Vector too long
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.add(trusted_certificate);
      output._data.add("Useless string");
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response  recieved after sending _data vector " +
                    "that is too long(5 with non-null signer_alias)",
                    "Reponse._type is _EXCEPTION and exception type equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Remove extra value from _data vector and begin testing for null values in _data Vector
		*/
      output._data.remove(output._data.size() - 1);

		/*
		Send Response with null protocol value in _data Vector
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(0, null);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending null protocol",
                    "Reponse._type is _EXCEPTION and exception equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));


		/*
		Send Response with null alias in _data Vector
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(0, "default");
      output._data.set(1, null);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending null alias",
                    "Reponse._type is _EXCEPTION and exceptions equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));


		/*
		Send Response with null HummerCertificate in _data Vector
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(1, _remote_user_alias);
      output._data.set(3, null);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending null HummerCertificate",
                    "Reponse._type is _EXCEPTION and exception type equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));



		/*
		Send Response object with invalid protocol
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(0, "ENCRYPTED");
      output._data.set(3, trusted_certificate);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending invalid protocol",
                    "Reponse._type is _EXCEPTION and exception type equals _PROTOCOL_NOT_KNOWN, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._PROTOCOL_NOT_KNOWN &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));


		/*
		Send Response object with unknown signer alias
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(0, "default");
      output._data.set(1, _remote_user_alias);
      output._data.set(2, "Unknown_alias");
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending invalid signer alias",
                    "Reponse._type is _EXCEPTION and exception type equals _UNKNOWN_HUMMER_CERTIFICATE_SIGNER, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._UNKNOWN_HUMMER_CERTIFICATE_SIGNER &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));


		/*
		Send Response object with expired HummerCertificate
		*/
      socket = new Socket(_inet_address, _port_number);
      HummerCertificate expired_certificate;
      expired_certificate = new HummerCertificate(StaticMethods.getSigningSignatureObject(_trusted_server_key_pair.getPrivate()),
                                                  _remote_user_key_pair.getPublic(), null,
                                                  new GregorianCalendar());

      output._data.set(0, "default");
      output._data.set(1, _remote_user_alias);
      output._data.set(2, _trusted_server_alias);
      output._data.set(3, expired_certificate);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending expired HummerCertificate",
                    "Reponse._type is _EXCEPTION and exceptions type equals _EXPIRED_HUMMER_CERTIFICATE, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._EXPIRED_HUMMER_CERTIFICATE &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Send Response object with invalid HummerCertificate (trusted server alias)
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(3, untrusted_certificate);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:In Response object recieved after sending " +
                    "invalid HummerCertificate (trusted server alias)",
                    "Reponse._type is _EXCEPTION and exception type equals _INVALID_LOGIN, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_LOGIN &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Send Response object with valid HummerCertificate for untrusted server
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(2, _untrusted_server_alias);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());

      runSimpleTest("Client:In Response object recieved after sending valid HummerCertificate " +
                    "for untrusted server alias",
                    "Reponse._type is _EXCEPTION and exception type _NON_TRUSTED_HUMMER_CERTIFICATE, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._NON_TRUSTED_HUMMER_CERTIFICATE &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Send Response object with valid information
		*/
      socket = new Socket(_inet_address, _port_number);

      output._data.set(2, _trusted_server_alias);
      output._data.set(3, _trusted_remote_user_certificate);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      Object input_object = in_stream.readObject();
		/*
		The object recieved should be a HummerSignedObject because the protocol went through.
		*/
      if(runSimpleTest("Client:Response object is valid (remote account)", "Object recieved is a HummerSignedObject", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        runSimpleTest("Client:Response object is valid (remote account)",
                      "Protocol is \"default\".",
                      input._data.get(0).equals("default"));

        HummerCertificate server_certificate = (HummerCertificate)(input._data.get(1));
        if(server_certificate != null) {
          runSimpleTest("Client:HummerCertificate.isValid with PublicKey of signing trusted server (remote account)",
                        "boolean true",
                        server_certificate.isValid(_trusted_server_verify));
          runSimpleTest("Client:HummerSignedObject.verify with server's PublicKey (remote account)",
                        "boolean true",
                        signed_input.verify(StaticMethods.getVerifyingSignatureObject(
                                                              server_certificate.getPublicKey())));

        }
        else {
          runSimpleTest("Client:HummerCertificate returned (remote account)", "Non-null value", false);
        }
        if(input._message != null) {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "(Long)(Response._message) equals sent id",
                        ((Long)input._message).longValue() == _input_message_id);
        }
        else {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "non-null value (represents client's message id)", false);
        }
        runSimpleTest("Client:Response object recieved in HummerSignedObject",
                      "Reponse._type = _OK", input._type == Response._OK);
        runSimpleTest("Client:Response object recieved in HummerSignedObject (this is a remote account)",
                      "Reponse._data.size() == 2",
                      input._data.size() == 2);
      }
      else {
        if(input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.Response")) {
          println("Client:Object type recieved: Response.\n" +
                  StaticMethods.responseToString((Response)input_object, "\t"), true);
        } else {
          println("Client:Object type recieved: " + input_object.getClass().getName(), true);
        }
      }

		/* ******************************************************
		Test local account
		********************************************************/
      socket = new Socket(_inet_address, _port_number);

      Response output_local = new Response();
      output_local._type = Response._OK;
      output_local._id = _input_message_id;
      output_local._message = null;
      output_local._data.add("default");
      output_local._data.add(_local_user_alias);
      output_local._data.add(_inet_address.getHostAddress());
      output_local._data.add(_local_user_certificate);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output_local);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      if(runSimpleTest("Client:Response object is valid (local account)", "Object recieved is a HummerSignedObject", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        runSimpleTest("Client:Response object is valid (local account)",
                      "Protocol is \"default\".",
                      input._data.get(0).equals("default"));
        HummerCertificate server_certificate = (HummerCertificate)(input._data.get(1));
        Signature signer_verify = _key_store.getPersonalVerifySignature(_local_user_alias);
        if(server_certificate != null) {
          runSimpleTest("Client:HummerCertificate.isValid with PublicKey of signing trusted server (local account)",
                           "boolean true",
                           server_certificate.isValid(signer_verify));
          signer_verify = StaticMethods.getVerifyingSignatureObject(server_certificate.getPublicKey());
          runSimpleTest("Client:HummerSignedObject.verify with server's PublicKey (local account)",
                        "boolean true", signed_input.verify(signer_verify));
        }
        else {
          runSimpleTest("Client:HummerCertificate returned (local account)", "Non-null value", false);
        }
        if(input._message != null) {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "(Long)(Response._message) equals send id",
                        ((Long)input._message).longValue() == _input_message_id);
        }
        else {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "non-null value (represents client's message id)", false);
        }
        runSimpleTest("Client:Response object recieved in HummerSignedObject",
                      "Reponse._type = _OK", input._type == Response._OK);
        runSimpleTest("Client:Response object recieved in HummerSignedObject (this is a local account)",
                      "Reponse._data.size() == 2",
                      input._data.size() == 2);
      }

		/* *********************************************************
		Server has changed key pair (HummerKeyStore.updateKeyPair)
		Verify that remote account does not work any longer
		************************************************************/
      
      socket = new Socket(_inet_address, _port_number);

      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      if(runSimpleTest("Client:Response object is valid (server has updated key pair)",
                       "Object recieved is a HummerSignedObject", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        runSimpleTest("Client:Response object is valid (server has updated key pair)",
                      "Protocol is \"default\".",
                      input._data.get(0).equals("default"));

        Signature signer_verify = _trusted_server_verify;
        HummerCertificate server_certificate = (HummerCertificate)(input._data.get(1));
        if(server_certificate != null) {
          runSimpleTest("Client:HummerCertifcate.isValid with PublicKey of signing " +
                        "trusted server (remote account - after server updated key pair)",
                        "boolean true",
                        server_certificate.isValid(signer_verify));
          signer_verify = StaticMethods.getVerifyingSignatureObject(server_certificate.getPublicKey());
          runSimpleTest("Client:HummerSignedObject.verify with server's PublicKey " +
                        "(remote account - after server updated key pair)",
                        "boolean true", signed_input.verify(signer_verify));
        }
        else {
          runSimpleTest("Client:HummerCertificate returned (remote account after server update KeyPair)",
                        "Non-null value", false);
        }

        if(input._message != null) {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "(Long)(Response._message) equals send id",
                        ((Long)input._message).longValue() == _input_message_id);
        }
        else {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "non-null value (represents client's message id)", false);
        }
        runSimpleTest("Client:Response object recieved in HummerSignedObject",
                      "Reponse._type = _OK", input._type == Response._OK);
        runSimpleTest("Client:Response object recieved in HummerSignedObject (this is a remote account)",
                      "Reponse._data.size() == 2",
                      input._data.size() == 2);
      }


		/* ************************************************************************
		Trusted server has changed its KeyPair in the key store.  Remote account
		should no longer be able to log in
		***************************************************************************/
      socket = new Socket(_inet_address, _port_number);

      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();



      if(runSimpleTest("Client:Response object is valid (trusted server, signer of certificate, has updated key pair)",
                       "Object recieved is a Response object (not HummerSignedObject)", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.Response"))) {
        input = (Response)input_object;
        runSimpleTest("Client:In Response object recieved after sending valid " +
                      "HummerCertificate for untrusted server alias",
                      "Reponse._type is _EXCEPTION and exception type equals _NON_TRUSTED_HUMMER_CERTIFICATE, " +
                      "and Response._message is a HummerKeyStoreException",
                      input._type == Response._EXCEPTION &&
                      ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_LOGIN &&
                      input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));
      }
      else {
        socket.close();
      }

		/****
		Local account after changing KeyPair on server - Unknown alias
		*****/
      socket = new Socket(_inet_address, _port_number);

      output_local._data.set(1, "unknown_alias");
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output_local);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      if(runSimpleTest("Client:Object recieved from server after sent remote user request",
                       "Object.getClass().getName() = edu.uidaho.hummer.dataobject.Response",
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.Response"))) {
        input = (Response)(input_object);
        runSimpleTest("Client:In Response object recieved from server after " +
                      "sending invalid alias information for local account(updated key pair)",
                      "Reponse._type is _EXCEPTION and exception type equals _INVALID_LOGIN, " +
                      "and Response._message is a HummerKeyStoreException",
                      input._type == Response._EXCEPTION &&
                      ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_LOGIN &&
                      input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));
      }
      else {
        input = (Response)(((HummerSignedObject)(input_object)).getObject());
      }

		/*
		Test with correct alias
		*/
      socket = new Socket(_inet_address, _port_number);

      output_local._data.set(1, _local_user_alias);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output_local);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      if(runSimpleTest("Client:Response object is valid (local account-updated key pair)",
                       "Object recieved is a HummerSignedObject", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        runSimpleTest("Client:Response object is valid (local account-updated key pair)",
                      "Protocol is \"default\".",
                      input._data.get(0).equals("default"));
                        
        Signature signer_verify = _key_store.getPersonalVerifySignature(_trusted_server_alias);
        HummerCertificate server_certificate = (HummerCertificate)(input._data.get(1));
        if(server_certificate != null) {
          runSimpleTest("Client:HummerCertificate.isExpired " +
                        "(local account - after server updated key pair)", "boolean false",
                        !server_certificate.isExpired());
          runSimpleTest("Client:HummerCertifcate.isValid with PublicKey of signing " +
                        "trusted server (local account - after server updated key pair)",
                        "boolean true",
                        server_certificate.isValid(signer_verify));
          signer_verify = StaticMethods.getVerifyingSignatureObject(server_certificate.getPublicKey());
          runSimpleTest("Client:HummerSignedObject.verify with server certificate's PublicKey " +
                        "(local account - after server updated key pair)",
                        "boolean true", signed_input.verify(signer_verify));
          runSimpleTest("Client:HummerSignedObject.verify with server's current PublicKey (key store)" +
                        "(local account - after server updated key pair)",
                        "boolean true",
                        signed_input.verify(_key_store.getPersonalVerifySignature()));
        }
        else {
          runSimpleTest("Client:HummerCertificate returned (local account after server updated KeyPair)",
                        "Non-null value", false);
        }

        if(input._message != null) {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "(Long)(Response._message) equals send id",
                        ((Long)input._message).longValue() == _input_message_id);
        }
        else {
          runSimpleTest("Client:Response object _message in HummerSignedObject",
                        "non-null value (represents client's message id)", false);
        }
        runSimpleTest("Client:Response object recieved in HummerSignedObject",
                      "Reponse._type is _OK", input._type == Response._OK);
        if(runSimpleTest("Client:Response object recieved in HummerSignedObject (this is a local account)",
                      "Reponse._data.size() == 3",
                      input._data.size() == 3)) {
          signer_verify = StaticMethods.getVerifyingSignatureObject(server_certificate.getPublicKey());
          runSimpleTest("Client:New HummerCertificate given by server.",
                        "Is valid using the PublicKey in server's HummerCertificate",
                        ((HummerCertificate)(input._data.get(2))).isValid(signer_verify));
          runSimpleTest("Client:New HummerCertificate given by server.",
                        "Is valid using the current PublicKey of server (key store)",
                        ((HummerCertificate)(input._data.get(2))).isValid(_key_store.getPersonalVerifySignature()));
        }
      }

		/* ******************************************
		+
		Test for local server logging into another local server (eg. ConfigServer->PeerServer)
		+
		********************************************/
      socket = new Socket(_inet_address, _port_number);

      output = new Response();
      output._id = _input_message_id;
      output._data.add("default");
      output._data.add(_inet_address.getHostAddress());
      output._data.add(_inet_address.getHostAddress());
      output._data.add(_key_store.getPersonalHummerCertificate());
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      if(runSimpleTest("Client:Response object is valid (local server to local server login)",
                       "Object recieved is a HummerSignedObject", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());

        runSimpleTest("Client:Response object is valid (local server to local server login)",
                      "Response._type equals _OK", input._type == Response._OK);

        runSimpleTest("Client:Response object is valid (local server to local server login)",
                      "Protocol is \"default\".",
                      input._data.get(0).equals("default"));

        runSimpleTest("Client:Response object is valid (local server to local server login)",
                      "Response._data size equals 2", input._data.size() == 2);

        HummerCertificate server_certificate = (HummerCertificate)(input._data.get(1));

        if(runSimpleTest("Client:Response object is valid (local server to local server login)",
                      "HummerCertificate for server is non-null value", server_certificate != null)) {

          runSimpleTest("Client:Response object is valid (local server to local server login)",
                        "HummerCertificate for server is not expired",
                        !server_certificate.isExpired());

          PublicKey server_public_key = server_certificate.getPublicKey();
          runSimpleTest("Client:Response object is valid (local server to local server login)",
                        "PublicKey recieved is equal to PublicKey sent to server",
                        ((HummerCertificate)(output._data.get(3))).getPublicKey().equals(server_public_key));

          runSimpleTest("Client:Response object is valid (local server to local server login)",
                        "PublicKey recieved is equal to PublicKey in HummerKeyStore (shared key store)",
                        _key_store.getPersonalKeyPair().getPublic().equals(server_public_key));


          Signature signer_verify = StaticMethods.getVerifyingSignatureObject(server_public_key);

          runSimpleTest("Client:Response object is valid (local server to local server login)",
                        "HummerCertificate can be verified with given PublicKey",
                        server_certificate.isValid(signer_verify));

          runSimpleTest("Client:Response object is valid (local server to local server login)",
                        "Recieved HummerSignedObject can be verified with given PublicKey",
                         signed_input.verify(signer_verify));
        }

        if(runSimpleTest("Client:Response object is valid (local server to local server login)",
                         "Response._message is not null", input._message != null)) {
          runSimpleTest("Client:Response object is valid (new account)",
                        "(Long)(Response._message) equals sent id",
                        ((Long)input._message).longValue() == _input_message_id);
        }
      }
  }




  private void testWriteResponseObjectServer(ServerSocket server_socket,
                                             PersonalConfig personal_config) throws Exception {
    Socket client_handler_socket;
    HummerServerSocket hummer_socket;
		/* ******************************************************************
		=====================================================================
		Begin testing for writeResponseObject
		=====================================================================
		********************************************************************/
    Thread.sleep(400);
    newMethod("writeResponseObject");
    Response message = new Response();
    message._message = "message";
    message._type = Response._OK;
    message._data.add("Data1");
    message._data.add("Data2");
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server:validation method has not been called yet", hummer_socket, "writeResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true, message);

    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    hummer_socket.validation(null);

    runExceptionTest("Normal operation", hummer_socket, "writeResponseObject", null, false, message);
  }

  private void testReadResponseObjectServer(ServerSocket server_socket,
                                            PersonalConfig personal_config) throws Exception {
    Socket client_handler_socket;
    HummerServerSocket hummer_socket;
		/* ******************************************************************
		=====================================================================
		Begin testing for readResponseObject
		=====================================================================
		********************************************************************/
    Thread.sleep(400);
    newMethod("readResponseObject");
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server:validation method has not been called yet", hummer_socket,
                     "readResponseObject", "edu.uidaho.hummer.util.socket.HummerSocketException",
                     true);

		/*
		Normal transmission
		*/
    hummer_socket.validation(null);
    Response input = (Response)(runExceptionTest("Server: Normal transmission (correct signatures, " +
                                                 "_type equals _OK, etc)", hummer_socket,
                                                 "readResponseObject", null, false)[2]);
    runSimpleTest("Server: Normal transmission (correct signatures, _type equals _OK, etc)",
                  "Response._id recieved equals response sent.", input._id == _client_message._id);
    runSimpleTest("Server: Normal transmission (correct signatures, _type equals _OK, etc)",
                  "Response._message recieved equals response sent.",
                  input._message.equals(_client_message._message));
    runSimpleTest("Server: Normal transmission (correct signatures, _type equals _OK, etc)",
                  "Response recieved equals response sent.",
                  input._data.equals(_client_message._data));

		/*
		Invalid signature
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Inccorrect signature after validation completed correctly",
    	                 hummer_socket, "validation", null, false, null);
    runExceptionTest("Server: Inccorrect signature",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);


		/*
		Invalid id
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Inccorrect id after validation completed correctly",
    	                 hummer_socket, "validation", null, false, null);
    runExceptionTest("Server: Inccorrect id",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);
    hummer_socket.close();
  }




		/**
		Test for commands recieved by the server in readResponseObject
		*/
  private void testCommandsServer(ServerSocket server_socket,
                                  PersonalConfig personal_config) throws Exception {
    Socket client_handler_socket;
    HummerServerSocket hummer_socket;
    Response input, output = new Response();
    output._id = _input_message_id;
		/**********************************************
		***********************************************
		Begin testing of socket commands
		***********************************************
		***********************************************/
    note("Socket commands");
    
		/*
		Invalid user (not-admin) command to add a new user (new_user)
		*/
    _key_store.store(true, null);
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Before adding a new user (new_user) with invalid account (not admin)",
    	                 hummer_socket, "validation", null, false, null);
    runSimpleTest("Server: Before adding a new user (new_user) with invalid account (not admin)",
    	             "Key store does not contain \"new_user\".",
    	             !_key_store.contains("new_user"));
    runExceptionTest("Server: Adding a new user (new_user) with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);
    runSimpleTest("Server: Adding a new user (new_user) with invalid account (not admin)",
                  "Key store still does not contain alias \"new_user\".",
                  !_key_store.contains("new_user"));


		/*
		Command to add a new user (new_user) as alias admin
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    Thread.sleep(50);
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runSimpleTest("Server: Before adding a new user (new_user)",
    	             "Key store does not contain \"new_user\".",
    	             !_key_store.contains("new_user"));
    runExceptionTest("Server:   Adding a new user (new_user)",
    	                 hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Adding a new user (new_user)",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: Adding a new user (new_user)",
                     "Key store now contains alias \"new_user\".",
                     _key_store.contains("new_user"))) {
      runSimpleTest("Server: Adding a new user (new_user)",
                    "Alias type for new_user is _USER_ALIAS_TYPE",
                    _key_store.getAliasType("new_user") == AliasInfo._USER_ALIAS_TYPE);
      runSimpleTest("Server: Adding a new user (new_user)",
      	           "Account for new_user is a new account (not initialized)",
      	           _key_store.isNewAccount("new_user"));
    }
    runSimpleTest("Server: Adding a new user (new_user)",
                  "Response._id recieved equals response sent.", input._id == _client_message._id);
    runSimpleTest("Server: Adding a new user (new_user)",
                  "Response._message recieved equals response sent.",
                  input._message.equals(_client_message._message));
    runSimpleTest("Server: Adding a new user (new_user)",
                  "Response recieved equals response sent.",
                  input._data.equals(_client_message._data));



		/*
		Command to add a new server (new_server) as alias admin
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    Thread.sleep(500);
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server:   Adding a new server (new_server)",
    	                 hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Adding a new server (new_server), " +
                                        "calling readResponseObject.",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: Adding a new server (new_server)",
                     "Key store now contains alias \"new_server\".",
                     _key_store.contains("new_server"))) {
      runSimpleTest("Server: Adding a new server (new_server)",
                    "Alias type for new_user is _SERVER_ALIAS_TYPE",
                    _key_store.getAliasType("new_server") == AliasInfo._SERVER_ALIAS_TYPE);
      runSimpleTest("Server: Adding a new server (new_server)",
      	           "Account for new_server is a new account (not initialized)",
      	           _key_store.isNewAccount("new_server"));
    }
    if(runSimpleTest("Server: Adding a new server (new_server)",
                     "Non-null reponse recieved.", input != null)) {
      runSimpleTest("Server: Adding a new server (new_server)",
                    "Response._id recieved equals response sent.", input._id == _client_message._id);
      runSimpleTest("Server: Adding a new server (new_server)",
                    "Response._message recieved equals response sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Adding a new server (new_server)",
                    "Response recieved equals response sent.",
                    input._data.equals(_client_message._data));
   }



		/*
		Command to add a new peer (new_peer) as alias admin
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server:   Adding a new peer (new_peer)",
    	                 hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Adding a new peer (new_peer)",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: Adding a new peer (new_peer)",
                     "Key store now contains alias \"new_peer\".",
                     _key_store.contains("new_peer"))) {
      runSimpleTest("Server: Adding a new peer (new_peer)",
                    "Alias type for new_user is _PEER_ALIAS_TYPE",
                    _key_store.getAliasType("new_peer") == AliasInfo._PEER_ALIAS_TYPE);
      runSimpleTest("Server: Adding a new peer (new_peer)",
      	           "Account for new_peer is a new account (not initialized)",
      	           _key_store.isNewAccount("new_peer"));
    }
    if(runSimpleTest("Server: Adding a new peer (new_peer)",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Adding a new peer (new_peer)",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Adding a new peer (new_peer)",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Adding a new peer (new_peer)",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }


		/* *********************************************************************
		Begin testing for addManager and addPeerModerator.  These both have
		an underlying connection to the ConfigServer, and multiply changes are
		made to the HummerKeyStore
		***********************************************************************/

		/**
		addManager
		*/

		/*
		Invalid user (not-admin) command to add a new manager
		*/
    client_handler_socket = server_socket.accept();
    note("addManager testing");
    hummer_socket.close();
    _key_store.store(true, null);
    String local_address = InetAddress.getLocalHost().getHostAddress();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Before adding a new manager (" + local_address +
                     ") with invalid account (not admin)",
    	                 hummer_socket, "validation", null, false, null);
    runSimpleTest("Server: Before adding a new manager (" + local_address +
                  ") with invalid account (not admin)",
    	             "Key store does not contain \"" + local_address + "\".",
    	             !_key_store.contains(local_address));
    runExceptionTest("Server: Adding a new manager (" + local_address +
                     ") with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);
    runSimpleTest("Server: Adding a new manager (" + local_address +
                  ") with invalid account (not admin)",
                  "Key store still does not contain alias \"" + local_address + "\".",
                  !_key_store.contains(local_address));


		/*
		Admin account used to add manager
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    runSimpleTest("Server:Before a connection is made which will call addManager",
                  "No account exists for the soon to be manager (" + local_address + ").",
                  !_key_store.contains(local_address));
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Adding a new manager (" + local_address + ") with an invalid " +
                     "account (not admin)",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Adding a new manager (" + local_address + ")",
                                        hummer_socket, "readResponseObject", null, false)[2]);
     hummer_socket.endOfTransmission(null);
    if(runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                     "Key store now contains alias \"" + local_address + "\".",
                     _key_store.contains(local_address))) {
      runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Account for " + local_address + " is initialized.",
                    !_key_store.isNewAccount(local_address));
      runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Account for " + local_address + " is a server type alias.",
                    _key_store.getAliasType(local_address) == AliasInfo._SERVER_ALIAS_TYPE);
      runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Account for " + local_address + " is trusted.",
                    _key_store.isCertificateHonored(local_address));
      if(runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Account for " + local_address + " holds a HummerCertificate.",
                    _key_store.holdingHummerCertificate(local_address))) {
        HummerCertificate certificate = _key_store.getHummerCertificate(local_address);
        runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                      "Accesss table in certificate held is not a null value.",
                      certificate.getAccessTable() != null);
      }
    }
    if(runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Adding a new manager (" + local_address + ")",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }

		/**
		addPeerModerator

		Invalid user (not-admin) command to add a new manager
		*/
    Thread.sleep(300);
    note("addModerator testing");
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.removeAlias(local_address);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Before adding a new peer moderator (" + local_address +
                     ") with invalid account (not admin)",
    	                 hummer_socket, "validation", null, false, null);
    runSimpleTest("Server: Before adding a new peer moderator (" + local_address +
                  ") with invalid account (not admin)",
    	             "Key store does not contain \"" + local_address + "\".",
    	             !_key_store.contains(local_address));
    runExceptionTest("Server: Adding a new peer moderator (" + local_address +
                     ") with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);
    runSimpleTest("Server: Adding a new peer moderator (" + local_address +
                  ") with invalid account (not admin)",
                  "Key store still does not contain alias \"" + local_address + "\".",
                  !_key_store.contains(local_address));


		/*
		Admin account used to add peer moderator
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    runSimpleTest("Server:Before a connection is made which will call addPeerModerator.",
    	             "No account exists for the soon to be peer moderator (" + local_address + ").",
                  !_key_store.contains(local_address));
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Adding a new peer moderator (" + local_address +
                     ") with a valid account.",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Adding a new peer moderator (" + local_address + ")",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                     "Key store now contains alias \"" + local_address + "\".",
                     _key_store.contains(local_address))) {
      runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                    "Account for " + local_address + " is initialized.",
                    !_key_store.isNewAccount(local_address));
      runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                    "Account for " + local_address + " is a peer type alias.",
                    _key_store.getAliasType(local_address) == AliasInfo._PEER_ALIAS_TYPE);
      if(runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                    "Account for " + local_address + " holds a HummerCertificate.",
                    _key_store.holdingHummerCertificate(local_address))) {
        HummerCertificate certificate = _key_store.getHummerCertificate(local_address);
        runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                      "Accesss table in certificate held is not a null value.",
                      certificate.getAccessTable() != null);
      }
    }
    if(runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Adding a new peer moderator (" + local_address + ")",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
   }


		/*
		Begin testing for _GET_ALIAS_LIST

		Attempt to get list by an invalid account (not admin)
		*/
    Thread.sleep(300);
    note("getAliasList");
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Getting alias list with invalid account (not admin)",
                      hummer_socket, "validation", null, false, null);
    runExceptionTest("Server: Getting alias list with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);

		/*
		Admin account used to get alias list
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Getting an alias list with a valid account.",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Getting an alias list with a valid account.",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    runSimpleTest("Server: Getting an alias list with a valid account.",
                  "Response._id recieved equals response id sent.", input._id == _client_message._id);
    runSimpleTest("Server: Getting an alias list with a valid account.",
                  "Response._message recieved equals response message sent.",
                  input._message.equals(_client_message._message));
    runSimpleTest("Server: Getting an alias list with a valid account.",
                  "Response._data recieved equals response data sent.",
                  input._data.equals(_client_message._data));
  

		/*
		Begin testing for _RESET_ALIAS

		Attempt to get list by an invalid account (not admin)
		*/
    client_handler_socket = server_socket.accept();
    note("resetAlias");
    hummer_socket.close();
    _key_store.store(true, null);
    runSimpleTest("Server: Before recieving command to reset alias (invalid alias)",
    	             "Account for \"untrusted_sever\" is initialized",
    	             !_key_store.isNewAccount("untrusted_server"));
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Reseting alias with invalid account (not admin)",
                      hummer_socket, "validation", null, false, null);
    Exception e = (Exception)(runExceptionTest("Server: Reseting alias with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true)[1]);
    runSimpleTest("Server: After recieving command to reset alias (invalid alias)",
    	             "Account for \"untrusted_sever\" is initialized",
    	             !_key_store.isNewAccount("untrusted_server"));

		/*
		Admin account used to reset alias
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runSimpleTest("Server: Before recieving command to reset alias (valid alias)",
    	             "Account for \"untrusted_sever\" is initialized",
    	             !_key_store.isNewAccount("untrusted_server"));
    runExceptionTest("Server: Reseting an alias with a valid account.  " +
                     "Calling validation.(admin).",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Reqest to Reset an alias by admin.",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    runSimpleTest("Server: After recieving command to reset alias (valid alias)",
    	             "Account for \"untrusted_sever\" is no longer initialized",
                  _key_store.isNewAccount("untrusted_server"));
    if(runSimpleTest("Server: Reseting an alias list with a valid account(admin).",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Reseting an alias list with a valid account(admin).",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Getting an alias list with a valid account.",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Getting an alias list with a valid account.",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }



		/*
		Begin testing for _DELETE_ALIAS

		Attempt to get list by an invalid account (not admin)
		*/
    client_handler_socket = server_socket.accept();
    note("deleteAlias");
    hummer_socket.close();
    _key_store.store(true, null);
    runSimpleTest("Server: Before recieving command to delete alias (invalid alias)",
    	             "Account for \"untrusted_sever\" exists",
    	             _key_store.contains("untrusted_server"));
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Deleting alias with invalid account (not admin)",
                      hummer_socket, "validation", null, false, null);
    runExceptionTest("Server: Deleting alias with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);
    runSimpleTest("Server: After recieving command to delete alias (invalid alias)",
    	             "Account for \"untrusted_sever\" still exists",
    	             _key_store.contains("untrusted_server"));

		/*
		Admin account used to delete alias
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runSimpleTest("Server: Before recieving command to delete alias (valid alias)",
    	             "Account for \"untrusted_sever\" exists",
    	             _key_store.contains("untrusted_server"));
    runExceptionTest("Server: Deleting an alias with a valid account.  " +
                     "Calling validation.(admin).",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Reqest to delete an alias by admin.",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    runSimpleTest("Server: After recieving command to delete alias (valid alias)",
    	             "Account for \"untrusted_sever\" no longer exists",
                  !_key_store.contains("untrusted_server"));
    if(runSimpleTest("Server: Reseting an alias list with a valid account(admin).",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Reseting an alias list with a valid account(admin).",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Getting an alias list with a valid account.",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Getting an alias list with a valid account.",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }


		/*
		Begin testing for _GET_ACCESS_VALUES

		Attempt to get access values with an invalid account (not admin)
		*/
    client_handler_socket = server_socket.accept();
    note("getAccessValues");
    hummer_socket.close();
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Getting access values with invalid account (not admin)",
                      hummer_socket, "validation", null, false, null);
    runExceptionTest("Server: Getting access values with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);

		/*
		Admin account used to get access values
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket.close();
    _key_store.store(true, null);
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Getting access values with a valid account.  " +
                     "Calling validation.(admin).",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Getting access values for " +
                                        "\"local_user\" with admin.",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: Getting access values for \"local_user\" with admin.",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Getting access values for \"local_user\" with admin.",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Getting access values for \"local_user\" with admin.",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Getting access values for \"local_user\" with admin.",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }


		/*
		Begin testing for _SET_ACCESS_VALUES

		Attempt to get access values with an invalid account (not admin)
		*/
    client_handler_socket = server_socket.accept();
    note("setAccessValues");
    _key_store.store(true, null);
    hummer_socket.close();
    runSimpleTest("Server: Before setting access values for account",
                  "Access table is not an empty table",
                  !_key_store.equalAccessTable("trusted_server", new Hashtable()));
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Setting access values with invalid account (not admin)",
                      hummer_socket, "validation", null, false, null);
    runExceptionTest("Server: Setting access values with invalid account (not admin)",
                     hummer_socket, "readResponseObject",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true);
    runSimpleTest("Server: After attempting to set access values for account",
                  "Access table is still not an empty table",
                  !_key_store.equalAccessTable("trusted_server", new Hashtable()));

		/*
		Admin account used to set access values
		*/
    client_handler_socket = server_socket.accept();
    _key_store.store(true, null);
    hummer_socket.close();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runSimpleTest("Server: Before setting access values for account",
                  "Access table is not an empty table",
                  !_key_store.equalAccessTable("trusted_server", new Hashtable()));
    runExceptionTest("Server: Setting access values with a valid account.  " +
                     "Calling validation.(admin).",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Setting access values for " +
                                        "\"local_user\" with admin.",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: After attempting to set access values for account",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: After attempting to set access values for account",
                    "Access table is now an empty table",
                    _key_store.equalAccessTable("trusted_server", new Hashtable()));
      runSimpleTest("Server: Setting access values for \"trusted_server\" with admin.",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Setting access values for \"trusted_server\" with admin.",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Setting access values for \"trusted_server\" with admin.",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }




		/*
		Begin testing for _UPDATE_PUBLIC_KEY

		Attempt to get access values with an invalid account (not admin)
		*/
    client_handler_socket = server_socket.accept();
    note("updatePublicKey");
    _key_store.store(true, null);
    hummer_socket.close();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server: Setting PublicKey. Calling validation.(admin).",
                     hummer_socket, "validation", null, false, null);
    input = (Response)(runExceptionTest("Server: Updating Publickey for " +
                                        "\"local_user\".",
                                        hummer_socket, "readResponseObject", null, false)[2]);
    if(runSimpleTest("Server: Updating PublicKey for \"local_user\".",
                     "Non-null response recieved", input != null)) {
      runSimpleTest("Server: Updating PublicKey for \"local_user\".",
                    "Response._id recieved equals response id sent.", input._id == _client_message._id);
      runSimpleTest("Server: Updating PublicKey for \"local_user\".",
                    "Response._message recieved equals response message sent.",
                    input._message.equals(_client_message._message));
      runSimpleTest("Server: Updating PublicKey for \"local_user\".",
                    "Response._data recieved equals response data sent.",
                    input._data.equals(_client_message._data));
    }
  }





  private void testWriteResponseObjectClient() throws Exception {
		/* ************************************************************
		===============================================================
		Begin testing of writeResponseObject
		===============================================================
		**************************************************************/
		/*
		Test for writeResponseObject before validation has been run.
		*/
      Socket socket;
      Response input, output = new Response();
      output._id = _input_message_id;
      long output_message_id;
      socket = new Socket(_inet_address, _port_number);

		/*
		Normal operation (there is no checking of data in the Response object)
		*/
      output = new Response();
      output._id = _input_message_id;
      output._type = Response._OK;

      socket = new Socket(_inet_address, _port_number);

      GregorianCalendar expiration_date = new GregorianCalendar();
      expiration_date.add(Calendar.DAY_OF_YEAR, 5);
      _trusted_server_sign = StaticMethods.getSigningSignatureObject(_trusted_server_key_pair.getPrivate());
      _trusted_server_verify = StaticMethods.getVerifyingSignatureObject(_trusted_server_key_pair.getPublic());
      HummerCertificate trusted_certificate = new HummerCertificate(_trusted_server_sign,
                                                  _remote_user_key_pair.getPublic(),
                                                  _remote_user_access_table, expiration_date);
      output._id = _input_message_id;
      output._data.add("default");
      output._data.add(_remote_user_alias);
      output._data.add(_trusted_server_alias);
      output._data.add(trusted_certificate);
      ObjectOutputStream out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      ObjectInputStream in_stream = new ObjectInputStream(socket.getInputStream());
      HummerSignedObject signed_input;
      Object input_object = in_stream.readObject();
      if(input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject")) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        HummerCertificate server_certificate = (HummerCertificate)(input._data.get(1));
        output_message_id = input._id;

        input_object = in_stream.readObject();
        String signed_object_name = "edu.uidaho.hummer.dataobject.HummerSignedObject";
        if(runSimpleTest("Client:Normal operation", "Second object recieved is of type HummerSignedObject",
                 input_object.getClass().getName().equals(signed_object_name))) {
          signed_input = (HummerSignedObject)(input_object);
          input = (Response)(signed_input.getObject());
          runSimpleTest("Client:Normal operation", "Response._id is incremented from value sent to server",
                        input._id == (_input_message_id + 1));
          Signature signer_verify = StaticMethods.getVerifyingSignatureObject(server_certificate.getPublicKey());
          runSimpleTest("Client:Normal operation",
          	          "HummerSignedObject recieved is valid for server's PublicKey",
                        signed_input.verify(signer_verify));
        }
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
  }














  private void testReadResponseObjectClient() throws Exception {
		/* ************************************************************
		===============================================================
		Begin testing of readResponseObject
		===============================================================
		**************************************************************/
		/*
		Test for readResponseObject with normal values.
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      Response input, output = new Response();
      output._id = _input_message_id;
      Socket socket = new Socket(_inet_address, _port_number);
      ObjectOutputStream out_stream = new ObjectOutputStream(socket.getOutputStream());
      Signature server_sign = _key_store.getPersonalSignSignature(_local_user_alias);
      HummerCertificate local_certificate;
      GregorianCalendar expiration_date = new GregorianCalendar();
      expiration_date.add(Calendar.DAY_OF_YEAR, 5);
      local_certificate = new HummerCertificate(server_sign, _local_user_key_pair.getPublic(),
      	                                      _key_store.getAccessTable(_local_user_alias),
      	                                      	expiration_date);
      output = new Response();
      output._id = _input_message_id;
      output._data.add("default");
      output._data.add(_local_user_alias);
      output._data.add(InetAddress.getLocalHost().getHostAddress());
      output._data.add(local_certificate);
      out_stream.writeObject(output);
      ObjectInputStream in_stream = new ObjectInputStream(socket.getInputStream());
      Object input_object = in_stream.readObject();
      String input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: With valid/normal input",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._message = "New message";
        _client_message._data.add("new data[0]");
        _client_message._data.add("new data[1]");
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for readResponseObject with invalid signature.
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Inccorrect signature after validation completed correctly",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._message = "New message2";
        _client_message._data.add("new data[0]2");
        _client_message._data.add("new data[1]2");
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _remote_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Inccorrect signature after validation completed correctly",
      	            "Type of object recieved is of type: HummerSignedObject",
                      input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();


		/*
		Test for readResponseObject with invalid id.
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: With invalid id",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id;
        _client_message._message = "New message2";
        _client_message._data.add("new data[0]2");
        _client_message._data.add("new data[1]2");
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: With invalid id",
                      "Type of object recieved is of type: HummerSignedObject",
                      input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();
  }


		/**
		Test for commands sent by the client manually
		*/
  private void testCommandsClient() throws Exception {
		/*
		Test for invalid alias trying to send commands (not admin)
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      Response input, output = new Response();
      Signature server_sign = _key_store.getPersonalSignSignature(_local_user_alias);
      HummerCertificate local_certificate;
      GregorianCalendar expiration_date = new GregorianCalendar();
      expiration_date.add(Calendar.DAY_OF_YEAR, 5);
      local_certificate = new HummerCertificate(server_sign, _local_user_key_pair.getPublic(),
      	                                      _key_store.getAccessTable(_local_user_alias),
      	                                      	expiration_date);

      output._id = _input_message_id;
      output._data.add("default");
      output._data.add(_local_user_alias);
      output._data.add(InetAddress.getLocalHost().getHostAddress());
      output._data.add(local_certificate);
      Socket socket = new Socket(_inet_address, _port_number);
      ObjectOutputStream out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      ObjectInputStream in_stream = new ObjectInputStream(socket.getInputStream());
      Object input_object = in_stream.readObject();
      String input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to add user alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._ADD_ALIAS;
        _client_message._data.add("new_user");
        _client_message._data.add(new Integer(AliasInfo._USER_ALIAS_TYPE));
        _client_message._data.add(null);
        _client_message._data.add("test");
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to add user alias",
      	               "Type of object recieved is of type: HummerSignedObject",
      	               input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Initialize the admin account before continuing.
		*/
      KeyPair admin_key_pair = StaticMethods.generateKeyPair();
      Signature admin_sign = StaticMethods.getSigningSignatureObject(admin_key_pair.getPrivate());
      Signature admin_verify = StaticMethods.getVerifyingSignatureObject(admin_key_pair.getPublic());
      _key_store.resetAlias("admin", "test".getBytes());
      byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(),
             digest = StaticMethods.getDigest("test".getBytes(), random_timestamp,
                                              admin_key_pair.getPublic());
      HummerCertificate admin_certificate = _key_store.initializeAccount("admin", digest,
      	                                                               random_timestamp,
      	                                                               admin_key_pair.getPublic());


		/*
		Test for admin trying to add user alias
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to add user alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._ADD_ALIAS;
        _client_message._data.add("new_user");
        _client_message._data.add(new Integer(AliasInfo._USER_ALIAS_TYPE));
        _client_message._data.add(null);
        _client_message._data.add("test".getBytes());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input._id + 2;
        _client_message._message = "Message-new user";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      Socket tmp_socket = socket;

		/*
		Ports for Peer server and Messaging server must be set, because
		when adding a server alias or peer alias a connection is made to
		the peer server and messaging server to request them to reload
		their key stores.
		*/
      Properties constants = StaticMethods.getHummerProperties();
      int peer_port = (new Integer(constants.getProperty("port.PEER_PORT"))).intValue(),
          message_port = (new Integer(constants.getProperty("port.MESSAGE_PORT"))).intValue();



		/*
		Test for admin trying to add server alias
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      socket = new Socket(_inet_address, _port_number);
      tmp_socket.close();
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to add server alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long output_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = output_id;
        output_id++;
        _client_message._command = Response._ADD_ALIAS;
        _client_message._data.add("new_server");
        _client_message._data.add(new Integer(AliasInfo._SERVER_ALIAS_TYPE));
        _client_message._data.add(new Boolean(true));
        _client_message._data.add("test".getBytes());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);

        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = output_id;
        _client_message._message = "Message-new user";
        System.out.println("Client: Sending final response.");
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
        System.out.println("Client: Have sent response to server..");
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      tmp_socket = socket;



		/*
		Test for admin trying to add peer alias
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      socket = new Socket(_inet_address, _port_number);
      System.out.println("Client: Have opened socket with server. Closing socket after sending " +
                         "request to add a new server (by admin).");
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      tmp_socket.close();
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to add peer alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        long output_id = input._id + 1;
        _client_message._id = output_id;
        output_id++;
        _client_message._command = Response._ADD_ALIAS;
        _client_message._data.add("new_peer");
        _client_message._data.add(new Integer(AliasInfo._PEER_ALIAS_TYPE));
        _client_message._data.add(null);
        _client_message._data.add("test".getBytes());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);

        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = output_id;
        _client_message._message = "Message-new user";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      tmp_socket = socket;

		/* *******************************************************************
		**********************************************************************
		Test for invalid alias (not admin) trying to add a manager
		**********************************************************************
		*********************************************************************/
		/*
		Sleep a little before continuing so that the output messages
		can get syncrhonized
		*/
      Thread.sleep(500);
      socket = new Socket(_inet_address, _port_number);
      tmp_socket.close();
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to add manager alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._ADD_MANAGER;
        _client_message._data.add(new PersonalConfig("local_machine", "test".getBytes()));
        _client_message._data.add(InetAddress.getLocalHost().getHostAddress());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to add manager alias remotely",
      	               "Type of object recieved is of type: HummerSignedObject",
      	               input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to add manager alias
		note:output object has already been defined above, so this does not
			have to be done again.
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();

      Response server_value;
      if(runSimpleTest("Client: Valid alias (admin) trying to add manager",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        HummerSignedObject signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._ADD_MANAGER;
        _client_message._data.add(new PersonalConfig("local_machine", "test".getBytes()));
        _client_message._data.add(InetAddress.getLocalHost().getHostAddress());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);

		/*
		A socket has to be opened mimicing a ConfigServer, because
		the server should try to connect to a ConfigServer requesting to validate.
		*/
        Object config_port = StaticMethods.getHummerPropertyTable(false).get("port.CONFIG_PORT");
        DefaultKeyStore tmp_key_store = (DefaultKeyStore)HummerKeyStore.getInstance();
        ServerSocket server_socket = new ServerSocket(((Integer)config_port).intValue());
        Socket config_socket = server_socket.accept();

        tmp_key_store.load("test".getBytes());
        tmp_key_store.addServerAlias("local_machine", "test".getBytes(), false,
                                     StaticMethods.getDefaultServerAccessTable());
        
        HummerServerSocket hummer_server_socket = new HummerServerSocket(null, config_socket,
                                                                         tmp_key_store);
        hummer_server_socket.validation(null);
        server_value = (Response)(runExceptionTest("Client: Reading response object from server to request connection " +
                                        "for adding a manager (client is acting as ConfigServer)",
                                         hummer_server_socket, "readResponseObject", null, false)[2]);
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input._id + 2;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));

        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a manager (client is acting as ConfigServer)",
                      "Response recieved is an End-Of-Transmission",
                      server_value._type == Response._EOT);
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a manager (client is acting as ConfigServer)",
                      "Key store now contains an initialized value for the server (local_machine)",
                      !tmp_key_store.isNewAccount("local_machine"));
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a manager (client is acting as ConfigServer)",
                      "A HummerCertificate is held for the server (local_machine)",
                      tmp_key_store.holdingHummerCertificate("local_machine"));
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a manager (client is acting as ConfigServer)",
                      "The server's account in the key store is not trusted",
                      !tmp_key_store.isCertificateHonored("local_machine"));
        config_socket.close();
        hummer_server_socket.close();
        server_socket.close();
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();


		/* *******************************************************************

		Test for invalid alias (not admin) trying to add a peer moderator
		*********************************************************************/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      HummerSignedObject  signed_input;
      if(runSimpleTest("Client: Invalid alias trying to add moderator alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._ADD_PEER_MODERATOR;
        _client_message._data.add(new PersonalConfig("local_machine", "test".getBytes()));
        _client_message._data.add(InetAddress.getLocalHost().getHostAddress());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to add moderator alias remotely",
      	               "Type of object recieved is of type: HummerSignedObject",
      	               input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to add moderator alias
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to add peer moderator",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._ADD_PEER_MODERATOR;
        _client_message._data.add(new PersonalConfig("local_machine", "test".getBytes()));
        _client_message._data.add(InetAddress.getLocalHost().getHostAddress());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);

		/*
		A socket has to be opened mimicing a ConfigServer, because
		the server should try to connect to a ConfigServer requesting to validate.
		*/
        Object config_port = StaticMethods.getHummerPropertyTable(false).get("port.CONFIG_PORT");
        DefaultKeyStore tmp_key_store = (DefaultKeyStore)HummerKeyStore.getInstance();
        ServerSocket server_socket = new ServerSocket(((Integer)config_port).intValue());
        Socket config_socket = server_socket.accept();
        tmp_key_store.load("test".getBytes());
        tmp_key_store.addPeerAlias("local_machine", "test".getBytes());
        
        HummerServerSocket hummer_server_socket = new HummerServerSocket(null, config_socket,
                                                                         tmp_key_store);
        hummer_server_socket.validation(null);
        server_value = (Response)(runExceptionTest("Client: Reading response object from server to " +
                                                   "request connection " +
                                                   "for adding a moderator (client is acting as ConfigServer)",
                                                   hummer_server_socket, "readResponseObject", null, false)[2]);
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a moderator (client is acting as ConfigServer)",
                      "Response recieved is an End-Of-Transmission",
                      server_value._type == Response._EOT);
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a moderator (client is acting as ConfigServer)",
                      "Key store now contains an initialized value for the server (local_machine)",
                      !tmp_key_store.isNewAccount("local_machine"));
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a moderator (client is acting as ConfigServer)",
                      "A HummerCertificate is held for the server (local_machine)",
                      tmp_key_store.holdingHummerCertificate("local_machine"));
        runSimpleTest("Client: Reading response object from server to request connection " +
                      "for adding a moderator (client is acting as ConfigServer)",
                      "The server's account in the key store is not trusted",
                      !tmp_key_store.isCertificateHonored("local_machine"));
        config_socket.close();
        hummer_server_socket.close();
        server_socket.close();
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input._id + 2;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/* ************************************************************
		***************************************************************
		Get aliasList Testing
		***************************************************************
		**************************************************************/


		/* *******************************************************************

		Test for invalid alias (not admin) trying to list the aliases
		*********************************************************************/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to get alias list (_GET_ALIASES)",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._GET_ALIASES;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to get alias list (_GET_ALIASES)",
      	             "Type of object recieved is of type: HummerSignedObject",
      	             input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to get alias list
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to get list of aliases",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        input_message_id++;
        _client_message._command = Response._GET_ALIASES;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        if(runSimpleTest("Client: Valid alias (admin) trying to get list of aliases",
                         "Type of object recieved is of type: HummerSignedObject",
                         input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
          signed_input = (HummerSignedObject)(input_object);
          input = (Response)(signed_input.getObject());
          String[] known_aliases = new String[9];
          known_aliases[0] = "admin";
          known_aliases[1] = "local_user";
          known_aliases[2] = "new_local_user";
          known_aliases[3] = "untrusted_server";
          known_aliases[4] = "trusted_server";
          known_aliases[5] = "new_server";
          known_aliases[6] = "new_user";
          known_aliases[7] = "new_peer";
          known_aliases[8] = InetAddress.getLocalHost().getHostAddress();
          Arrays.sort(known_aliases);

          String[] recieved = new String[input._data.size() - 1];
          for(int i = 1; i < input._data.size(); i++) {
            recieved[i - 1] = ((Object[])input._data.get(i))[0].toString();
          }
          Arrays.sort(recieved);
          _key_store.store(true, null);

          runSimpleTest("Client: Valid alias (admin) trying to get list of aliases",
                        "Numer of aliases returned equals number known to be in " +
                        "key store - 1 (header)",
                        known_aliases.length == recieved.length);
          runSimpleTest("Client: Valid alias (admin) trying to get list of aliases",
                        "Elements recieved equal elements known to be in key store",
                        Arrays.equals(known_aliases, recieved));
        }
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();


		/* ************************************************************
		***************************************************************
		reset alias testing
		***************************************************************
		**************************************************************/


		/* *******************************************************************

		Test for invalid alias (not admin) trying to reset an alias
		*********************************************************************/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to get reset an alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._RESET_ALIAS;
        _client_message._data.add("untrusted_server");
        _client_message._data.add("test".getBytes());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to get reset an alias",
      	             "Type of object recieved is of type: HummerSignedObject",
      	             input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to get alias list
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to reset alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        _client_message._data.add("untrusted_server");
        _client_message._data.add("test".getBytes());
        input_message_id++;
        _client_message._command = Response._RESET_ALIAS;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();


		/* ************************************************************
		***************************************************************
		delete alias testing
		***************************************************************
		**************************************************************/


		/* *******************************************************************

		Test for invalid alias (not admin) trying to reset an alias
		*********************************************************************/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to get delete an alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._DELETE_ALIAS;
        _client_message._data.add("untrusted_server");
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to get delete an alias",
      	             "Type of object recieved is of type: HummerSignedObject",
      	             input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to delete an alias
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to delete alias",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        _client_message._data.add("untrusted_server");
        input_message_id++;
        _client_message._command = Response._DELETE_ALIAS;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();



		/* ************************************************************
		***************************************************************
		get access values testing
		***************************************************************
		**************************************************************/


		/* *******************************************************************

		Test for invalid alias (not admin) trying to get access values
		*********************************************************************/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to get alias access values",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._GET_ACCESS_VALUES;
        _client_message._data.add("trusted_server");
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to get access values for an alias",
      	             "Type of object recieved is of type: HummerSignedObject",
      	             input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to get access rights for an alias
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to get access values for alias",
                       "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        _client_message._data.add("trusted_server");
        input_message_id++;
        _client_message._command = Response._GET_ACCESS_VALUES;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);

        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        if(runSimpleTest("Client: Valid alias (admin) trying to get access values for alias",
                         "Type of object recieved is of type: HummerSignedObject",
                         input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
          signed_input = (HummerSignedObject)(input_object);
          input = (Response)(signed_input.getObject());
          Hashtable access_table = (Hashtable)input._message;
          runSimpleTest("Client: Valid alias (admin) trying to get access values for alias",
                        "Access values (Hashtable) recieved equals values in key store",
                        _key_store.equalAccessTable("trusted_server", access_table));
        }
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-get access values";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();



		/* ************************************************************
		***************************************************************
		get access values testing
		***************************************************************
		**************************************************************/


		/* *******************************************************************

		Test for invalid alias (not admin) trying to set access values
		*********************************************************************/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Invalid alias trying to get alias access values",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        _client_message = new Response();
        _client_message._id = input._id + 1;
        _client_message._command = Response._SET_ACCESS_VALUES;
        _client_message._data.add("trusted_server");
        _client_message._data.add(new Hashtable());
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        input_type = input_object.getClass().getName();
        runSimpleTest("Client: Invalid alias trying to get access values for an alias",
      	             "Type of object recieved is of type: HummerSignedObject",
      	             input_type.equals("edu.uidaho.hummer.dataobject.Response"));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();

		/*
		Test for admin trying to set access rights for an alias
		*/
      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, "admin");
      output._data.set(3, admin_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid alias (admin) trying to get access values for alias",
                       "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        _client_message._data.add("trusted_server");
        _client_message._data.add(new Hashtable());
        input_message_id++;
        _client_message._command = Response._SET_ACCESS_VALUES;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, admin_sign);
        out_stream.writeObject(signed_output);

        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-get access values";
        out_stream.writeObject(new HummerSignedObject(_client_message, admin_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();





		/* ************************************************************
		***************************************************************
		update public key testing (not a priviledged command)
		***************************************************************
		**************************************************************/

      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _local_user_alias);
      output._data.set(3, local_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid login for updating PublicKey",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        _local_user_key_pair = StaticMethods.generateKeyPair();
        _client_message._data.add(_local_user_key_pair.getPublic());
        input_message_id++;
        _client_message._command = Response._UPDATE_PUBLIC_KEY;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        input_object = in_stream.readObject();
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)signed_input.getObject();
        input_type = input._message.getClass().getName();
        if(runSimpleTest("Client: Valid login for updating PublicKey",
                         "Recieved a new HummerCertificate",
                    input_type.equals("edu.uidaho.hummer.dataobject.HummerCertificate"))) {
          local_certificate = (HummerCertificate)input._message;
          runSimpleTest("Client: Valid login for updating PublicKey",
                        "Certificate holds correct PublicKey",
                        local_certificate.getPublicKey().equals(_local_user_key_pair.getPublic()));
          runSimpleTest("Client: Valid login for updating PublicKey",
                        "Certificate is not expired",
                        !local_certificate.isExpired());
          runSimpleTest("Client: Valid login for updating PublicKey",
                        "Certificate is valid for key store's PublicKey",
                        local_certificate.isValid(_key_store.getPersonalVerifySignature()));
        }
//        _local_user_sign = StaticMethods.getSigningSignatureObject(_local_user_key_pair.getPrivate());
        runSimpleTest("Client: Valid login for updating PublicKey",
                      "PublicKey in key store is the new PublicKey",
                      _key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE,_local_user_alias).equals(
                                           _local_user_key_pair.getPublic()));
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, _local_user_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
      socket.close();


		/* ************************************************************
		***************************************************************
		setHummerCertificate testing (not a priviledged command)
		***************************************************************
		**************************************************************

      socket = new Socket(_inet_address, _port_number);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      output._data.set(1, _trusted_server_alias);
      HummerCertificate trusted_certificate = _key_store.generateHummerCertificate(_trusted_sever_alias);
      output._data.set(3, trusted_certificate);
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input_object = in_stream.readObject();
      input_type = input_object.getClass().getName();
      if(runSimpleTest("Client: Valid login for setting HummerCertificate",
      	             "Type of object recieved is of type: HummerSignedObject",
                       input_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());
        long input_message_id = input._id + 1;
        _client_message = new Response();
        _client_message._id = input_message_id;
        
        _client_message._data.add(_local_user_key_pair.getPublic());
        input_message_id++;
        _client_message._command = Response._UPDATE_PUBLIC_KEY;
        HummerSignedObject signed_output = new HummerSignedObject(_client_message, _local_user_sign);
        out_stream.writeObject(signed_output);
        HummerCertificate new_certificate = null;
        try {
          new_certificate = (HummerCertificate)in_stream.readObject();
        }
        catch(ClassCastException e) {}
        if(runSimpleTest("Client: Valid login for updating PublicKey",
                         "Recieved a new HummerCertificate",
                         new_certificate != null)) {
          runSimpleTest("Client: Valid login for updating PublicKey",
                        "Certificate holds correct PublicKey",
                        new_certificate.getPublicKey().equals(_local_user_key_pair.getPublic()));
          runSimpleTest("Client: Valid login for updating PublicKey",
                        "Certificate is not expired",
                        !new_certificate.isExpired());
          runSimpleTest("Client: Valid login for updating PublicKey",
                        "Certificate is valid for key store's PublicKey",
                        new_certificate.isValid(_key_store.getPersonalVerifySignature()));
        }
        _local_user_sign = StaticMethods.getSigningSignatureObject(_local_user_key_pair.getPrivate());
        runSimpleTest("Client: Valid login for updating PublicKey",
                      "PublicKey in key store is the new PublicKey",
                      _key_store.getKey(AliasInfo._PUBLIC_KEY_TYPE,_local_user_alias).equals(
                                           _local_user_key_pair.getPublic()));
        _client_message = new Response();
        _client_message._type = Response._OK;
        _client_message._id = input_message_id;
        _client_message._message = "Message-new manager";
        out_stream.writeObject(new HummerSignedObject(_client_message, _local_user_sign));
      }
      else {
        String type_name = input_object.getClass().getName();
        if(type_name.equals("edu.uidaho.hummer.dataobject.Response")) {
          System.out.println("Client:Recieved Response object:\n" +
                             StaticMethods.responseToString((Response)input_object, "\t"));
        }
        else {
          System.out.println("Client: Unknown object type: " + type_name);
        }
      }
*/



  }


  private void testNewAccountValidationClient() throws Exception {
		/* ******************************************************
		=========================================================
		* Begin testing for newAccountValidation
		=========================================================
		********************************************************/
      byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(),
             correct_message_digest = StaticMethods.getDigest(_new_local_user_pass.getBytes(),
                                                              random_timestamp,
                                                              _new_local_user_key_pair.getPublic()),
             incorrect_message_digest = StaticMethods.getDigest("Wrong password".getBytes(),
                                                                random_timestamp,
                                                                _new_local_user_key_pair.getPublic());
		/*
		Test for short _data Vector
		*/
      Socket socket = new Socket(_inet_address, _port_number);
      Response input, output = new Response();
      output._id = _input_message_id;
      output._data.removeAllElements();
      output._data.add("default");
      output._data.add(_new_local_user_alias);
      output._data.add(null);
      output._data.add(_new_local_user_key_pair.getPublic());
      output._data.add(correct_message_digest);
      ObjectOutputStream out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      ObjectInputStream in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending short _data vector (size=5, new account)",
                    "Reponse._type is _EXCEPTION and exception type equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Test for long _data Vector
		*/
      socket = new Socket(_inet_address, _port_number);
      output._data.add(correct_message_digest);
      output._data.add("Useless string");
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending long _data vector (size=7, new account)",
                    "Reponse._type is _EXCEPTION and exception equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));


		/*
		Test for null PublicKey
		*/
      socket = new Socket(_inet_address, _port_number);
      output._data.remove(output._data.size() - 1);
      output._data.set(3, null);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending null PublicKey (new account)",
                    "Reponse._type is _EXCEPTION and exception equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Test for null message digest
		*/
      socket = new Socket(_inet_address, _port_number);
      output._data.set(3, _new_local_user_key_pair.getPublic());
      output._data.set(4, null);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending null message digest (new account)",
                    "Reponse._type is _EXCEPTION and exception equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Test for null random value and timestamp
		*/
      socket = new Socket(_inet_address, _port_number);
      output._data.set(4, correct_message_digest);
      output._data.set(5, null);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending null random_timestamp (new account)",
                    "Reponse._type is _EXCEPTION and exception type equals _INVALID_DATA_FORMAT, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_DATA_FORMAT &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Test for unknown alias (remote alias is not known by system)
		*/
      socket = new Socket(_inet_address, _port_number);
      output._data.set(1, _remote_user_alias);
      output._data.set(5, random_timestamp);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending unknown alias (new account)",
                    "Reponse._type is _EXCEPTION and exception type equals _INVALID_LOGIN, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type ==  HummerSocketException._INVALID_LOGIN &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Test for invalid message digest
		*/
      socket = new Socket(_inet_address, _port_number);
      output._data.set(1, _new_local_user_alias);
      output._data.set(4, incorrect_message_digest);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      input = (Response)(in_stream.readObject());
      runSimpleTest("Client:Response object recieved after sending incorrect message digest (new account)",
                    "Reponse._type is _EXCEPTION and exception equals _INVALID_LOGIN, " +
                    "and Response._message is a HummerKeyStoreException",
                    input._type == Response._EXCEPTION &&
                    ((HummerSocketException)input._message)._type == HummerSocketException._INVALID_LOGIN &&
                    input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException"));

		/*
		Test for valid information to initialize account
		*/
      socket = new Socket(_inet_address, _port_number);

      output._id = _input_message_id;
      output._data.set(4, correct_message_digest);
      out_stream = new ObjectOutputStream(socket.getOutputStream());
      out_stream.writeObject(output);
      in_stream = new ObjectInputStream(socket.getInputStream());
      Object input_object = in_stream.readObject();
      HummerSignedObject signed_input;
      if(runSimpleTest("Client:Response object is valid (new account)",
                       "Object recieved is a HummerSignedObject", 
                       input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject"))) {
        signed_input = (HummerSignedObject)(input_object);
        input = (Response)(signed_input.getObject());

        runSimpleTest("Client:Response object is valid (new account)",
                      "Protocol is \"default\".",
                      input._data.get(0).equals("default"));

        PublicKey server_public_key = (PublicKey)(input._data.get(1));
        HummerCertificate client_certificate = (HummerCertificate)(input._data.get(2));
        byte[] new_random_timestamp = (byte[])(input._data.get(4)),
               new_message_digest = (byte[])(input._data.get(3));

        runSimpleTest("Client:Response object is valid (new account)",
                      "Random value and timestamp are different from recieved values",
                      !Arrays.equals(random_timestamp, new_random_timestamp) &&
                      !Arrays.equals(correct_message_digest, new_message_digest));
        runSimpleTest("Client:Response object is valid (new account)",
                      "HummerCertificate is non-null value", client_certificate != null);
        runSimpleTest("Client:Response object is valid (new account)",
                      "Message digest is non-null value", new_message_digest != null);

        if(client_certificate != null) {
          runSimpleTest("Client:Response object is valid (new account)",
                        "HummerCertificate is not expired",
                        !client_certificate.isExpired());
          runSimpleTest("Client:Response object is valid (new account)",
                        "PublicKey recieved is equal to server's newest PublicKey",
                        _key_store.getPersonalKeyPair().getPublic().equals(server_public_key));
          Signature signer_verify = StaticMethods.getVerifyingSignatureObject(server_public_key);
          runSimpleTest("Client:Response object is valid (new account)",
                        "HummerCertificate can be verified with given PublicKey",
                        client_certificate.isValid(signer_verify));
          runSimpleTest("Client:Response object is valid (new account)",
                        "Recieved HummerSignedObject can be verified with given PublicKey",
                         signed_input.verify(signer_verify));
          runSimpleTest("Client:Response object is valid (new account)",
                        "PublicKey in HummerCertificate equals client's PublicKey",
                         client_certificate.getPublicKey().equals(_new_local_user_key_pair.getPublic()));
        }
        else {
          runSimpleTest("Client:Response object is valid (new account)",
                        "HummerCertificate is a non-null value", false);
        }

        if(new_message_digest != null) {
          byte[] generated_digest = StaticMethods.getDigest(_new_local_user_pass.getBytes(),
                                                            new_random_timestamp,
                                                            server_public_key);
          runSimpleTest("Client:Response object is valid (new account)",
                        "Recieved message digest is valid",
                        MessageDigest.isEqual(new_message_digest, generated_digest));
        }

        if(input._message != null) {
          runSimpleTest("Client:Response object is valid (new account)",
                        "(Long)(Response._message) equals sent id",
                        ((Long)input._message).longValue() == _input_message_id);
        }
        else {
          runSimpleTest("Client:Response object is valid (new account)",
                        "_message is non-null value", false);
        }
        runSimpleTest("Client:Response object is valid (new account)",
                      "Reponse._type is _OK", input._type == Response._OK);
        runSimpleTest("Client:Response object is valid (new account)", "Reponse._data.size() == 5",
                      input._data.size() == 5);
      }
  }


  private void testValidationServer(ServerSocket server_socket, HummerServerSocket hummer_socket,
                                    PersonalConfig personal_config) throws Exception {
		/* *********************************
		validation testing
		***********************************/
    newMethod("validation");
		/*
		Test for a null Response object
		*/
    note("Remote Account");
    HummerSocketException socket_exception = null;


		/*
		Test for null Response object
		*/
    Object[] exception_results = runExceptionTest("Server:Recieved a null Response object",
                                                  hummer_socket, "validation",
                                                  "edu.uidaho.hummer.util.socket.HummerSocketException",
                                                  true, null);
    
    if(((Boolean)exception_results[0]).booleanValue()) {
      if(exception_results[1] == null) {
        Response new_response = new Response();
        new_response._type = 0;
        socket_exception = new HummerSocketException("No exception thrown", new_response, 0);
      }
      else {
        socket_exception = (HummerSocketException)exception_results[1];
      }
      runSimpleTest("Server: Response object recieved in HummerSocketException", "null object",
                    socket_exception._recieved_information == null);
    }


		/*
		Test for a null _data Vector
		*/
    Socket client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    exception_results = runExceptionTest("Server:Recieved a null _data Vector in Response object",
                                         hummer_socket, "validation",
                                         "edu.uidaho.hummer.util.socket.HummerSocketException",
                                         true, null);
    if(((Boolean)exception_results[0]).booleanValue()) {
      if(exception_results[1] == null) {
        Response new_response = new Response();
        new_response._type = 0;
        socket_exception = new HummerSocketException("No exception thrown", new_response, 0);
      }
      else {
        socket_exception = (HummerSocketException)exception_results[1];
      }
      runSimpleTest("Server: Response object recieved from client (held in HummerSocketException)",
                    "_data Vector is null",
                    socket_exception._recieved_information._data == null);
    }


		/*
		Test for a _data Vector that is too short
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    exception_results = runExceptionTest("Server:Recieved an invalid size _data Vector " +
                                         "in Response object(too short)",
                                         hummer_socket, "validation",
                                         "edu.uidaho.hummer.util.socket.HummerSocketException",
                                         true, null);
    if(((Boolean)exception_results[0]).booleanValue()) {
      if(exception_results[1] == null) {
        Response new_response = new Response();
        new_response._type = 0;
        socket_exception = new HummerSocketException("No exception thrown", new_response, 0);
      }
      else {
        socket_exception = (HummerSocketException)exception_results[1];
      }
      runSimpleTest("Server: _data Vector.size() in Response object recieved in HummerSocketException", "3",
                    socket_exception._recieved_information._data.size() == 3);
    }


		/*
		Test for a _data Vector that is too long
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    exception_results = runExceptionTest("Server:Recieved an invalid size _data Vector " +
                                         "in Response object(too long)",
                                         hummer_socket, "validation",
                                         "edu.uidaho.hummer.util.socket.HummerSocketException",
                                         true, null);
    if(((Boolean)exception_results[0]).booleanValue()) {
      if(exception_results[1] == null) {
        Response new_response = new Response();
        new_response._type = 0;
        socket_exception = new HummerSocketException("No exception thrown", new_response, 0);
      }
      else {
        socket_exception = (HummerSocketException)exception_results[1];
      }
      runSimpleTest("Server: _data Vector.size() in Response object recieved in HummerSocketException", "5",
                    socket_exception._recieved_information._data.size() == 5);
    }

		/*
		Test for a null protocol name
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved a null protocol name", hummer_socket, "validation",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);

		/*
		Test for a null alias
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved a null alias", hummer_socket, "validation",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);

		/*
		Test for a null HummerCertificate
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved a null HummerCertificate", hummer_socket, "validation",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


		/*
		Test for an invalid protocol name
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved an invalid protocol name", hummer_socket, "validation",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);


		/*
		Test for an unknown signer alias
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved an invalid signer alias",
                                         hummer_socket, "validation",
                                         "edu.uidaho.hummer.util.socket.HummerSocketException",
                                         true, null);


		/*
		Test for expired HummerCertificate
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved an expired HummerCertificate",
                     hummer_socket, "validation", "edu.uidaho.hummer.util.socket.HummerSocketException",
                     true, null);

		/*
		Test for an incorrect HummerCertificate (HummerCertificate for wrong alias)
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved an incorrect HummerCertificate (does not match alias)",
                     hummer_socket, "validation", "edu.uidaho.hummer.util.socket.HummerSocketException",
                     true, null);


		/*
		Test for an untrusted HummerCertificate
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved a valid HummerCertificate from untrusted server",
                     hummer_socket, "validation", "edu.uidaho.hummer.util.socket.HummerSocketException",
                     true, null);
    _key_store.store(true, null);

    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

		/*
		Test for valid values sent
		*/
    runExceptionTest("Server:Recieved valid alias, signer alias, and correct HummerCertificate",
                     hummer_socket, "validation", null, false, null);
    runSimpleTest("Server:Current alias access table", "Equals access table for remote user",
                     hummer_socket.getCurrentClientAccessTable().equals(_remote_user_access_table));
    hummer_socket.close();

	/* ***********************************************
	Testing for a local account
	*************************************************/
		/*
		Sleep a bit to make sure that the client thread has finished outputing
		its tests before continuing.
		*/
    Thread.sleep(500);
    note("Local Account");

		/*
		Test for login of local account with normal values
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved valid alias, signer alias, and correct HummerCertificate (local account)",
                     hummer_socket, "validation", null, false, null);
    runSimpleTest("Server:Current alias access table (local user)", "Equals access table",
                     hummer_socket.getCurrentClientAccessTable().equals(_local_user_access_table));

    hummer_socket.close();

		/* **********
		Force the key store to update its key pair
		*************/
    _key_store.updateKeyPair(true);
    _key_store.store(true, null);

		/*
		Test that remote accounts can still login
		*/
    Thread.sleep(300);
    note("Update KeyPair - Remote Account");
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved valid alias, signer alias, " +
                     "and correct HummerCertificate (remote account)",
                     hummer_socket, "validation",
                     "edu.uidaho.hummer.util.socket.HummerSocketException", false, null);
    runSimpleTest("Server:Current alias access table (key store key pair update - remote account)",
                     "Equals access table for remote user",
                     hummer_socket.getCurrentClientAccessTable().equals(_remote_user_access_table));
    client_handler_socket.close();

		/*
		Change the password for the trusted server in the key store
		and verify that the remote account can no longer log in
		*/
    Thread.sleep(300);
    PublicKey old_key = _trusted_server_key_pair.getPublic();

    _trusted_server_key_pair = StaticMethods.generateKeyPair();
    _key_store.setKey(AliasInfo._PUBLIC_KEY_TYPE, "trusted_server", _trusted_server_key_pair.getPublic());
    _key_store.store(true, null);
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Recieved valid alias and signer alias. " +
                     "Trusted server has changed key pair (remote account)",
                     hummer_socket, "validation", "edu.uidaho.hummer.util.socket.HummerSocketException", true, null);
    client_handler_socket.close();

		/*
		Test local account after updating key pair
		*/
    Thread.sleep(300);
    note("Update KeyPair - Local Account");
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:After changing key pair and recieved an invalid alias, " +
                     "signer alias, and correct HummerCertificate (local account)",
                     hummer_socket, "validation", null, true, null);
    client_handler_socket.close();


    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:After changing key pair and recieved valid alias, " +
                     "signer alias, and correct HummerCertificate (local account)",
                     hummer_socket, "validation", null, false, null);
    runSimpleTest("Server:Current alias access table (local user)", "Equals access table",
                     hummer_socket.getCurrentClientAccessTable().equals(_local_user_access_table));
    hummer_socket.close();    



		/*
		Test for local server logging into another local server (eg. ConfigServer->PeerServer)
		*/
    note("local server logging into another local server");
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server:Valid information sent from client (local server to local server login)",
                     hummer_socket, "validation", null, false, null);                   
    hummer_socket.close();
  }






  private void testNewAccountValidationServer(ServerSocket server_socket,
                                              PersonalConfig personal_config) throws Exception {
	/* *************************************************************************
	----------------------------------------------------------------------------
	 * Begin testing newAcountValidation
	----------------------------------------------------------------------------
	 **************************************************************************/
    Thread.sleep(500);
    newMethod("newAccountValidation");

		/*
		Test for Response._data vector that is too short (5)
		*/
    Socket client_handler_socket = server_socket.accept();
    HummerServerSocket hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Response._data Vector sent by client is too short (size=4) (new account)",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();

		/*
		Test for Response._data vector that is too long (7)
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Response._data Vector sent by client is too long (size=7)",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();

		/*
		Test for null PublicKey (_data[3])
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Null client PublicKey sent by client (new account)",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();


		/*
		Test for null message digest (_data[4])
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Null message digest sent by client (new account)",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();


		/*
		Test for null random value and timestamp (_data[5])
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Null random value and timestamp sent by client (new account)",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();


		/*
		Test for unknown alias
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Unknown alias sent by client (new account)",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();

		/*
		Test for invalid message digest
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);

    runExceptionTest("Server:Invalid message digest sent by client",
                     hummer_socket, "validation", null, true, null);
    hummer_socket.close();


		/*
		Test for valid information
		*/
    client_handler_socket = server_socket.accept();
    hummer_socket = new HummerServerSocket(personal_config, client_handler_socket, _key_store);
    runExceptionTest("Server:Valid information sent from client for initializing account",
                     hummer_socket, "validation", null, false, null);
    runSimpleTest("Server:Valid information sent from client for initializing account",
                  "Account for new user is no longer considered \"new\" by HummerKeyStore (" +
                  _new_local_user_alias + ")",
                  !_key_store.isNewAccount(_new_local_user_alias));
    hummer_socket.close();
  }






  private void setStaticVariables() throws Exception {
    _local_user_key_pair = StaticMethods.generateKeyPair();
    _new_local_user_key_pair = StaticMethods.generateKeyPair();
    _remote_user_key_pair = StaticMethods.generateKeyPair();
    _untrusted_server_key_pair = StaticMethods.generateKeyPair();
    _trusted_server_key_pair = StaticMethods.generateKeyPair();


    _remote_user_sign = StaticMethods.getSigningSignatureObject(_remote_user_key_pair.getPrivate());
    _local_user_sign = StaticMethods.getSigningSignatureObject(_local_user_key_pair.getPrivate());
    _trusted_server_sign = StaticMethods.getSigningSignatureObject(_trusted_server_key_pair.getPrivate());
    _trusted_server_verify = StaticMethods.getVerifyingSignatureObject(_trusted_server_key_pair.getPublic());
    _untrusted_server_sign = StaticMethods.getSigningSignatureObject(_untrusted_server_key_pair.getPrivate());
    _untrusted_server_verify = StaticMethods.getVerifyingSignatureObject(_untrusted_server_key_pair.getPublic());

		/*
		Create unique access tables for all aliases
		*/
    _remote_user_access_table.put("table1.read", Boolean.FALSE);
    _remote_user_access_table.put("table2.read", Boolean.FALSE);
    _remote_user_access_table.put("table1.write", Boolean.FALSE);

    _local_user_access_table.put("table1.read", Boolean.FALSE);
    _local_user_access_table.put("table2.read", Boolean.TRUE);
    _local_user_access_table.put("table1.write", Boolean.FALSE);

    _new_local_user_access_table.put("table1.read", Boolean.TRUE);
    _new_local_user_access_table.put("table2.read", Boolean.FALSE);
    _new_local_user_access_table.put("table1.write", Boolean.FALSE);

    _untrusted_server_access_table.put("table1.read", Boolean.TRUE);
    _untrusted_server_access_table.put("table2.read", Boolean.TRUE);
    _untrusted_server_access_table.put("table1.write", Boolean.FALSE);

    _trusted_server_access_table.put("table1.read", Boolean.TRUE);
    _trusted_server_access_table.put("table2.read", Boolean.TRUE);
    _trusted_server_access_table.put("table1.write", Boolean.TRUE);

    GregorianCalendar expiration_date = new GregorianCalendar();
    expiration_date.add(Calendar.DAY_OF_YEAR, 5);

    _trusted_remote_user_certificate = new HummerCertificate(_trusted_server_sign,
                                                      _remote_user_key_pair.getPublic(),
                                                      _remote_user_access_table, expiration_date);

    _untrusted_remote_user_certificate = new HummerCertificate(_untrusted_server_sign,
                                                      _remote_user_key_pair.getPublic(),
                                                      _remote_user_access_table, expiration_date);

    _key_store = (DefaultKeyStore)HummerKeyStore.getInstance();



    _key_store.load("admin_pass".getBytes());
    _key_store.addUserAlias(_local_user_alias, _local_user_pass.getBytes(), _local_user_access_table);
    _key_store.addUserAlias(_new_local_user_alias, _new_local_user_pass.getBytes(), _new_local_user_access_table);
    _key_store.addServerAlias(_untrusted_server_alias, _untrusted_server_pass.getBytes(),
                             false, _untrusted_server_access_table);
    _key_store.addServerAlias(_trusted_server_alias, _trusted_server_pass.getBytes(),
                             true, _trusted_server_access_table);
     byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(),
            untrusted_digest = StaticMethods.getDigest(_untrusted_server_pass.getBytes(),
                                                       random_timestamp,
                                                       _untrusted_server_key_pair.getPublic()),
            trusted_digest = StaticMethods.getDigest(_trusted_server_pass.getBytes(),
                                                     random_timestamp,
                                                     _trusted_server_key_pair.getPublic()),
            local_user_digest = StaticMethods.getDigest(_local_user_pass.getBytes(),
                                                       random_timestamp,
                                                       _local_user_key_pair.getPublic());
    HummerCertificate server_certificate_trusted, server_certificate_untrusted;
    server_certificate_trusted = new HummerCertificate(_trusted_server_sign,
                                               _key_store.getPersonalKeyPair().getPublic(), null,
                                               _key_store.getPersonalKeyPairExpiration());
    server_certificate_untrusted = new HummerCertificate(_untrusted_server_sign,
                                               _key_store.getPersonalKeyPair().getPublic(), null,
                                               _key_store.getPersonalKeyPairExpiration());

    _key_store.initializeAccount(_untrusted_server_alias, untrusted_digest,
                                 random_timestamp,
                                 _untrusted_server_key_pair.getPublic());
    _key_store.setHummerCertificate(_untrusted_server_alias, server_certificate_untrusted);
    _key_store.initializeAccount(_trusted_server_alias, trusted_digest,
                                 random_timestamp,
                                 _trusted_server_key_pair.getPublic());
    _key_store.setHummerCertificate(_trusted_server_alias, server_certificate_trusted);
    _key_store.initializeAccount(_local_user_alias, local_user_digest,
                                 random_timestamp,
                                 _local_user_key_pair.getPublic());
    _local_user_certificate = _key_store.generateHummerCertificate(_local_user_alias);
    _key_store_sign1 = _key_store.getPersonalSignSignature();
    _key_store_verify1 = _key_store.getPersonalVerifySignature();
  }
}