/*
Java Hummer Project
File: Tool.java
Date: 4-14-99
Author: Chad Ward
Description:  The parent class of all tool objects.
              This defines

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.tool;

import java.util.Properties;
import java.net.InetAddress;
import java.net.UnknownHostException;

import edu.uidaho.hummer.util.HummerKeyStore;

/**
Parent class of all tools on the Hummer system.
@author Chad Ward
*/
public abstract class HummerTool implements Runnable {
	/**
	String representation of the host address that this tool is running on.
	*/
  protected String _host_address;

	/**
	String representation of the host name that this tool is running on.
	*/
  protected String _host_name;

	/**
	Name of the file that will be used for manipluation
	*/
  protected String _logfile;
  
	/**
	Name of the operating system that this tool is working in.
	*/
  protected String _operating_system;

	/**
	True if the tool is currently running, and false if it is not
	running.
	*/
  protected boolean _running;

	/**
	Thread used to wrap around this Runnable object.  This is done
	rather than having Messaging server extends Thread because once
	a thread completes the run() method it is considered dead...forever.
	So a new thread must be created each time that the server is started up
	again.
	*/
  protected Thread _wrapper_thread;

	/**
	Information about the current state of messaging server.  This will
	usually hold error information.  If MessagingServer is in a normal state,
	this will be null.
	*/
  protected String _status = null;

	/**
	ThreadGroup used to handle all the Thread processes.
	*/
  private ThreadGroup _thread_group;

	/**
	Key store used for logging into MessagingServer.
	*/
  protected HummerKeyStore _key_store;


	/**
	Amount of time (in milliseconds) that the tool should wait before gathering
	information again. (Current default: 300000 ms)
	*/
  protected long _wait_time = 300000;

	/**
	Name of the tool
	*/
  protected String _name;

	/**
	Retrieves the operating system name from the System properties,
	as well as the host name and host address.
	*/
  protected HummerTool(String name, ThreadGroup thread_group, HummerKeyStore key_store) {
    this._name = name;
    this._thread_group = thread_group;
    this._key_store = key_store;
    Properties sys_properties = System.getProperties();
    _operating_system = sys_properties.getProperty("os.name");
    try{
      _host_address = InetAddress.getLocalHost().getHostAddress();
      _host_name = InetAddress.getLocalHost().getHostName();
    }
    catch(UnknownHostException e){
      System.err.println("HummerTool:constructor: " + e);
      System.exit(0);
    }  
  }

	/**
	Returns the name of this tool.
	*/
  public final String getName() { return _name; }

	/**
	Method defining the part of the process for gathering
	system information to be placed in the Hummer database.
	*/
  public abstract void gatherInformation() throws Exception;

	/**
	Method returning a description of the concrete tool.
	This is a method (rather than a string) to insure that the child class defines
	a description.
	*/
  public abstract String getDescription();

	/**
	Returns the current state of the thread (running or not running).
	This method does not use the Thread.isRunning method, so the Thread
	may be performing a last operation before closing and this method
	will return false.
	*/
  public final boolean isRunning() { return _running; }


	/**
	Sets the _running field to the given boolean.
	If this tool is not currently running, then
	the start() method will be called
	*/
  public final void setRunning(boolean running) {
    if(running) {
      if(!this._running) {
        this._running = true;
        _wrapper_thread = new Thread(_thread_group, this);
        _wrapper_thread.start();
      }
    }
    else {
      this._running = running;
      if(!running) {
        _status = "Stopping.";
      }
    }
  }

	/**
	Returns the current value of _status.
	*/
  public final String getStatus() { return _status; }

	/**
	Method defining the order of operations that will be performed.
	<ul>
	  While _running == true, do the following:
	  <li> gatherInformation()
	  <li> sleep(_wait_time)
	</ul>
	*/
  public final void run() {
    try{
      while(_running){
        _status = null;
        gatherInformation();
        _wrapper_thread.sleep(_wait_time);
      }
    }
    catch(InterruptedException e) {
      _status = "Thread Interruption Error: " + e;
      System.err.println("HummerTool(" + this.getName() + "):run:" + e);
      _running = false;
    }
    catch(Exception e) {
      _status = "Error: " + e;
      System.err.println("Error in tool \"" + _name + "\":");
      e.printStackTrace();
      _running = false;
    }
    if(_status != null && !_status.startsWith("Error")) {
      _status = null;
    }
  }
}
