/*
Java Hummer Project
File: DefaultKeyStore.java
Date: 1-24-99
Author: Chad Ward
Description:  Object for tracking a user's authentication and access
              rights to the system.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util;

import java.io.*;
import java.util.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.security.KeyPair;
import java.security.Key;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStoreException;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidKeySpecException;
import edu.uidaho.hummer.dataobject.HummerCertificate;
import edu.uidaho.hummer.dataobject.NewUserValidator;
import edu.uidaho.hummer.dataobject.AliasInfo;
import edu.uidaho.hummer.dataobject.DefaultAliasInfo;


/**
Concrete instance of abstract HummerKeystore class.  This key store holds
information needed by the "default" protocol.  The default protocol
uses signatures for authentication, along with message digests for a first
time login.
<h4>Class Usage and Flow:</h4>
<ul>
  <li>Constructor calls superclass constructor with Properties object to initialize
      internal variables.
  <li> command - Handles command line arguments (only thing called by main method in HummerKeyStore)
  <li> addPeerAlias/addServerAlias/addUserAlias - Add an alias to the key store
  <li> generateHummerCertificate - Generates a HummerCertificate for the given alias.
  <li> getAliasStatistics/getAliasStatitisticsHeader - Used to get a read-out of the
            information for a given alias.  This method is used by command when getting
            a listing of aliases in the key store.
  <li> getPersonalHummerCertificate - Returns HummerCertificate holding the personal
              public key of the key store and signed by the personal private key of
              the key store.
  <li> getPersonalXXXX - Returns information held for the key store, meaning information
             for the owner of the key store.
  <li> getVerifySignature - Returns a Signature object that will verify signatures
                            for the given alias.
  <li> holdingHummerCertifiate - Returns true if there the given alias has granted
                                 a HummerCertifiate to this key store (signed with
                                 the alias' private key)
  <li> initializeAccount - Will initialize an account for an alias that has been
                           created and not yet initialized, or has been reset. The
                           message digest is created using the password, random value
                           and timestamp, and the PublicKey to be placed for the account.
                           This value can be retrieved using StaticMethods.getDigest
  <li> isCertifiateHonored - Will return true if the given alias is of type
                             AliasInofo._SERVER_ALIAS_TYPE and certificates_honored
                             is true.
  <li> isNewAccount - Returns true if the account has not been initialized after
                      being created, or the account has been reset.
  <li> load(byte[]) - Creates a new DefaultKeyStore with the given array of bytes
                      for the admin account password (every key store will have
                      an admin account because it is created for each new key store
                      and cannot be removed)
  <li> load(InputStream) - It is not recommended to use this method, because the
                           HummerKeyStore.load(boolean,String) method will
                           guarantee that the correct filename is used.
  <li> resetAlias - Resets the alias' account with a new password and removes
                    all key and certificate information.
  <li> setHummerCertifiate - Sets the HummerCertificate given by the alias for this
                             key store in the key store.
  <li> setKey - Sets the given PublicKey for the given alias' account.  The int value
                representing the kind of key is ignored, because there is only one
                type of key held for each account in this implementation of key store.
  <li> setServerTrust - The value for certificates_honored for _SERVER_ALIAS_TYPES is
                        set with this method.
  <li> store - Stores the key store to the given output stream.  This method is
               used by HummerKeyStore.store(boolean,String), but should not be used
               directly, because it does not guarantee that the correct filename is
               being used to store to.
                            
</ul>
@author Chad Ward
*/
public final class DefaultKeyStore extends HummerKeyStore {

	/**
	Table of Signature objects that have been initialized for verifying signatures.
	This table is completely filled when the DefaultKeyStore is loaded.
	The table is indexed by aliases in the DefaultKeyStore.  The Signature objects
	are initialized using the PublicKey of the alias that the table is referenced by.
	The reason for this table is because generating Signatures takes time, and the
	Signature class is not serializeable.
	*/
  private Hashtable _alias_verify_signatures = new Hashtable();

	/**
	Signature object used to generate HummerCertifiate objects and signatures.
	*/
  private Signature _signing_signature_object, _verifying_signature_object;


	/**
	Method used to handle command line arguements.  The main method in
	HummerKeyStore only calls this method.
	*/
  public void command(String[] argv) 
         throws IOException, KeyStoreException, ClassNotFoundException,
                NoSuchProviderException, NoSuchAlgorithmException,
                UnloadedKeyStoreException, UnknownAliasException,
                CertificateException {
    final String unrecognized_command_header =
            "\n---------------------------------------------------------------------------------------\n" +
            "Unrecognized command.  Correct format:\n";
    final String invalid_command_format_header =
            "\n---------------------------------------------------------------------------------------\n" +
            "Invalid command format.  Correct format:\n";
    final String description =
            "   java edu.uidaho.hummer.util.DefaultKeyStore <f key_store_filename>|<u user_name>|<s <l>|<address>> -command_type- " +
            "-value1-< -value2->\n\n" +
            "f option - Specify an explicit filename for the DefaultKeyStore\n" +
            "u option - Specify the username for the DefaultKeyStore\n" +
            "s option - Specify the address (name or IP) of the server that will be using the DefaultKeyStore (l - local system)\n" +
            "Command Descriptions\t\t\tCommand Name\t\tCommand Value\n" +
            "---------------------------------------------------------------------------------------\n" +
            "Create New KeyStore\t\t\tnew\t\t\t<admin_password>\n" +
            "List Aliases in KeyStore\t\tlist\t\t\tNone\n" +
            "List Access Rights in KeyStore\t\talias_rights\t\t<alias>\n" +
            "Add a user alias to the system\t\tadd_user\t\t<new_user_alias> <alias_password>\n" +
            "Add a peer alias to the system\t\tadd_peer\t\t<new_peer_alias> <alias_password>\n" +
            "Add a server alias to the system\tadd_server\t\t<new_alias_alias> <y|n> <alias_password>\n" +
            "\tNote: y = Certificates honored.  n = Certificates not honored.\n" +
            "Reset an account on the system\t\treset_alias\t\t<alias> <alias_password>\n" +
            "Remove an alias from system\t\tremove_alias\t\t<alias>\n" +
            "Update KeyStore signature key pair\tupdate_key_pair\t\tNone\n" +
            "Update outdated KeyStore signature\tupdate_old_key_pair\tNone\n" +
            "   key pair (if not outdated,\n" +
            "   then no action is taken)";
		/*
		command_table is a table of possible commands that could be passed to
		the key store.  Each of the values associated with each of the commands
		is the number of additional parameters are needed for the command.
		eg. 'add_user' requires the new alias and a password, so '2' is associated
		with the 'add_user' command.
		*/
    Hashtable command_table = new Hashtable();
    command_table.put("new", new Integer(1));
    command_table.put("list", new Integer(0));
    command_table.put("alias_rights", new Integer(1));
    command_table.put("add_user", new Integer(2));
    command_table.put("add_peer", new Integer(2));
    command_table.put("add_server", new Integer(3));
    command_table.put("reset_alias", new Integer(2));
    command_table.put("remove_alias", new Integer(1));
    command_table.put("update_key_pair", new Integer(0));
    command_table.put("update_old_key_pair", new Integer(0));

		/*
		command_list_length is the number of parameters that has
		not been previously used (for instance s l uses 2 parameters, so the
		command_list_length counter is decremented by 2).
		command_index is the current index in the argv parameter array to
		be parsed.
		A single varaible could be used, two are just used for clarity.
		*/
    int command_list_length = argv.length,
        command_index = 0;
    String alias = null;

		/*
		The first value to be parsed from the command lines is the type
		of key store (server, user or neither/f) so that the filename can
		be determined.
		*/
    if(argv.length < 1) {
      System.err.println(unrecognized_command_header + description);
      System.exit(-1);
    }
    String filename = null;
    if(argv[0].equals("f")) {
      if(argv.length < 2) {
        System.err.println(invalid_command_format_header + description);
        System.exit(-1);
      }
      filename = argv[1];
      command_list_length -= 2;
      command_index = 2;
    }
    else if(argv[0].equals("u")) {
      if(argv.length < 2) {
        System.err.println(invalid_command_format_header + description);
        System.exit(-1);
      }
      filename = DefaultKeyStore.getDefaultFilename(false, argv[1]);
      alias = argv[1];
      command_list_length -= 2;
      command_index = 2;
    }
    else if(argv[0].equals("s")) {
      if(argv.length < 2) {
        System.err.println(invalid_command_format_header + description);
        System.exit(-1);
      }
      String address = argv[1];
      alias = address;
      try {
        if(argv[1].equals("l")) {
          address = null;
          alias = java.net.InetAddress.getLocalHost().getHostAddress();
        }
        filename = DefaultKeyStore.getDefaultFilename(true, address);
        command_list_length -= 2;
        command_index = 2;
      }
      catch(java.net.UnknownHostException e) {
        System.err.println("UnknownHostException:\n" + unrecognized_command_header + description);
        System.exit(-1);
      }
    }
    else {
      System.err.println("Must specify information to know the filename of the DefaultKeyStore:\n" +
                         unrecognized_command_header + description);
      System.exit(-1);
    }

		/*
		Now that the filename has been determined, the validity of the rest
		arguments must be determined before continueing (otherwise NullPointerExceptions
		and IndexOutOfBoundsExceptions could be thrown)
		*/
    if(argv.length <= command_index) {
      System.err.println("Wrong number of arguments for the given command:\n" + unrecognized_command_header + description);
      System.exit(-1);
    }
    if(!command_table.containsKey(argv[command_index])) {
      System.err.println(unrecognized_command_header + description);
      System.exit(-1);
    }
    Integer command_number_arguments = (Integer)(command_table.get(argv[command_index]));
    if(command_number_arguments.intValue() != (command_list_length - 1)) {
        System.err.println(invalid_command_format_header + description);
        System.exit(-1);       
    }


		/*
		Validity of parameter lenght has been verified, so run the commands
		*/

		/*
		Creating a new key store is different from the other commands because
		the other commands can simply load the key store, but 'new' will
		have to create one.
		*/
    if(argv[command_index].equals("new")) {
      System.out.println("Creating new key store \"" + filename + "\".");
      this.load(argv[command_index + 1].getBytes());
      System.out.println("Have loaded a new HummerKeyStore.  Will now store...");
    }


    else {
      DataInputStream in_stream = null;
      try {
        in_stream = new DataInputStream(new FileInputStream(filename));
      }
      catch(FileNotFoundException e) {
        System.err.println("\nDefaultKeyStore:main: Could not find DefaultKeyStore with given filename (" +
                           filename + ")\n");
        System.exit(-1);
      }
      this.fileLoad(in_stream);
      in_stream.close();

		/*
		Gather information to display about the key store.
		*/
      Date expiration_date = this.getPersonalKeyPairExpiration().getTime();
      String expiration_string = DateFormat.getInstance().format(expiration_date);
      String key_store_info ="\nHummerKeyStore filename:\t" + filename;
      if(argv[0].equals("s")) {
        key_store_info += " (" + java.net.InetAddress.getByName(alias).getHostName() + ")";
      }
      key_store_info += "\nKeyPair expiration date:\t" + expiration_string +
                       "\n==============================================================" +
                       "===========================\n";

		/*
		List alias info in the key store
		*/
      if(argv[command_index].equals("list")) {
        String current_alias;
        final int MAX_ALIAS_LENGTH = 20,
                  MAX_KEY_LENGTH = 16;
        String[] header = this.getAliasStatisticsHeader();
        System.out.println(key_store_info +
                           statisticsToString(header, MAX_ALIAS_LENGTH, MAX_KEY_LENGTH) +
                           "\n---------------------------------------------------------------------" +
                           "--------------------");
        Enumeration enum = this.getAliases();
        Object[] alias_stats;
        String alias_type = null;
        int alias_type_value;
        while(enum.hasMoreElements()) {
          alias_stats = this.getAliasStatistics(enum.nextElement().toString());
          System.out.println(statisticsToString(alias_stats, MAX_ALIAS_LENGTH, MAX_KEY_LENGTH));
        }
        System.out.println("------------------------------------------------------------------" +
                           "-----------------------\n");

      }


		/*
		List alias rights
		*/
      else if(argv[command_index].equals("alias_rights")) {
        final int MAX_ACCESS_NAME_LENGTH = 25;
        char[] char_filler = new char[MAX_ACCESS_NAME_LENGTH - "Access Name".length()];
        Arrays.fill(char_filler, ' ');
        String string_filler = new String(char_filler);
        if(!this.contains(argv[command_index + 1])) {
          System.err.println("DefaultKeyStore:main: Unknown alias: " + (argv[command_index + 1]));
          System.exit(-1);
        }
        AliasInfo user_info = this.getAliasInfo(argv[command_index + 1]);
        Hashtable access_rights = user_info.getAccessTable();
        System.out.println(key_store_info);
        if(access_rights == null) {
          System.out.println("This account has a null access rights table.\n");
        }
        else {
          System.out.println("   Access Name" + string_filler + "Value\n" +
                             "-----------------------------------------------------------------------------");
          for(Enumeration enum = access_rights.keys(); enum.hasMoreElements();) {
            String current_key = (String)(enum.nextElement());
            System.out.print("   " + current_key);
            int spacing_left = MAX_ACCESS_NAME_LENGTH - current_key.length();
            for(int i = 0; i < spacing_left; i++) {
              System.out.print(" ");
            }
            System.out.println(access_rights.get(current_key));
          }
          System.out.println("-----------------------------------------------------------------------------\n");
        }
      }


		/*
		Add user alias
		*/
      else if(argv[command_index].equals("add_user")) {
        this.addUserAlias(argv[command_index + 1], argv[command_index + 2].getBytes(),
                                 StaticMethods.getDefaultUserAccessTable());
        System.out.println("User alias \"" + argv[command_index + 1] + "\" added to HummerKeyStore.");
      }

		/*
		Add peer alias
		*/
      else if(argv[command_index].equals("add_peer")) {
        this.addPeerAlias(argv[command_index + 1], argv[command_index + 2].getBytes());
        String new_server_address = null;
        try {
          new_server_address = java.net.InetAddress.getByName(argv[command_index + 1]).getHostAddress();
        }
        catch(java.net.UnknownHostException e) {
          System.err.println("The given peer server address cannot be found by this system (HostNotFoundException).");
          System.exit(-1);
        }
        System.out.println("Peer server alias \"" + new_server_address + "\" added to HummerKeyStore.");
      }


		/*
		Add server alias
		*/
      else if(argv[command_index].equals("add_server")) {
        if(!argv[command_index + 2].toLowerCase().equals("y") && !argv[command_index + 2].toLowerCase().equals("n")) {
          System.err.println("\nMust use either \"y\" or \"n\" for server trust.(recieved: " + argv[command_index + 1] + ")\n" +
                             unrecognized_command_header + description);
          System.exit(-1);
        }
        boolean certificates_honored = argv[command_index + 2].toLowerCase().equals("y");
        String new_server_address = null;
        try {
          new_server_address = java.net.InetAddress.getByName(argv[command_index + 1]).getHostAddress();
        }
        catch(java.net.UnknownHostException e) {
          System.err.println("The given server address cannot be found by this system (HostNotFoundException).");
          System.exit(-1);
        }
        this.addServerAlias(new_server_address, argv[command_index + 3].getBytes(), certificates_honored,
                                 StaticMethods.getDefaultServerAccessTable());
        System.out.println("Server alias \"" + new_server_address + "\" added to HummerKeyStore.");
      }


		/*
		Reset alias information
		*/
      else if(argv[command_index].equals("reset_alias")) {
        this.resetAlias(argv[command_index + 1], argv[command_index + 2].getBytes());
        System.out.println("Alias \"" + argv[command_index + 1] + "\" reset.");
      }


		/*
		Remove an alias from the key store
		*/
      else if(argv[command_index].equals("remove_alias")) {
        this.removeAlias(argv[command_index + 1]);
        System.out.println("Alias \"" + argv[command_index + 1] + "\" removed.");
      }


		/*
		Force an update of the key stores KeyPair
		*/
      else if(argv[command_index].equals("update_key_pair")) {
        this.updateKeyPair(true);
        System.out.println("KeyPair for HummerKeyStore changed.");
      }


		/*
		Update the key store's key pair only if it is old
		*/
      else if(argv[command_index].equals("update_old_key_pair")) {
        this.updateKeyPair(false);
        System.out.println("KeyPair for HummerKeyStore is updated.");
      }

		/*
		Print an error because something has gone wrong (unrecognized command
		should have already been caught before this is reached)
		*/
      else {
        System.out.println(unrecognized_command_header + description);
        System.exit(-1);
      }
    }

		/*
		Write out the key store
		*/
    DataOutputStream out_stream = new DataOutputStream(new FileOutputStream(filename));
    this.store(out_stream);
  }



	/**
	Default constructor setting the local constants object
	*/
  public DefaultKeyStore(Properties constants) {
    super(constants);
  }
	/**
	Private method that is used internally to only load values from an
	input stream without initializing any Signature objects. (main method uses this also)
	*/
  protected void fileLoad(InputStream input_stream)
            throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                   KeyStoreException {
    try {
      ObjectInputStream object_input_stream = new ObjectInputStream(input_stream);
      _alias_table = (Hashtable)(object_input_stream.readObject());
      _key_pair = (KeyPair)(object_input_stream.readObject());
      _key_pair_expiration_date = (GregorianCalendar)(object_input_stream.readObject());
      _backup_keys = (Vector[])(object_input_stream.readObject());
    }
    catch(InvalidClassException e) {
      throw new KeyStoreException("DefaultKeyStore:fileLoad: " +
                                  "Cannot load DefaultKeyStore with given InputStream (InvalidClass error).");
    }
    catch(ClassNotFoundException e) {
      throw new KeyStoreException("DefaultKeyStore:fileLoad: " +
                                  "Cannot load DefaultKeyStore with given InputStream.(ClassNotFound error)");
    }
    _has_loaded = true;
  }

	/**
	Loads this DefaultKeyStore from the given input stream.
	@param input_stream InputStream object where the key store can be loaded from.
	@throws KeyStoreException There was an error when reading from the InputStream, so that the objects
		for the DefaultKeyStore could not be created.
	*/
  public void load(InputStream input_stream)
         throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                KeyStoreException {
    try {
      KeyPair old_key_pair = _key_pair;
      this.fileLoad(input_stream);
      _signing_signature_object = StaticMethods.getSigningSignatureObject(_key_pair.getPrivate());
      _verifying_signature_object = StaticMethods.getVerifyingSignatureObject(_key_pair.getPublic());
      _hummer_certificate = new HummerCertificate(_signing_signature_object, _key_pair.getPublic(),
                                                  null, _key_pair_expiration_date);
      String current_alias;
      AliasInfo current_alias_info;
      for(Enumeration enum = _alias_table.keys(); enum.hasMoreElements();) {
        current_alias = enum.nextElement().toString();
        current_alias_info = (AliasInfo)_alias_table.get(current_alias);
        if(!current_alias_info.isNewAccount()) {
          PublicKey public_key = (PublicKey)current_alias_info.getKey(AliasInfo._PUBLIC_KEY_TYPE);
          Signature signature_object = StaticMethods.getVerifyingSignatureObject(public_key);
          _alias_verify_signatures.put(current_alias, signature_object);
        }
      }
    }
    catch(InvalidClassException e) {
      throw new KeyStoreException("DefaultKeyStore:load: " +
                                  "Cannot load DefaultKeyStore with given InputStream (InvalidClass error).");
    }
    catch(ClassNotFoundException e) {
      throw new KeyStoreException("DefaultKeyStore:load: " +
                                  "Cannot load DefaultKeyStore with given InputStream.(ClassNotFound error)");
    }
    catch(NoSuchAlgorithmException e) {
      throw new KeyStoreException("DefaultKeyStore:load: " +
                                  "Cannot load DefaultKeyStore with given InputStream.(ClassNotFound error)");
    }
    catch(InvalidKeyException e) {
      throw new KeyStoreException("DefaultKeyStore:load: " +
                                  "Cannot load DefaultKeyStore with given InputStream.(InvalidKey error)");
    }
    catch(CertificateException e) {
      throw new KeyStoreException("DefaultKeyStore:load: " +
                                  "Cannot load DefaultKeyStore with given InputStream.(InvalidKey error)");
    }
  }

	/**
	Creates a DefaultKeyStore with a new KeyPair and an 'admin' account with the given
	password.  The admin account is created using the addUserAlias method using
      alias="admin" and password=admin_password.
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li> Create a new Hashtable for the alias information
	  <li> Generate a new KeyPair for the key store
	  <li> Create a new expiration date and set its value to the number of days
	       after the current date specified by value const.LENGTH_KEY_PAIR_VALIDITY
	       in the Hummer.properties file
	  <li> Generate Signature objects for signing and verifying data
	  <li> Create a HummerCertificate to vadidate the key stores keys
	  <li> Create data structures for backup key tracking
	  <li> Add 'admin' alias to the key store with the given password
	*/
  public void load(byte[] admin_password)
         throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                KeyStoreException, NoSuchAlgorithmException, NoSuchProviderException, CertificateException {
    _has_loaded = true;
    _alias_table = new Hashtable();
    _key_pair = StaticMethods.generateKeyPair();
    _key_pair_expiration_date = new GregorianCalendar();
    String key_pair_validity = _constants.getProperty("const.LENGTH_KEY_PAIR_VALIDITY");
    int validity_length = (new Integer(key_pair_validity)).intValue();
    _key_pair_expiration_date.add(Calendar.DATE, validity_length);
    try {
      _signing_signature_object = StaticMethods.getSigningSignatureObject(_key_pair.getPrivate());
      _verifying_signature_object = StaticMethods.getVerifyingSignatureObject(_key_pair.getPublic());
    }
    catch(InvalidKeyException e) {
      throw new KeyStoreException("DefaultKeyStore:load: " +
                                  "Cannot load DefaultKeyStore: " + e);
    }
    if(key_pair_validity == null) {
      System.err.println("DefaultKeyStore:load(String): key_pair String in Hummer.properties is null.");
      System.exit(-1);
    }
    _hummer_certificate = new HummerCertificate(_signing_signature_object,
                                                _key_pair.getPublic(),
                                                null, _key_pair_expiration_date);
    _backup_keys = new Vector[3];
    _backup_keys[_BACKUP_TIMESTAMP_INDEX] = new Vector();
    _backup_keys[_BACKUP_PUBLIC_KEY_INDEX] = new Vector();
    _backup_keys[_BACKUP_PRIVATE_KEY_INDEX] = new Vector();
    try {
      this.addUserAlias("admin", admin_password, new Hashtable());
    }
    catch(UnloadedKeyStoreException e) {
      throw new KeyStoreException("DefaultKeyStore:load: Programming error.  " +
                                  "UnloadedKeyStoreException caught after DefaultKeyStore was loaded.");
    }
  }

	/**
	Resets the information for the account represented by <i>alias</i>.  After calling this method,
	isNewAccount will return true and there will no longer be any keys or HummerCertificates.
	@param new_password The password that will be associated with this account after it has been
		reset.
	*/
  public void resetAlias(String alias, byte[] new_password)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:resetAlias: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)_alias_table.get(alias);
    alias_info.reset(new_password);
    _has_changed = true;
    _alias_verify_signatures.remove(alias);
  }

	/**
	Adds a new alias to the key store for a server (not a user) with a password and trust value.
	The alias and password are placed into the NewUserValidator object for later validation when the server
	logs in for the first time. The access table will be null.
	@param password Temporary password of the account to be changed upon the first login of the user
	@param certificates_honored True if certificates signed by this server will be give the holder access
		to this system, false if it is only another server that holds a local account.
	@param access_table Table of access priveledges given to the server.
	@throws KeyStoreException The given alias cannot already be in the system.
	@see DefaultKeyStore#addUserAlias(String, byte[], Hashtable)
	@see DefaultKeyStore#addPeerAlias(String, byte[])
	*/
  public void addServerAlias(String alias, byte[] password,
                             boolean certificates_honored, Hashtable access_table)
         throws UnloadedKeyStoreException, KeyStoreException {
    checkLoaded();
    if(alias == null || alias.equals("")) {
      throw new KeyStoreException("DefaultKeyStore:addServerAlias: " +
                                  "Must supply a non-null/non-empty alias.");
    }
    if(this.contains(alias)) {
      return;
    }
    DefaultAliasInfo alias_info;
    alias_info = new DefaultAliasInfo(AliasInfo._SERVER_ALIAS_TYPE, certificates_honored,
                                      password, access_table);
    _alias_table.put(alias, alias_info);
    _has_changed = true;
  }

	/**
	Adds a new alias to the key store for a peer with a password (access tables are equal for all peer servers).
	The alias and password are placed into the NewUserValidator object for later validation when the server
	logs in for the first time. The access table will be null.
	@param password Temporary password of the account to be changed upon the first login of the user
	@throws KeyStoreException The given alias cannot already be in the system.
	@see DefaultKeyStore#addUserAlias(String, byte[], Hashtable)
	@see DefaultKeyStore#addPeerAlias(String, byte[])
	*/
  public void addPeerAlias(String alias, byte[] password) throws UnloadedKeyStoreException, KeyStoreException {
    checkLoaded();
    if(alias == null || alias.equals("")) {
      throw new KeyStoreException("DefaultKeyStore:addPeerAlias: " +
                                  "Must supply a non-null/non-empty alias.");
    }
    if(this.contains(alias)) {
      return;
    }
    DefaultAliasInfo alias_info = new DefaultAliasInfo(AliasInfo._PEER_ALIAS_TYPE,
                                                       false, password,
                                                       StaticMethods.getDefaultPeerAccessTable());
    _alias_table.put(alias, alias_info);
    _has_changed = true;
  }

	/**
	Adds a new alias to the key store for a user (not a server) with a password and a trust value.
	The alias and password are placed into the NewUserValidator object for later validation when the server
	logs in for the first time. The access table will be null.
	@param alias Username
	@param password Temporary password of the account to be changed upon the first login of the user
	@throws KeyStoreException The given alias cannot already be in the system.
	@see DefaultKeyStore#addServerAlias(String, byte[], boolean, Hashtable)
	*/
  public void addUserAlias(String alias, byte[] password, Hashtable access_table)
         throws UnloadedKeyStoreException, KeyStoreException {
    checkLoaded();
    if(alias == null || alias.equals("")) {
      throw new KeyStoreException("DefaultKeyStore:addUserAlias: " +
                                  "Must supply a non-empty alias.");
    }
    if(this.contains(alias)) {
      return;
    }
    DefaultAliasInfo alias_info = new DefaultAliasInfo(AliasInfo._USER_ALIAS_TYPE,
                                                       false, password, access_table);
    _alias_table.put(alias, alias_info);
    _has_changed = true;
  }

	/**
	Generates a new certificate for the given alias.  Placing the alias' PublicKey, access_table,
	and the expiration date of the signing PrivateKey into the HummerCertificate.  The
	HummerCertificate is signed using the DefaultKeyStore's PublicKey associated with the alias.
	*/
  public HummerCertificate generateHummerCertificate(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException, CertificateException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:generateHummerCertificate: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    if(!alias_info.isNewAccount()) {
      return new HummerCertificate(_signing_signature_object,
                                   (PublicKey)alias_info.getKey(AliasInfo._PUBLIC_KEY_TYPE),
                                   alias_info.getAccessTable(), _key_pair_expiration_date);
    }
    else {
      throw new KeyStoreException("DefaultKeyStore:generateHummerCertificate: " +
                                  "Cannot create a HummerCertificate for an account " +
                                  "that has not been initialized.");
    }
  }

	/**
	Returns the first alias found associated with the given PublicKey.
	Does not check whether the key store is loaded.
	@returns null if the PublicKey is not associated with any of the
		aliases in the key store.
	*/
  private String getAlias(PublicKey public_key) {
    String return_value = null, current_alias;
    AliasInfo current_info;
    for(Enumeration enum = _alias_table.keys(); enum.hasMoreElements();) {
      current_alias = enum.nextElement().toString();
      current_info = (AliasInfo)_alias_table.get(current_alias);
      if(!current_info.isNewAccount() &&
        current_info.getKey(AliasInfo._PUBLIC_KEY_TYPE).equals(public_key)) {
        return_value = current_alias;
        break;
      }
    }
    return return_value;
  }

	/**
	Returns the header to be used when displaying alias statistics.  For
	any implementation of a DefaultKeyStore, the first three values should be:
	String[0]=Alias
	String[1]=Alias Type
	String[2]=Initialized
	@see DefaultKeyStore#getAliasStatistics(String)
	*/
  public final String[] getAliasStatisticsHeader() {
    String[] return_value = new String[5];
    return_value[0] = "Alias";
    return_value[1] = "Alias Type";
    return_value[2] = "Initialized";
    return_value[3] = "Cert. Honored";
    return_value[4] = "Holding Cert.";
    return return_value;
  }

	/**
	Returns values for a given alias.  The meanings of these can be gathered
	from getAliasStatisticHeader.
	For any implementation of a DefaultKeyStore, the first three values should be:
	String[0]=Alias
	String[1]=Alias Type
	String[2]=Initialized
	@see DefaultKeyStore#getAliasStatisticsHeader()
	*/
  public final Object[] getAliasStatistics(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:getAliasStatistics: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    Object[] return_value = new Object[5];
    DefaultAliasInfo alias_info = (DefaultAliasInfo)_alias_table.get(alias);
    return_value[0] = alias;
    return_value[1] = new Integer(alias_info.getType());
    return_value[2] = new Boolean(!alias_info.isNewAccount());
    if(alias_info.getType() == AliasInfo._SERVER_ALIAS_TYPE) {
      return_value[3] = new Boolean(alias_info.isCertificateHonored());
    }
    else {
      return_value[3] = Boolean.FALSE;
    }
    if(alias_info.getType() == AliasInfo._SERVER_ALIAS_TYPE ||
       alias_info.getType() == AliasInfo._PEER_ALIAS_TYPE) {
      return_value[4] = new Boolean(alias_info.holdingHummerCertificate());
    }
    else {
      return_value[4] = Boolean.FALSE;
    }
    return return_value;
  }

	/**
	Finds the given PublicKey in the _backup_keys array.  Does not check that DefaultKeyStore is loaded
	or that the given alias is valid. <br>
	<B> This method should be replaced by getBackupKeyIndex(int,Key), where int is
	the type of value in the backup Vector and Key is the value to search for </B>
	@returns If timetamp is found in the _backup_keys array, then it is returned.  If
		the timestamp equals -1, then -1 is returned, otherwise -2 is returned
		(timestamp does not exist in list of backup keys).
	@see DefaultKeyStore#getBackupKeyIndex(PrivateKey)
	*/
  private int getBackupKeyIndex(PublicKey public_key) {
    int return_value = -2;
    if(public_key != null) {
      if(public_key.equals(_key_pair.getPublic())) {
        return -1;
      }
      return_value = _backup_keys[_BACKUP_PUBLIC_KEY_INDEX].indexOf(public_key);
      if(return_value < 0) {
        return_value = -2;
      }
    }
    return return_value;
  }

	/**
	Finds the given PrivateKey in the _backup_keys array.  Does not check that DefaultKeyStore is loaded
	or that the given alias is valid. <Br>
	<B> This method should be replaced by getBackupKeyIndex(int,Key), where int is
	the type of value in the backup Vector and Key is the value to search for </B>
	@returns If timetamp is found in the _backup_keys array, then it is returned.  If
		the timestamp equals -1, then -1 is returned, otherwise -2 is returned
		(timestamp does not exist in list of backup keys).
	@see DefaultKeyStore#getBackupKeyIndex(PublicKey)
	*/
  private int getBackupKeyIndex(PrivateKey private_key) {
    int return_value = -2;
    if(private_key != null) {
      if(private_key.equals(_key_pair.getPublic())) {
        return -1;
      }
      return_value = _backup_keys[_BACKUP_PRIVATE_KEY_INDEX].indexOf(private_key);
      if(return_value < 0) {
        return_value = -2;
      }
    }
    return return_value;
  }

	/**
	Gets a MessageDigest byte array for the given alias using the password of the account, along
	with the given random timestamp bytes (StaticMethods.getRandomTimestampBytes())
	and a PublicKey.  This will only work for accounts that have not been initialized, because
	the password is removed during initialization.
	*/
  public byte[] getDigest(String alias, byte[] random_timestamp, PublicKey public_key)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException,
                NoSuchAlgorithmException, IOException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:getDigest: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    DefaultAliasInfo alias_info = (DefaultAliasInfo)(_alias_table.get(alias));
    if(alias_info.isNewAccount()) {
      return alias_info.getNewUserValidator().getDigest(random_timestamp, public_key);
    }
    else {
      throw new KeyStoreException("DefaultKeyStore:getDigest: Cannot retrieve message digest with an account " +
                                  "that has already been initialized.(" + alias + ")");
    }
  }

	/**
	True if the alias is a server or peer and that account holds a HummerCertificate
	for the alias.
	*/
  public boolean holdingHummerCertificate(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:holdingHummerCertificate: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    DefaultAliasInfo alias_info = (DefaultAliasInfo)_alias_table.get(alias);
    return alias_info.holdingHummerCertificate();
  }


	/**
	Gets a HummerCertificate containing the key store's current public signature key.
	The HummerCertificate is signed with the PrivateKey associated with the given alias.
	@see DefaultKeyStore#getPersonalKeyPair(int,Object)
	*/
  public HummerCertificate getPersonalHummerCertificate(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException,
                KeyStoreException, CertificateException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:getPersonalHummerCertificate: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    HummerCertificate return_value = null;
    AliasInfo alias_info = (AliasInfo)_alias_table.get(alias);
    if(alias_info.isNewAccount()) {
      throw new KeyStoreException("DefaultKeyStore:getPersonalHummerCertificate: " +
                                  "Cannot create a HummerCertificate for an account " +
                                  "that has not been initialized.");
    }
    int index = getBackupKeyIndex(alias);
    if(index == -1) {
      return_value = _hummer_certificate;
    }
    else if(index < 0) {
      return_value = null;
    }
    else {
      return_value = new HummerCertificate(getPersonalSignSignatureInternal(getBackupKeyIndex(alias)),
                                            _key_pair.getPublic(),
                                            null, _key_pair_expiration_date);
    }
    return return_value;
  }



	/**
	Gets a Signature object used to sign data with the DefaultKeyStore's
	current PrivateKey
	*/
  public Signature getPersonalSignSignature() throws UnloadedKeyStoreException {
    checkLoaded();
    return _signing_signature_object;
  }

	/**
	Finds the KeyPair associated with the given alias, generates
	a Signature object, and uses the found PublicKey to initialize
	the Signature object for signing.
	*/
  public Signature getPersonalSignSignature(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:getPersonalVerifySignature: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    int index = getBackupKeyIndex(alias);
    return getPersonalSignSignatureInternal(index);
  }


	/**
	Finds the KeyPair associated with the given PublicKey, generates
	a Signature object, and uses the found PublicKey to initialize
	the Signature object for signing.
	*/
  public Signature getPersonalSignSignature(PublicKey public_key)
         throws UnloadedKeyStoreException, KeyStoreException {
    this.checkLoaded();
    int index = getBackupKeyIndex(public_key);
    return getPersonalSignSignatureInternal(index);
  }

	/**
	Finds the KeyPair associated with the given alias, generates
	a Signature object, and uses the found PublicKey to initialize
	the Signature object. (Does not check for DefaultKeyStore loaded, or that alias exists)
	*/
  private Signature getPersonalSignSignatureInternal(int index)
          throws KeyStoreException {
    if(index == -1) {
      return _signing_signature_object;
    }
    else if(index < 0) {
      return null;
    }
    else {
      PrivateKey private_key = (PrivateKey)_backup_keys[_BACKUP_PRIVATE_KEY_INDEX].get(index);
      try {
        return StaticMethods.getSigningSignatureObject(private_key);
      }
      catch(NoSuchAlgorithmException e) {
        throw new KeyStoreException("DefaultKeyStore:getPersonalSignSignatureInternal: " +
                                    "System does not support signature algorithm: " + e);
      }
      catch(InvalidKeyException e) {
        throw new KeyStoreException("DefaultKeyStore:getPersonalSignSignatureInternal: " + e);
      }
    }
  }

	/**
	Gets a Signature object used to verify signatures with the DefaultKeyStore's
	current PublicKey.
	*/
  public Signature getPersonalVerifySignature() throws UnloadedKeyStoreException {
    checkLoaded();
    return _verifying_signature_object;
  }


	/**
	Finds the KeyPair associated with the given alias, generates
	a Signature object, and uses the found PublicKey to initialize
	the Signature object.
	*/
  public Signature getPersonalVerifySignature(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:getPersonalVerifySignature: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    return getPersonalVerifySignatureInternal(alias);
  }

	/**
	Finds the KeyPair associated with the given alias, generates
	a Signature object, and uses the found PublicKey to initialize
	the Signature object. (Does not check for DefaultKeyStore loaded, or that alias exists)
	*/
  private Signature getPersonalVerifySignatureInternal(String alias)
          throws KeyStoreException {
    int index = getBackupKeyIndex(alias);
    if(index == -1) {
      return _verifying_signature_object;
    }
    else if(index < 0) {
      return null;
    }
    else {
      PublicKey public_key = (PublicKey)_backup_keys[_BACKUP_PUBLIC_KEY_INDEX].get(index);
      try {
        return StaticMethods.getVerifyingSignatureObject(public_key);
      }
      catch(NoSuchAlgorithmException e) {
        throw new KeyStoreException("DefaultKeyStore:getPersonalVerifySignatureInternal: " +
                                    "System does not support signature algorithm: " + e);
      }
      catch(InvalidKeyException e) {
        throw new KeyStoreException("DefaultKeyStore:getVerifyingSignatureObject: " + e);
      }
    }
  }


	/**
	Returns a Signature object initialized with the PublicKey for <i>alias</i> for
	verifying data.  If the Signature object for the given alias has not already been
	generated previously, then it must be generated.
	@throws KeyStoreException The Signature object could not be initialized.
	*/
  public Signature getVerifySignature(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:getVerifySignature: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }      
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    if(alias_info.isNewAccount()) {
      throw new KeyStoreException("DefaultKeyStore:getVerifySignature(String): " +
                                  "Cannot retrieve signature for account that has not been initialized (" + alias + ").");
    }
    if(!_alias_verify_signatures.contains(alias)) {
      PublicKey public_key = (PublicKey)alias_info.getKey(AliasInfo._PUBLIC_KEY_TYPE);
      try {
        _alias_verify_signatures.put(alias, StaticMethods.getVerifyingSignatureObject(public_key));
      }
      catch(NoSuchAlgorithmException e) {
        throw new KeyStoreException("DefaultKeyStore:getVerifySignature: " +
                                    "Could not create verifying Signature object from " +
                                    alias + "'s PublicKey: " + e);
      }
      catch(InvalidKeyException e) {
        throw new KeyStoreException("DefaultKeyStore:getVerifySignature: " +
                                    "Could not create verifying Signature object from " +
                                    alias + "'s PublicKey: " + e);
      }
    }
    return (Signature)(_alias_verify_signatures.get(alias));
  }

	/**
	Searches for the first alias that matches the given PublicKey, and returns
	the Signature object to verify signatures signed by that alias.
	@see DefaultKeyStore#getVerifySignature(String)
	*/
  public Signature getVerifySignature(PublicKey public_key)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    String alias = this.getAlias(public_key);
    if(alias == null) {
      throw new UnknownAliasException("DefaultKeyStore:getVerifySignature: " +
                                      "Could not find the alias associated with the given PublicKey.");
    }
    return getVerifySignature(alias);
  }

	/**
	Verifies that password-digest is correct, then sets the public key and -1 for timestamp, and
	removes NewUserValidator from alias info.
	@return A new HummerCertificate using the given PublicKey, access table for alias, and
	the date that the signing PrivateKey expires.
	*/
  public HummerCertificate initializeAccount(String alias, byte[] message_digest,
                                             byte[] random_timestamp, Key key)
         throws UnloadedKeyStoreException, KeyStoreException, UnknownAliasException, CertificateException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:initializeAccount: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    if(key == null) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " +
                                  "PublicKey must be a non-null (valid) value.");
    }
    DefaultAliasInfo alias_info = (DefaultAliasInfo)(_alias_table.get(alias));
    if(!alias_info.isNewAccount()) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " +
                                  "Cannot initialize an account that has already been initialized.");
    }
    NewUserValidator validator = alias_info.getNewUserValidator();
    boolean valid_password = false;
    try {
      valid_password = validator.isValidPassword(message_digest, random_timestamp, (PublicKey)key);
    }
    catch(NoSuchAlgorithmException e) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " + e);
    }
    catch(IOException e) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " + e);
    }
    if(!valid_password) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " +
                                  "Invalid password recieved.");
    }
    alias_info.initializeAccount(key);
    _has_changed = true;
    try {
      _alias_verify_signatures.put(alias, StaticMethods.getVerifyingSignatureObject((PublicKey)key));
    }
    catch(NoSuchAlgorithmException e) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " +
                                  "System does not support signature algorithm: " + e);
    }
    catch(InvalidKeyException e) {
      throw new KeyStoreException("DefaultKeyStore:initializeAccount: " + e);
    }
    HummerCertificate return_value = this.generateHummerCertificate(alias);
    return return_value;
  }

	/**
	Finds if the account for the given alias has been initialized with keys
	@param alias Alias of the account to search for
	@returns True if the account has been initialized, false otherwise
	*/
  public boolean isNewAccount(String alias) throws UnknownAliasException, UnloadedKeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:isNewAccount: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)_alias_table.get(alias);
    return alias_info.isNewAccount();
  }

	/**
	Determines whether certificates given by this alias should be honored and
	treat owners with priveledges given in HummerCertificate (user aliases always return false).
	@param alias Alias to search for.
	@see edu.uidaho.hummer.dataobject.HummerCertificate
	*/
  public boolean isCertificateHonored(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:isCertificateHonored: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    DefaultAliasInfo alias_info = (DefaultAliasInfo)(_alias_table.get(alias));
    if(alias_info.getType() != AliasInfo._SERVER_ALIAS_TYPE) {
      return false;
    }
    return alias_info.isCertificateHonored();
  }

	/**
	Sets the public signature key for the given alias.
	@param alias Alias of account to change PublicKey for
	@param ignored_value Normally this is an index into an arry of keys for the
		alias' account, but this value is ignored because there is only one key
		type.
	@param new_key The new key for <i>alias</i>
	@throws KeyStoreException The account for the given alias has not been initialized.
	*/
  public void setKey(int ignored_value, String alias, Key new_key)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:setPublicSignatureKey: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    PublicKey new_public_key = (PublicKey)new_key;
    if(alias_info.isNewAccount()) {
      throw new KeyStoreException("DefaultKeyStore:setPublicSignatureKey: " +
                                  "Cannot change PublicKey for account that has not been initialized " +
                                  alias + ".");
    }
		/*
		Before setting the value in the key store, make sure that it is
		actually different.  Placing the PublicKey is not a problem, but
		generating Signature objects can be.
		*/
    if(!new_key.equals(alias_info.getKey(AliasInfo._PUBLIC_KEY_TYPE))) {
      alias_info.setKey(AliasInfo._PUBLIC_KEY_TYPE, new_public_key);
      _has_changed = true;
      try {
        _alias_verify_signatures.put(alias, StaticMethods.getVerifyingSignatureObject(new_public_key));
      }
      catch(NoSuchAlgorithmException e) {
        throw new KeyStoreException("DefaultKeyStore:setPublicSignatureKey: " +
                                    "System does not support signature algorithm: " + e);
      }
      catch(InvalidKeyException e) {
        throw new KeyStoreException("DefaultKeyStore:setPublicSignatureKey: " + e);
      }
    }
  }

	/**
	Sets the the value for a server to alias representing whether HummerCertificates granted by the server
	are trusted or not.  (true=Holder's of HummerCertificates signed by this server can login to the machine,
	false=HummerCertificates signed by this server have no meaning)
	@throws KeyStoreException The alias type for the given alias is not equal to _SERVER_ALIAS_TYPE.
	*/
  public void setServerTrust(String server_alias, boolean certificates_honored)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(server_alias)) {
      throw new UnknownAliasException("DefaultKeyStore:setServerTrust: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + server_alias + ")");
    }
    DefaultAliasInfo alias_info = (DefaultAliasInfo)(_alias_table.get(server_alias));
    if(alias_info.getType() != AliasInfo._SERVER_ALIAS_TYPE) {
      throw new KeyStoreException("DefaultKeyStore:setServerTrust: " +
                                  "Cannot set the HummerCertificate trust for a non-server alias.");
    }
    alias_info.setCertificateHonored(certificates_honored);
    _has_changed = true;
  }

	/**
	Assigns the PublicKey to the given alias.
	@param alias Alias used to access key
	@param certificate Certificate to be associated with the alias
	@param certificate_honored Specifies whether the certificates given out by this alias will be given
	honored as allowing users to login with rights delegated by this alias
	(authentication forwarding - Is this alias a trusted Hummer?)
	@throws KeyStoreException The account for the given alias is not for a server, the account has not
		been initialized, the certificate does not contain the most recent PublicKey, or the certificate
		cannot be verified with the server's PublicKey (could also be outdated).
	*/
  public void setHummerCertificate(String alias, HummerCertificate certificate) 
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException, CertificateException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("DefaultKeyStore:setHummerCertificate: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    if(alias_info.getType() == AliasInfo._USER_ALIAS_TYPE) {
      throw new KeyStoreException("DefaultKeyStore:setHummerCertificate: " +
                                  "Attempted to place a certificate for a user. " +
                                  "Only servers can give out HummerCertificates.");
    }
    if(alias_info.isNewAccount()) {
      throw new KeyStoreException("DefaultKeyStore:setHummerCertificate: " +
                                  "Cannot set a HummerCertificate for an account (" + alias + ") that" +
                                  "has not been initialized.");
    }
    if(!certificate.getPublicKey().equals(_key_pair.getPublic())) {
      throw new KeyStoreException("DefaultKeyStore:setHummerCertificate: " +
                                  "Certificate must contain DefaultKeyStore's most recent PublicKey.");
    }
    try {
      Signature verify_signature = null;
      if(_alias_verify_signatures.containsKey(alias)) {
        verify_signature = (Signature)_alias_verify_signatures.get(alias);
      }
      else {
        PublicKey public_key = (PublicKey)alias_info.getKey(AliasInfo._PUBLIC_KEY_TYPE);
        verify_signature = StaticMethods.getVerifyingSignatureObject(public_key);
      }
      if(!certificate.isValid(verify_signature)) {
        throw new KeyStoreException("DefaultKeyStore:setCertificate: " +
                                    "Certificate could not be verified " +
                                    "with the PublicKey for \"" + alias + "\".");
      }
    }
    catch(NoSuchAlgorithmException e) {
      throw new KeyStoreException("DefaultKeyStore:setHummerCertificate: " +
                                  "System does not support signature algorithm: " + e);
    }
    catch(InvalidKeyException e) {
      throw new KeyStoreException("DefaultKeyStore:setHummerCertificate: " + e);
    }
    alias_info.setHummerCertificate(certificate);
    _has_changed = true;
  }


	/**
	Stores this DefaultKeyStore to the given output stream.
	@param output_stream OutputStream object where the key store will be stored to.
	*/
  public void store(OutputStream output_stream)
         throws IOException, UnloadedKeyStoreException {
    this.checkLoaded();
    if(_has_changed) {
      if(output_stream != null) {
        ObjectOutputStream object_output_stream = new ObjectOutputStream(output_stream);
        object_output_stream.writeObject(_alias_table);
        object_output_stream.writeObject(_key_pair);
        object_output_stream.writeObject(_key_pair_expiration_date);
        object_output_stream.writeObject(_backup_keys);
        object_output_stream.close();
        _has_changed = false;
      }
    }
  }

	/**
	Replaces _key_pair, places previous value in _backup_keys along with a timestamp
	representing the time that the keypair had expired. <br>
	<b>Note:</b><br>
	All accounts (alias') are check for the timestamp associated with them.  If it is -1, then it
	is set the timestamp of the KeyPair being replaced
      @param force_update If this is true then a new _key_pair will be generated. If it is false,
		then a new _key_pair will only be generated if it has expired.
	*/
  public boolean updateKeyPair(boolean force_update)
         throws UnloadedKeyStoreException, NoSuchAlgorithmException,
                NoSuchProviderException, CertificateException, KeyStoreException {
    this.checkLoaded();
    if(force_update || (new GregorianCalendar()).after(_key_pair_expiration_date)) {
      if(_key_pair != null) {
		/*
		If the maximum number of backup keys has already been reached, then
		remove backup keys and their dates until there is one less than
		the maximum (a space for a new backup key
		*/
        int number_backup_keys = (new Integer(_constants.getProperty("const.NUMBER_BACKUP_KEYS"))).intValue();
        while(_backup_keys[_BACKUP_TIMESTAMP_INDEX].size() >= number_backup_keys) {
          _backup_keys[_BACKUP_TIMESTAMP_INDEX].remove(0);
          _backup_keys[_BACKUP_PUBLIC_KEY_INDEX].remove(0);
          _backup_keys[_BACKUP_PRIVATE_KEY_INDEX].remove(0);
        }
        long old_timestamp = _key_pair_expiration_date.getTime().getTime();
        _backup_keys[_BACKUP_TIMESTAMP_INDEX].add(new Long(old_timestamp));
        _backup_keys[_BACKUP_PUBLIC_KEY_INDEX].add(_key_pair.getPublic());
        _backup_keys[_BACKUP_PRIVATE_KEY_INDEX].add(_key_pair.getPrivate());
        for(Enumeration enum = _alias_table.keys(); enum.hasMoreElements();) {
          String current_alias = (String)(enum.nextElement());
          AliasInfo alias_info = (AliasInfo)(_alias_table.get(current_alias));
          if(!alias_info.isNewAccount()) {
            alias_info.setBackupTimestamp(old_timestamp);
          }
        }
      }
      _key_pair = StaticMethods.generateKeyPair();
      try {
        _signing_signature_object = StaticMethods.getSigningSignatureObject(_key_pair.getPrivate());
        _verifying_signature_object = StaticMethods.getVerifyingSignatureObject(_key_pair.getPublic());
      }
      catch(NoSuchAlgorithmException e) {
        throw new KeyStoreException("DefaultKeyStore:updateKeyPair: " +
                                    "System does not support signature algorithm: " + e);
      }
      catch(InvalidKeyException e) {
        throw new KeyStoreException("DefaultKeyStore:updateKeyPair: " + e);
      }
      _key_pair_expiration_date = new GregorianCalendar();
      Integer length_key_pair_validity = (new Integer(_constants.getProperty("const.LENGTH_KEY_PAIR_VALIDITY")));
      _key_pair_expiration_date.add(Calendar.DAY_OF_YEAR, length_key_pair_validity.intValue());
      _hummer_certificate = new HummerCertificate(_signing_signature_object,
                                                  _key_pair.getPublic(),
                                                  null, _key_pair_expiration_date);
      _has_changed = true;
      return true;
    }
    else {
      return false;
    }
  }
}
