/*
Java Hummer Project
File: HummerKeyStore.java
Date: 1-24-99
Author: Chad Ward
Description:  Object for tracking a user's authentication and access
              rights to the system.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util;

import java.io.*;
import java.util.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.security.KeyPair;
import java.security.Key;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStoreException;
import java.security.InvalidKeyException;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidKeySpecException;
import edu.uidaho.hummer.dataobject.HummerCertificate;
import edu.uidaho.hummer.dataobject.NewUserValidator;
import edu.uidaho.hummer.dataobject.AliasInfo;

/**
Class used to store a KeyPair for the holder of the key store and a set of
accounts.  Each account is referenced using a unique alias.  The information
associated with each account is an alias type (user,server or peer server),
a timestamp to reference which of the key store's backup keys
the account is associated with, and an access table defining the rights given to the account.
Server accounts also hold a HummerCertificate (given by the server) and a boolean
representing whether HummerCertificates granted by that server are honored.

<h4>Class Usage and Flow:</h4>
<ul>
  <li>main - HummerKeyStore can be intantiated and edited from the command line.
  <li>Constructor attempts to initialize a Properties object from the Hummer.properties file.
  <li>load(byte[]) - Initializes a new HummerKeyStore with only an "admin" account
  <li>load(InputStream) - Initializes a HummerKeyStore from the given InputStream
      (store(OutputStream) should have been used to place the HummerKeyStore first)
  <li>store - Writes the values in the HummerKeyStore to the given output stream
  <li>getPersonalXXXX - Returns values that are specific to this HummerKeyStore
      (not any of the accounts in the key store)
  <li>addPeerAlias/addUserAlias/addServerAlias - Add a new uninitialized account to the key store
  <li>getCertificateAliasList - Returns a list of all the aliases in the HummerKeyStore that
      have given a HummerCertificate (HummerCertificate are only accepted from servers)
  <li>getHummerCertificate - Returns the HummerCertificate granted by a given server
  <li>initializeAccount - Initializes a given account.  Will verify that the correct password (message digest)
      has been given before initializing the account with the new PublicKey
  <li>updateKeyPair - When true is passed to this method, then the KeyPair associated
      with the key store is always updated.  If false is passed, then the KeyPair is only updated if
      it is outdated.
</ul>
<h4>Usage Example</h4>
Create an instance of a HummerKeyStore.  The actual type of HummerKeyStore is determined
by the value of 'default' in the Hummer.properties file.
	<blockquote><b>
	HummerKeyStore key_store = HummerKeyStore.getInstance();
	</b></blockquote>

Create a new HummerKeyStore, setting the admin account password (not loaded from a file).
	<blockquote><b>
	key_store.load("admin_pass".getBytes());
	</b></blockquote>

The key store is now initialized, and operations can be performed on it.  Set up
variables to initialize the admin account.  Start with the PublicKey.
	<blockquote><b>
	KeyPair key_pair = StaticMethods.generateKeyPair();
	</b></blockquote>

Generate the random timestamp and message digest bytes.
	<blockquote><b>
	byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(); <br>
	byte[] message_digest = StaticMethods.getDigest("admin_pass".getBytes(), random_timestamp, key_pair.getPublic());
	</b></blockquote>

Initialize the admin account.
	<blockquote><b>
	key_store.initializeAccount("admin", message_digest, random_timestamp, key_pair.getPublic());
	</b></blockquote>

The key pair for the HummerKeyStore may be outdated, so periodically
the updateKeyPair method should be called.  In this case the update is not forced,
so it will only be updated if the KeyPair is outdated.  If true is passed, then
the KeyPair will always be updated.
	<blockquote><b>
	key_store.updateKeyPair(false);
	</b></blockquote>

The old KeyPair will still be associated with the admin account, to retrieve this association,
the getPersonalKeyPair method should be used.
	<blockquote><b>
	KeyPair key_pair_known_by_admin = key_store.getPersonalKeyPair("admin");
	</b></blockquote>

To find out the rights that are held when trying to login to another server, the
access table can be found in the HummerCertificate granted by that server (if it has
been previously given).
	<blockquote><b>
	HummerCertificate remote_certificate;
	remote_certificate = key_store.getHummerCertificate("129.101.54.53");
	Hashtable access_table = remote_certificate.getAccessTable();
	</b></blockquote>

@author Chad Ward
*/
public abstract class HummerKeyStore {
/* -------------------------------------------------------------------
 *	Variables
 ---------------------------------------------------------------------*/
	/**
	Hashtable of AliasInfo objects, where each object in the table is indexed
	by the account alias.
	*/
  protected Hashtable _alias_table;

	/**
	True if the HummerKeyStore has been loaded, false if HummerKeyStore.load() has not been called.
	*/
  protected boolean _has_loaded = false;

	/**
	Pair of Vectors (array of size=2) containing backup keys and the timestamp associated with them.
	([0]=KeyPair, [1]=Long(timestamp) <br>
	Size of the Vector should never exceed NUMBER_OF_BACKUP_KEYS property in Hummer.properties
	*/
  protected Vector[] _backup_keys;

	/**
	Holds the contant values for number of backup keys and length of KeyPair validity.
	Is initialized with the system properties.
	*/
  protected Properties _constants = new Properties(System.getProperties());

	/**
	PublicKey and PrivateKey pair used by this HummerKeyStore.  The actual
	type of keys used are protocol specific.
	*/
  protected KeyPair _key_pair;

	/**
	Date that _key_pair will expire (also used when generating a HummerCertificate)
	*/
  protected GregorianCalendar _key_pair_expiration_date;

	/**
	HummerCertificate for the current PublicKey (from _key_pair)
	*/
  protected HummerCertificate _hummer_certificate;

	/**
	Message given by an exception when an alias is requested which is not known by the HummerKeyStore.
	*/
  public final String _UNKNOWN_ALIAS_MESSAGE = "The requested alias is not known by the system.";

	/**
	Boolean value used to determine whether a change has occured in the key store
	since it was last stored.  This value is used by store to determine whether a
	write operation should occur.
	*/
  protected boolean _has_changed = true;

	/**
	Index into the backup key array of vectors.
	*/
  public static final int _BACKUP_TIMESTAMP_INDEX = 0,
                          _BACKUP_PUBLIC_KEY_INDEX = 1,
                          _BACKUP_PRIVATE_KEY_INDEX = 2;


/* -------------------------------------------------------------------
 *	Constructors
 ---------------------------------------------------------------------*/
	/**
	Default constructor (Attempts to load Hummer.properties file from local directory).
	*
  protected HummerKeyStore() throws KeyStoreException {
    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch (FileNotFoundException e) {
      throw new KeyStoreException("HummerKeyStore:constructor:" + e);
    }
    catch (IOException e) {
      throw new KeyStoreException("HummerKeyStore:constructor:" + e);
    }
  }

	/**
	Returns an instance of a HummerKeyStore using the protocol
	value in the Hummer.properties file.
	*/
  public final static HummerKeyStore getInstance() throws KeyStoreException {
    Properties constants = null;
    try {
      constants = StaticMethods.getHummerProperties();
    }
    catch (FileNotFoundException e) {
      throw new KeyStoreException("HummerKeyStore:getInstance:" + e);
    }
    catch (IOException e) {
      throw new KeyStoreException("HummerKeyStore:getInstance:" + e);
    }
    String protocol = constants.getProperty("const.PROTOCOL");
    if(protocol.equals("default")) {
      return new DefaultKeyStore(constants);
    }
    else {
      throw new KeyStoreException("HummerKeyStore:getInstance: Invalid protocol in " +
                                  "Hummer.properties file: " + protocol + ".");
    }
  }

	/**
	Sets the properties (_constants) object equal to constants.
	*/
  protected HummerKeyStore(Properties constants) {
    this._constants = constants;
  }

/* ------------------------------------------------------------------
 * main method for command line arguments
 --------------------------------------------------------------------*/

	/**
	Main method so that HummerKeyStore can be run from the command line.
	Only one command be sent to the key store at one time.  <br>
	The format of a command is: <br>
	<ul>
	  <li> <B>java edu.uidaho.hummer.util.HummerKeyStore
	      &lt;f key_store_filename&gt;|&lt;u username&gt;|&lt;s &lt;l&gt;|&lt;address&gt;&gt;
	      -command_type- -value1-&lt; -value2-&gt;</B>
	</ul>
	<Br> <B>f option</B> - Specify a filename for the HummerKeyStore
        <Br> <B>u option</B> - Specify the username for the HummerKeyStore
	<Br> <B>s option</B> - Specify the address (name or IP) of the server that will be using the HummerKeyStore (l - local system)
	<Br> The types of commands and their types of values are:
	<table border=1>
	  <tr bgcolor="#8F8FBD">
	    <td> <font color="#FFFFFF"> <B>Command Descriptions</B> </font></td>
	    <td> <font color="#FFFFFF"> <B>Command Name</B> </font></td>
	    <td> <font color="#FFFFFF"> <B>Type of Value</B> </font></td>
	  </tr><tr>
	    <td> Create New KeyStore </td>
	    <td> new </td>
	    <td> &lt;admin_password&gt; </td>
	  </tr><tr>
	    <td> List Aliases in KeyStore </td>
	    <td> list </td>
	    <td> None </td>
	  </tr><tr>
	    <td> List Aliases Access Rights in KeyStore </td>
	    <td> alias_rights </td>
	    <td> &lt;alias&gt; </td>
	  </tr><tr>
	    <td> Add a user alias to the system </td>
	    <td> add_user </td>
	    <td> &lt;new_alias&gt; &lt;alias_password&gt;</td>
	  </tr><tr>
	    <td> Add a server alias to the system </td>
	    <td> add_server </td>
	    <td> &lt;new_alias&gt; &lt;y|n&gt; &lt;alias_password&gt; <br>
	         <BlockQuote>Note: y = Certificates honored.  n = Certificates not honored.</BlockQuote>
	  </tr><tr>
	    <td> Reset an account on the system </td>
	    <td> reset_alias </td>
	    <td> &lt;alias&gt; &lt;alias_password&gt; </td>
	  </tr><tr>
	    <td> Remove an alias from system </td>
	    <td> remove_alias </td>
	    <td> &lt;alias&gt; &lt;alias_password&gt; </td>
	  </tr><tr>
	    <td> Update KeyStore key pair </td>
	    <td> update_key_pair </td>
	    <td> None </td>
	  </tr><tr>
	    <td> Update outdated KeyStore key pair<br>(if not outdated, then no action is taken)</td>
	    <td> update_old_key_pair </td>
	    <td> None </td>
	  </tr>
	</table>
	*/
  public static void main(String[] argv)
         throws IOException, KeyStoreException, ClassNotFoundException,
                NoSuchProviderException, NoSuchAlgorithmException,
                UnloadedKeyStoreException, UnknownAliasException,
                CertificateException {
    HummerKeyStore key_store = HummerKeyStore.getInstance();
    key_store.command(argv);
  }

  public abstract void command(String[] argv)
    throws IOException, KeyStoreException, ClassNotFoundException,
                NoSuchProviderException, NoSuchAlgorithmException,
                UnloadedKeyStoreException, UnknownAliasException,
                CertificateException;

/* ------------------------------------------------------------------
 *	Methods
 --------------------------------------------------------------------*/
	/**
	Adds a new alias to the key store for a server (not a user) with a password and trust value.
	The alias and password are placed into the NewUserValidator object for later validation when the server
	logs in for the first time. The access table will be null.
	@param password Temporary password of the account to be changed upon the first login of the user
	@param certificates_honored True if certificates signed by this server will be give the holder access
		to this system, false if it is only another server that holds a local account.
	@param access_table Table of access priveledges given to the server.
	@throws KeyStoreException The given alias cannot already be in the system.
	@see HummerKeyStore#addUserAlias(String, byte[], Hashtable)
	@see HummerKeyStore#addPeerAlias(String, byte[])
	*/
  public abstract void addServerAlias(String alias, byte[] password,
                             boolean honored, Hashtable access_table)
         throws UnloadedKeyStoreException, KeyStoreException;

	/**
	Adds a new alias to the key store for a peer with a password (access tables are equal for all peer servers).
	The alias and password are placed into the NewUserValidator object for later validation when the server
	logs in for the first time. The access table will be null.
	@param password Temporary password of the account to be changed upon the first login of the user
	@throws KeyStoreException The given alias cannot already be in the system.
	@see HummerKeyStore#addUserAlias(String, byte[], Hashtable)
	@see HummerKeyStore#addPeerAlias(String, byte[])
	*/
  public abstract void addPeerAlias(String alias, byte[] password)
         throws UnloadedKeyStoreException, KeyStoreException;

	/**
	Adds a new alias to the key store for a user (not a server) with a password and a trust value.
	The alias and password are placed into the NewUserValidator object for later validation when the server
	logs in for the first time. The access table will be null.
	@param alias Username
	@param password Temporary password of the account to be changed upon the first login of the user
	@throws KeyStoreException The given alias cannot already be in the system.
	@see HummerKeyStore#addServerAlias(String, byte[], boolean, Hashtable)
	*/
  public abstract void addUserAlias(String alias, byte[] password, Hashtable access_table)
         throws UnloadedKeyStoreException, KeyStoreException;


	/**
	Returns a list of aliases for servers that have granted
	HummerCertificate to this HummerKeyStore
	*/
  public final Vector getCertificateAliasList()
         throws UnloadedKeyStoreException, KeyStoreException {
    this.checkLoaded();
    Vector return_value = new Vector();
    String current_alias;
    AliasInfo alias_info;
    for(Enumeration enum = _alias_table.keys(); enum.hasMoreElements();) {
      current_alias = (String)(enum.nextElement());
      alias_info = (AliasInfo)_alias_table.get(current_alias);
      if(alias_info.getType() == AliasInfo._SERVER_ALIAS_TYPE &&
         !alias_info.isNewAccount() && alias_info.holdingHummerCertificate()) {
        return_value.add(current_alias);
      }
    }
    return return_value;
  }

	/**
	Method for converting the values returned by getAliasStatisticsHeader
	to a single string.
	@param max_alias_length The maximum number of charcters that could
		be in an alias
	@param max_key_length The maximum number of characters that could
		be in the String representation of any key or key name
	*/
  public final static String statisticsToString(Object[] statistics,
                                           int max_alias_length,
                                           int max_key_length) {
    String return_value = "";
    char[] max_filler = null;
    if(max_alias_length > max_key_length) {
      max_filler = new char[max_alias_length];
    }
    else {
      max_filler = new char[max_key_length];
    }
    Arrays.fill(max_filler, ' ');
    String max_filler_string = new String(max_filler);
    int spacer = max_alias_length - statistics[0].toString().length();
    if(spacer <= 0) {
      spacer = 1;
    }
    if(statistics[1].getClass().getName().equals("java.lang.Integer")) {
      int alias_type_value = ((Integer)statistics[1]).intValue();
      switch(alias_type_value) {
        case AliasInfo._SERVER_ALIAS_TYPE:
          statistics[1] = "server";
          break;
        case AliasInfo._PEER_ALIAS_TYPE:
          statistics[1] = "peer";
          break;
        case AliasInfo._USER_ALIAS_TYPE:
          statistics[1] = "user";
          break;
        default:
          statistics[1] ="unknown";
      }
    }
    return_value = statistics[0].toString() + max_filler_string.substring(0, spacer) +
                   statistics[1].toString();
    for(int i = 2; i < statistics.length; i++) {
      spacer = max_key_length - statistics[i -1].toString().length();
      if(spacer <= 0) {
        spacer = 1;
      }
      return_value += max_filler_string.substring(0, spacer) + statistics[i].toString();
    }
    return return_value;
  }


	/**
	Internal method that insures that the HummerKeyStore has already been loaded.  All
	methods (except load) call this method before performing any actions.
	*/
  protected final void checkLoaded() throws UnloadedKeyStoreException {
    if(!_has_loaded) {
      throw new UnloadedKeyStoreException("HummerKeyStore:checkLoaded(): " +
                                          "Must load HummerKeyStore before operations can be made.");
    }
  }


	/**
	Checks if the given alias exists in this keystore (uses Hashtable.containsKey() method).
	@param alias Alias used to access key
	@return True if the alias exists, false otherwise
	*/
  public final boolean contains(String alias) throws UnloadedKeyStoreException {
    this.checkLoaded();
    if(alias == null || alias.equals("")) {
      return false;
    }
    return _alias_table.containsKey(alias);
  }


	/**
	Protected method that is used internally to only load values from an
	input stream.  This method may or may not be faster than just load, depending
	upon the type of HummerKeyStore.
	*/
  protected abstract void fileLoad(InputStream input_stream)
         throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                KeyStoreException;


	/**
	Returns a HummerCertificate generated by the given <i>alias</i>.
	@param alias Alias to find HummerCertificate for.
	@throws KeyStoreException The given alias is for an account that has not been initialized,
		or the account is not for a server (alias type != _SERVER_ALIAS_TYPE)
	*/
  public final HummerCertificate getHummerCertificate(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:getHummerCertificate: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)_alias_table.get(alias);
    if(alias_info.isNewAccount()) {
      throw new KeyStoreException("HummerKeyStore:getHummerCertificate: " +
                                  "Attempt to retrieve a HummerCertificate for an account " +
                                  "that has not been loaded.");
    }
    if(alias_info.getType() == AliasInfo._USER_ALIAS_TYPE) {
      throw new KeyStoreException("HummerKeyStore:getHummerCertificate: " +
                                  "Attempt to retrieve HummerCertificate for a user account.");
    }
    if(!alias_info.holdingHummerCertificate()) {
      throw new KeyStoreException("HummerKeyStore:getCertificate: " +
                                  "Attempt to retrieve HummerCertificate that " +
                                  "has not been placed in DefaultKeyStore yet.");
    }
    HummerCertificate return_value = alias_info.getHummerCertificate();
    return return_value;
  }


	/**
	Removes all the aliases in the key store of type _USER_ALIAS_TYPE
	*/
  public final void removeAllUserAliases() {
    AliasInfo current_alias_info;
    String current_alias;
    for(Enumeration enum = _alias_table.keys(); enum.hasMoreElements();) {
      current_alias = enum.nextElement().toString();
      current_alias_info = (AliasInfo)_alias_table.get(current_alias);
      if(current_alias_info.getType() == AliasInfo._USER_ALIAS_TYPE) {
        _alias_table.remove(current_alias);
      }
    }
  }
  
	/**
	Deletes the entry identified by the given alias from this keystore.
	@param key_type Type of key being accessed
	@param alias Alias used to access key
	@see HummerKeyStore#_alias_table
	*/
  public final void removeAlias(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:removeAlias: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    if(alias.equals("admin")) {
      throw new KeyStoreException("HummerKeyStore:removeAlias: " +
                                  "Alias-\"admin\" cannot be removed from HummerKeyStore");
    }
    _alias_table.remove(alias);
    _has_changed = true;
  }

	/**
	Compares the given Hashtable with the Hashtable representing the given aliases access
	table.
	*/
  public final boolean equalAccessTable(String alias, Hashtable access_table)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:equalAccessTable: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    Hashtable alias_access_table = alias_info.getAccessTable();
    if(alias_access_table == null) {
      if(access_table == null) {
        return true;
      }
      else {
        return false;
      }
    }
    else {
      return alias_access_table.equals(access_table);
    }
  }

	/**
	Gets the value of the access rights for the given alias.
	@param alias Alias or username to look up information about
	@param access_name Name of the access right
	@return Value found for the given <i>alias</i> and <i>access_name</i>
	*/
  public final Hashtable getAccessTable(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:getAccessTable: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    return ((AliasInfo)_alias_table.get(alias)).getAccessTable();
  }

	/**
	Gets an Enumeration of the aliases in the HummerKeyStore
	*/
  public final Enumeration getAliases() throws UnloadedKeyStoreException{
    this.checkLoaded();
    return _alias_table.keys();
  }

	/**
	Returns the header to be used when displaying alias statistics.  For
	any implementation of a HummerKeyStore, the first three values should be:
	String[0]=Alias
	String[1]=Alias Type
	String[2]=Initialized
	@see HummerKeyStore#getAliasStatistics(String)
	*/
  public abstract String[] getAliasStatisticsHeader();

	/**
	Returns values for a given alias.  The meanings of these can be gathered
	from getAliasStatisticHeader.
	For any implementation of a HummerKeyStore, the first three values should be:
	String[0]=Alias
	String[1]=Alias Type
	String[2]=Initialized
	@see HummerKeyStore#getAliasStatisticsHeader()
	*/
  public abstract Object[] getAliasStatistics(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException;
	/**
	Gets an AliasInfo object for the given alias
	*/
  public final AliasInfo getAliasInfo(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:getAliasInfo: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    return (AliasInfo)_alias_table.get(alias);
  }

	/**
	Gets the alias type for the given alias (uses getAliasTypeInternal after checking that
	the alias is in the key store).
	@see AliasInfo#_USER_ALIAS_TYPE
	@see AliasInfo#_SERVER_ALIAS_TYPE
	@see AliasInfo#_PEER_ALIAS_TYPE
	*/
  public final int getAliasType(String alias) throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:getAccessTable: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    return ((AliasInfo)(_alias_table.get(alias))).getType();
  }      


	/**
	Finds the given alias in the _backup_keys array.  Does not check that HummerKeyStore is loaded
	or that the given alias is valid.
	@returns If timetamp is found in the _backup_keys array, then it is returned.  If
		the timestamp equals -1, then -1 is returned, otherwise -2 is returned
		(timestamp does not exist in list of backup keys).
	*/
  protected final int getBackupKeyIndex(String alias) {
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    long timestamp = alias_info.getBackupTimestamp();
    if(timestamp == -1) {
      return -1;
    }
    int return_value = -2,
        index = _backup_keys[_BACKUP_TIMESTAMP_INDEX].indexOf(new Long(timestamp));
    if(index >= 0) {
      return_value = index;
    }
    return return_value;
  }

	/**
	Returns a Key for the specified alias.  Type of key returned is specified
	by _key_type_index.  For a list of the key types see AliasInfo or the
	protocol specific version of AliasInfo that is being used.
	@return If <i>alias</i> exists in the key store then, returns PublicKey for alias, otherwise
	returns null
	*/
  public final Key getKey(int key_type_index, String alias)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:getKey: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    if(alias_info.isNewAccount()) {
      throw new KeyStoreException("HummerKeyStore:getKey: " +
                                  "Cannot retrieve a key for an account that has not been initialized.");
    }
    return alias_info.getKey(key_type_index);
  }

	/**
	Returns the current HummerCertificate.
	*/
  public final HummerCertificate getPersonalHummerCertificate() throws UnloadedKeyStoreException {
    this.checkLoaded();
    return _hummer_certificate;
  }

	/**
	Returns a HummerCertificate for this HummerKeyStore signed with a Key that is
	known by the given alias (associated timestamp).
	*/
  public abstract HummerCertificate getPersonalHummerCertificate(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException,
                KeyStoreException, CertificateException;

	/**
	Gets the backup KeyPair associated with timestamp.
	@param timestamp Long value representing the time that the KeyPair was created
	@return The backup KeyPair with the timestamp given by <i>timetstamp</i>.  If <i>timestamp</i>
	equals -1 then current KeyPair is returned.  If there is no KeyPair associated with the given timestamp,
	then null is returned.
	@see HummerKeyStore#getPersonalKeyPair(String)
	*/
  public final KeyPair getPersonalKeyPair(int backup_value_index, Object search_value)
         throws UnloadedKeyStoreException {
    this.checkLoaded();
    if(search_value == null) {
      return null;
    }
    KeyPair return_value = null;
    if(backup_value_index == _BACKUP_TIMESTAMP_INDEX &&
       ((Long)search_value).longValue() == -1) {
      return _key_pair;
    }
    if(backup_value_index == _BACKUP_PUBLIC_KEY_INDEX &&
       search_value.equals(_key_pair.getPublic())) {
      return _key_pair;
    }
    if(backup_value_index == _BACKUP_PRIVATE_KEY_INDEX &&
       search_value.equals(_key_pair.getPrivate())) {
      return _key_pair;
    }
    int index = _backup_keys[backup_value_index].indexOf(search_value);
    if(index == -1) {
      return_value = null;
    }
    else {
      return_value = new KeyPair((PublicKey)_backup_keys[_BACKUP_PUBLIC_KEY_INDEX].get(index),
                                 (PrivateKey)_backup_keys[_BACKUP_PRIVATE_KEY_INDEX].get(index));
    }
    return return_value;
  }

	/**
	Gets the KeyPair that was used to create a HummerCertificate for <i>alias</i>.  If this key does not
	exist or alias is not known(uninitialized account), then <i>null</i> is returned
	@param alias The alias to search for the KeyPair
	@return If HummerCertificate created for <i>alias</i> is from an only KeyPair, then the KeyPair
	that was used is returned, otherwise <i>null</i> is returned.
	@see HummerKeyStore#getPersonalKeyPair(int,Object)
	*/
  public final KeyPair getPersonalKeyPair(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:getPersonalKeyPair: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)_alias_table.get(alias);
    if(alias_info.isNewAccount()) {
      return null;
    }
    int index = getBackupKeyIndex(alias);
    KeyPair return_value = null;
    if(index == -1) {
      return_value = _key_pair;
    }
    else if(index >= 0) {
      return_value = new KeyPair((PublicKey)_backup_keys[_BACKUP_PUBLIC_KEY_INDEX].get(index),
                                 (PrivateKey)_backup_keys[_BACKUP_PRIVATE_KEY_INDEX].get(index));
    }
    return return_value;
  }



	/**
	Gets the expiration date for the KeyPair that this HummerKeyStore's KeyPair will expire
	*/
  public final GregorianCalendar getPersonalKeyPairExpiration() throws UnloadedKeyStoreException {
    this.checkLoaded();
    return _key_pair_expiration_date;
  }

	/**
	Returns true if the key store has changed since storing or first load.
	*/
  public final boolean hasChanged() { return _has_changed; }

	/**
	Returns a copy of the Vector of backup keys.
	*/
  public final Vector[] getPersonalBackupKeys() throws UnloadedKeyStoreException {
    this.checkLoaded();
    return _backup_keys;
  }


	/**
	Returns true if <i>alias</i> has a HummerCertificate signed with the most recent PublicKey,
	otherwise false.
	@param alias Alias to search for information on
	*/
  public final boolean hasUpdatedHummerCertificate(String alias)
         throws UnknownAliasException, UnloadedKeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:hasUpdatedHummerCertificate: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)_alias_table.get(alias);
    return (alias_info.getBackupTimestamp() == -1);
  }



	/**
	Verifies that password-digest is correct, then sets the public key and -1 for timestamp, and
	removes NewUserValidator from alias info.
	@return A new HummerCertificate using the given PublicKey, access table for alias, and
		the date that the signing PrivateKey expires.
	*/
  public abstract HummerCertificate initializeAccount(String alias, byte[] message_digest,
                                                      byte[] random_timestamp, Key key)
         throws UnloadedKeyStoreException, KeyStoreException,
                UnknownAliasException, CertificateException;

	/**
	Returns true if the method "load" has been called already for this
	key store object.
	*/
  public final boolean isLoaded() { return _has_loaded; }

	/**
	Finds if the account for the given alias has been initialized with keys
	@param alias Alias of the account to search for
	@returns True if the account has been initialized, false otherwise
	*/
  public abstract boolean isNewAccount(String alias)
         throws UnknownAliasException, UnloadedKeyStoreException;

	/**
	Generates a new certificate for the given alias.  Placing the alias' PublicKey, access_table,
	and the expiration date of the signing PrivateKey into the HummerCertificate.  The
	HummerCertificate is signed using the HummerKeyStore's PublicKey associated with the alias.
	*/
  public abstract HummerCertificate generateHummerCertificate(String alias)
         throws UnloadedKeyStoreException, UnknownAliasException,
                KeyStoreException, CertificateException;

	/**
	Gets the current KeyPair.
	@see HummerKeyStore#getPersonalKeyPair(String)
	*/
  public final KeyPair getPersonalKeyPair() throws UnloadedKeyStoreException {
    this.checkLoaded();
    return _key_pair;
  }


	/**
	Assigns the PublicKey to the given alias.
	@param alias Alias used to access key
	@param certificate Certificate to be associated with the alias
	@param certificate_honored Specifies whether the certificates given out by this alias will be given
	honored as allowing users to login with rights delegated by this alias
	(authentication forwarding - Is this alias a trusted Hummer?)
	@throws KeyStoreException The account for the given alias is not for a server, the account has not
		been initialized, the certificate does not contain the most recent PublicKey, or the certificate
		cannot be verified with the server's PublicKey (could also be outdated).
	*/
  public abstract void setHummerCertificate(String alias, HummerCertificate certificate) 
         throws UnloadedKeyStoreException, UnknownAliasException,
                KeyStoreException, CertificateException;

	/**
	Returns a filename for a HummerKeyStore with the Hummer naming conventions that follow: <br>
	server: &lt;address&gt;&lt;const.KEY_STORE_FILE_EXTENSION&gt;<br>
	client: &lt;username&gt;&lt;const.KEY_STORE_FILE_EXTENSION&gt; <br>
	where &lt;const.KEY_STORE_FILE_EXTENSION&gt; is a value in the Hummer.properties file.
	*/
  public final static String getDefaultFilename(boolean is_server, String alias)
         throws UnknownHostException, IOException, FileNotFoundException {
    String filename,
           extension = StaticMethods.getHummerProperties().getProperty("const.KEY_STORE_FILE_EXTENSION");
    if(is_server) {
      String address = alias;
      if(address == null || address.equals("")) {
        address = InetAddress.getLocalHost().getHostAddress();
      }
      else {
        address = InetAddress.getByName(alias).getHostAddress();
      }
      filename = address + extension;
    }
    else {
      filename = alias + extension;
    }
    return filename;
  }

	/**
	Loads the key store from a file.  The name of the file is generated
	using HummerKeyStore.getDefaultFilename.
	@return The filename that the key store was loaded from.
	*/
  public final String load(boolean is_server, String alias)
         throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                KeyStoreException {
    String filename = getDefaultFilename(is_server, alias);
    DataInputStream in_stream = new DataInputStream(new FileInputStream(filename));
    this.load(in_stream);
    in_stream.close();
    return filename;
  }

	/**
	Loads this HummerKeyStore from the given input stream.
	@param input_stream InputStream object where the key store can be loaded from.
	@throws KeyStoreException There was an error when reading from the InputStream, so that the objects
		for the HummerKeyStore could not be created.
	*/
  public abstract void load(InputStream input_stream)
         throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                KeyStoreException;

	/**
	Creates a HummerKeyStore with a new KeyPair and an 'admin' account with the given
	password.  The admin account is created using the addUserAlias method using
      alias="admin" and password=admin_password.
	*/
  public abstract void load(byte[] admin_password)
         throws IOException, ClassNotFoundException, StreamCorruptedException, OptionalDataException,
                KeyStoreException, NoSuchAlgorithmException, NoSuchProviderException, CertificateException;
	/**
	Resets the information for the account represented by <i>alias</i>.  After calling this method,
	isNewAccount will return true and there will no longer be any keys or HummerCertificates.
	@param new_password The password that will be associated with this account after it has been
		reset.
	*/
  public abstract void resetAlias(String alias, byte[] new_password)
         throws UnloadedKeyStoreException, UnknownAliasException;

	/**
	Sets the access table for the given alias
	*/
  public final void setAccessTable(String alias, Hashtable access_table)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:setAccessTable: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    alias_info.setAccessTable(access_table);
    _has_changed = true;
  }

	/**
	Sets a single value in the access table for a given alias
	*/
  public final void setAccessValue(String alias, String access_name, boolean value)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:setAccessValue: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    if(access_name == null || access_name.equals("")) {
      throw new KeyStoreException("HummerKeyStore:setAccessValue: Access name cannot be a null or empty string.");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    alias_info.setAccessValue(access_name, value);
    _has_changed = true;
  }

	/**
	Sets the type for the alias (user, server, or peer)
	@see edu.uidaho.hummer.dataobject.AliasInfo#_USER_ALIAS_TYPE
	@see edu.uidaho.hummer.dataobject.AliasInfo#_SERVER_ALIAS_TYPE
	@see edu.uidaho.hummer.dataobject.AliasInfo#_PEER_ALIAS_TYPE
	*/
  public final void setAliasType(String alias, int alias_type)
         throws UnloadedKeyStoreException, UnknownAliasException {
    this.checkLoaded();
    if(!this.contains(alias)) {
      throw new UnknownAliasException("HummerKeyStore:setAliasType: " +
                                       _UNKNOWN_ALIAS_MESSAGE + "(" + alias + ")");
    }
    AliasInfo alias_info = (AliasInfo)(_alias_table.get(alias));
    alias_info.setType(alias_type);
    _has_changed = true;
  }

  public abstract void setKey(int key_type_index, String alias, Key new_key)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException;

	/**
	Sets the the value for a server to alias representing whether HummerCertificates granted by the server
	are trusted or not.  (true=Holder's of HummerCertificates signed by this server can login to the machine,
	false=HummerCertificates signed by this server have no meaning)
	@throws KeyStoreException The alias type for the given alias is not equal to _SERVER_ALIAS_TYPE.
	*/
  public abstract void setServerTrust(String server_alias, boolean trusted)
         throws UnloadedKeyStoreException, UnknownAliasException, KeyStoreException;


	/**
	Returns number of alias known by the system.
	*/
  public final int size() throws UnloadedKeyStoreException {
    this.checkLoaded();
    return _alias_table.size();
  }

	/**
	Stores the key store to a file with the standardized name
	returned by HummerKeyStore.getDefaultFilename
	@return The filename used to store the key store.
	*/
  public final String store(boolean is_server, String alias)
         throws IOException, UnloadedKeyStoreException {
    this.checkLoaded();
    String filename = null;
    if(_has_changed) {
      filename = getDefaultFilename(is_server, alias);
      FileOutputStream out_stream = new FileOutputStream(filename);
		/*
		Using a normal file output stream can cause portability
		problems between Unix and NT, so DataoutputStream should be used.
		*/
      store(out_stream);
      out_stream.close();
    }
    return filename;
  }

	/**
	Stores this HummerKeyStore to the given output stream.
	@param output_stream OutputStream object where the key store will be stored to.
	*/
  public abstract void store(OutputStream output_stream)
                  throws IOException, UnloadedKeyStoreException;

	/**
	Replaces _key_pair, places previous value in _backup_keys along with a timestamp
	representing the time that the keypair had expired. <br>
	<b>Note:</b><br>
	All accounts (alias') are check for the timestamp associated with them.  If it is -1, then it
	is set the timestamp of the KeyPair being replaced
      @param force_update If this is true then a new _key_pair will be generated. If it is false,
		then a new _key_pair will only be generated if it has expired.
	*/
  public abstract boolean updateKeyPair(boolean force_update)
         throws UnloadedKeyStoreException, NoSuchAlgorithmException,
                NoSuchProviderException, CertificateException, KeyStoreException;
}