/*
Java Hummer Project
File: DefaultServerProtocol.java
Date: 4-5-99
Author: Chad Ward
Description:  Protocol for a server-side connection of communication.
              This "default" version does not use any of the JCE classes, therefore
              it does do any two-way encryption/decryption.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;

import java.io.IOException;
import java.io.OptionalDataException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.FileNotFoundException;
import java.util.Date;
import java.util.Calendar;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;
import java.net.Socket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.Key;
import java.security.KeyPair;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStoreException;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import java.security.NoSuchProviderException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.dataobject.NewUserValidator;
import edu.uidaho.hummer.dataobject.PersonalConfig;
import edu.uidaho.hummer.dataobject.HummerSignedObject;
import edu.uidaho.hummer.dataobject.HummerCertificate;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.DefaultKeyStore;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;


/**
Concrete HummerProtocol instance for a server using
Signatures to authenticate itself to a client.  This method uses only classes
from the JDK 1.2 library, so encryption/decryption does not occur.

The following table describes the protocol between a DefaultServerProtocol
and a DefaultClientProtocol when both call their validation() methods.
Any tranmsissions following this are handled by the HummerSocket class. <br>

  <table border=1><tr><td>
  <b>For normal transmission</b>
  </td></tr><tr><td>
  <table border=0> <tr><td VAlign=top>
  int _type</td><td VAlign=top>(see Response._type) Should be equal to Response._OK
		</td></tr><tr><td VAlign=top>
  long _id</td><td VAlign=top>Long that client generated using (new Date()).getTime(), which will be used to give
              a "unique" id for each message.  This is done by incrementing the id for each new message.
              (Used to eliminate replay attacks)
		</td></tr><tr><td VAlign=top>
  Object _message</td><td VAlign=top>For this first transmission, will be assumed to be null (ignored unless _type=Response._EXCEPTION)
		</td></tr><tr><td VAlign=top>
  Vector _data</td><td VAlign=top>Holder of major data.  See below values for contents of this Vector.
		</td></tr><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>String alias - The recognized alias for the account to loggin under
		</td></tr><tr><td VAlign=top>

  _data[2]</td><td VAlign=top>String signer_alias - Alias (IP address) of server that signed the certificate object (data[3])
		</td></tr><tr><td VAlign=top>

  _data[3]</td><td VAlign=top>HummerCertificate signature_certificate - Certificate signed by mutually known Hummer (data[2])
      </td></tr><tr><td VAlign=top>&nbsp;</td></tr><tr><td VAlign=top>
  <b> Response&nbsp;by&nbsp;Server:</b></td><td VAlign=top>
      Certificate is Valid, or requesting <i>alias</i> is for a local account, and the version
      of the HummerCertificate given can be verified as the most recent copy for the <i>alias</i>
		</td></tr><tr><td VAlign=top>
  int _type</td><td VAlign=top>(see Response._type) Should be equal to Response._OK
		</td></tr><tr><td VAlign=top>
  long _id</td><td VAlign=top>Long that server generated using (new Date()).getTime(), which will be used to give
              a "unique" id for each message.  This is done by incrementing the id for each new message.
              (Used to eliminate replay attacks)
		</td></tr><tr><td VAlign=top>
  Object _message</td><td VAlign=top>A Long representation of message id that the client sent
		</td></tr><tr><td VAlign=top>
  Vector _data</td><td VAlign=top>Holder of major data.  See below values for contents of this Vector.
		</td></tr><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>HummerCertificate server_certificate - Certificate holding most recent PublicKey of server.
                                      It is with the same key that the client used to sign its HummerCertificate
                                      (Commonly trusted source).
		</td></tr><tr><td VAlign=top>
 #_data[2]</td><td VAlign=top>HummerCertificate new_client_certificate - If client is a local account, and its
                                      HummerCertificate was outdated, then a new HummerCertificate created
                                      and sent back to the client (can be verified with PublicKey in server_certificate).
   </td></tr><tr><td colspan=2>
   # Only when for local accounts, and when <i>alias'</i> HummerCertificate has expired
   </td></tr>
 </table></td></tr></table><br><br>

  <table border=1><tr><td VAlign=top>
  <b>For new accounts</b> Initial transmission by the client is the same, except, the <i>_data</i>
    Vector will hold the following:
		</td></tr><tr><td VAlign=top>
  <table border=0><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>String alias - The recognized alias for the account to loggin under
		</td></tr><tr><td VAlign=top>

  _data[2]</td><td VAlign=top>String signer_alias - null
		</td></tr><tr><td VAlign=top>

  _data[3]</td><td VAlign=top>client_public_key - PublicKey of the client wishing to initialize account
		</td></tr><tr><td VAlign=top>

  _data[4]</td><td VAlign=top>message_digest - Array of bytes representing a checksum like encoding
                                (java.security.MessageDigest) of the password for the account,
                                random_timestamp (_data[5]), and the encoded version of client_public_key
		</td></tr><tr><td VAlign=top>

  _data[5]</td><td VAlign=top>random_timestamp - Array of bytes representing a random number (double) and a
                                timestamp (long) that client generated and used for message_digest
      </td></tr><tr><td VAlign=top>&nbsp;</td></tr><tr><td VAlign=top>
  <b> Response&nbsp;by&nbsp;Server:</b></td><td VAlign=top>
      <i>alias</i> is for a new account, and message_digest is validated
		</td></tr><tr><td VAlign=top>
  int _type</td><td VAlign=top>(see Response._type) Should be equal to Response._OK
		</td></tr><tr><td VAlign=top>
  long _id</td><td VAlign=top>Long that server generated using (new Date()).getTime(), which will be used to give
              a "unique" id for each message.  This is done by incrementing the id for each new message.
              (Used to eliminate replay attacks)
		</td></tr><tr><td VAlign=top>
  Object _message</td><td VAlign=top>For this first transmission, will be assumed to be null 
                                     (ignored unless _type=Response._EXCEPTION)
		</td></tr><tr><td VAlign=top>
  Vector _data</td><td VAlign=top>Holder of major data.  See below values for contents of this Vector.
		</td></tr><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>PublicKey server_public_key - The server's public key needed to verify signatures
		</td></tr><tr><td VAlign=top>
  _data[2]</td><td VAlign=top>HummerCertificate client_certificate - The client's new HummerCertificate
                                        which will be needed
                                        to log back into the server, and can be used to authenticate to other
                                        servers that trust the server that signed <i>client_certificate</i>
		</td></tr><tr><td VAlign=top>
  _data[3]</td><td VAlign=top>message_digest - Array of bytes representing a checksum like encoding
                                (java.security.MessageDigest) of the password for the account,
                                random_timestamp (_data[4]), and the encoded version of server_public_key   
		</td></tr><tr><td VAlign=top>

  _data[4]</td><td VAlign=top>random_timestamp - Array of bytes representing a random number (double) and a
                                timestamp (long) that client generated and used for message_digest
  		</td></tr>
  </table> </td></tr></table><br>
  For further documentation on protocol see the documentation for Hummer.
@author Chad Ward
*/

public final class DefaultServerProtocol extends HummerProtocol {

	/**
	Overrides the key store of the parent class, because
	protocol specific information is needed.
	*/
  private DefaultKeyStore _key_store;

	/**
	Object for creating and verifying SignedObjects and HummerCertificates
	*/
  private Signature _signing_signature_object = null, _verifying_signature_object;

	/**
	Alias used to sign certificate held by client.
	*/
  private String _signer_alias;

	/**
	HummerCertificate that the client is using to log in with.
	*/
  private HummerCertificate _client_certificate;

	/**
	HummerCertificate that the server that will be sent to client to verify server's credentials.
	*/
  private HummerCertificate _server_certificate;

	/**
	Boolean representing whether the client needs to update its HummerCertificate
	*/
  private boolean _new_certificate_needed = false;

	/**
	For new account validation, this message digest must be generated to be sent back to
	the client.
	*/
  private byte[] _new_message_digest;

	/**
	For new account validation, this random timestamp bytes must be generated to be sent back to
	the client.
	*/
  private byte[] _new_random_timestamp;

	/**
	Calls superclass constructor with the given parameters.
	@param empty_config This parameter is ignored.
		This is because there is never a password for the server, and the alias will always
		equal the IP address that the server is running on.
		This parameter is only to meet the requirements of the HummerSocket constructor
	@see HummerSocket#HummerSocket(boolean, PersonalConfig, Socket, HummerKeyStore)
	*/
  public DefaultServerProtocol(PersonalConfig empty_config, HummerKeyStore key_store)
         throws NoSuchAlgorithmException, KeyStoreException, UnloadedKeyStoreException, UnknownHostException {
    super(new PersonalConfig(InetAddress.getLocalHost().getHostAddress(), null), key_store);
    this._name = "default";
    this._key_store = (DefaultKeyStore)key_store;
  }


  public void initializeInformation(Object protocol_information, Object other_information) {}

  public void checkBasicInputDataFormat(Object input_object) throws HummerSocketException {
    _validation_response = (Response)input_object;
    if(_validation_response == null) {
      throw new HummerSocketException("Recieved input is null", null, HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data == null) {
      throw new HummerSocketException("Invalid foramat of data sent to server (data Vector is null).",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.size() < 3) {
      throw new HummerSocketException("Invalid format of data sent to server" +
                                      "(data Vector does not contain enough values).",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }

    if(_validation_response._data.get(0) == null) {
      throw new HummerSocketException("Null protocol name recieved.",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.get(1) == null) {
      throw new HummerSocketException("Null alias recieved.", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if((_validation_response._type & Response._EXCEPTION) != 0) {
      if(_validation_response._message == null) {
        throw new HummerSocketException("DefaultServerProtocol:validation: " +
                                        "Exception type Response object sent by client, but " +
                                        "_message(Exception) is null.", null,
                                        HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)_validation_response._message;
    }
  }


  public boolean isNewAccountValidation() {
    return (String)(_validation_response._data.get(2)) == null;
  }

  public void initializedAccountCheckInputDataFormat()
            throws HummerSocketException {
    if(_validation_response._data.size() != 4) {
      throw new HummerSocketException("Invalid format of data sent to server " +
                     "(data Vector is not correct size for logging into system",
                     _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.get(3) == null) {
      throw new HummerSocketException("Null HummerCertificate recieved.", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
  }

  public void initializedAccountVerifyInformation()
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException {
		/*
		Get the PublicKey that was used for client's certificate (if it is valid).
		And a HummerCertificate signed with the same KeyPair
		*/
    Signature signer_verify_signature;
    try {
      _current_partner_alias = (String)(_validation_response._data.get(1));
      _signer_alias = (String)(_validation_response._data.get(2));
      _client_certificate = (HummerCertificate)(_validation_response._data.get(3));
    }
    catch(ClassCastException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " + e,
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
    _output_message_id = _validation_response._id;
		/*
		It is possible for a local user to have an outdated(invalid) certificate, and still be
		validated, because there must be some way for client's to re-new their certificates.
		This is done by finding which PublicKey was used to sign the HummerCertificate
		(key store keeps track of which PublicKey it is supposed to be with a timestamp),
		and validating the client with it.
		*/
    try {
      _verifying_signature_object = StaticMethods.getVerifyingSignatureObject(_client_certificate.getPublicKey());
    }
    catch(NoSuchAlgorithmException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                     "This system does support the algorithm for signatures: " + e, _validation_response, 0);
    }
    catch(InvalidKeyException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " + e,
                                      _validation_response, 0);
    }
    String local_address = null;
    try {
      local_address = InetAddress.getLocalHost().getHostAddress();
    }
    catch(UnknownHostException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                      "Not able to pull address from local host: " + e, _validation_response, 0);
    }
    if(_current_partner_alias.equals(local_address)) {
		/*
		If the current partner is actually a server or tool from the same
		Hummer, then the alias will not be present in the HummerKeyStore.
		The information needed to send back should be pulled from the
		HummerKeyStore's personal information.
		*/
        _signing_signature_object = _key_store.getPersonalSignSignature();

        signer_verify_signature = _key_store.getPersonalVerifySignature();

        _server_certificate = _key_store.getPersonalHummerCertificate();

        _current_client_access_table = StaticMethods.getUserFullAccessTable();
    }
    else if(_signer_alias.equals(_personal_config.getAlias()) &&
              !_current_partner_alias.equals(_personal_config.getAlias())) {

		/*
		Local accounts are allowed to use HummerCertificates that may be outdated, as long
		as the PublicKey is the same that the system recognizes as what the user is supposed to have.
		This is done by looking for the PublicKey that is associated with an alias rather than
		what the current PublicKey is.
		(see HummerKeyStore for details)
		*/
      try {
		/*
		The key store should only be reloaded if the alias is not
		another server logging in from the same key store (partner_alias=personal_alias)
		*/
        _key_store.load(true, null);
        if(!_key_store.contains(_current_partner_alias)) {
          throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation:  Invalid login by \"" + _current_partner_alias + "\".",
                                          _validation_response, HummerSocketException._INVALID_LOGIN);
        }
        if(_key_store.getAccessTable(_current_partner_alias) == null) {
          throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                          "An account does not exist for the given alias.(" +
                                          _current_partner_alias + ")",
                                          _validation_response, HummerSocketException._INVALID_LOGIN);
        }
        _signing_signature_object = _key_store.getPersonalSignSignature();
        signer_verify_signature = _key_store.getPersonalVerifySignature(_current_partner_alias);
		/*
		If the PublicKey known by the client is not the most recent or
		the access table in the HummerCertificate is not equal to the current
		access table known by the system, then a new HummerCertificate must be
		given to the client.
		*/
        _new_certificate_needed = (!_key_store.hasUpdatedHummerCertificate(_current_partner_alias) ||
                                   !_key_store.equalAccessTable(_current_partner_alias,
                                                             _client_certificate.getAccessTable()));

        _server_certificate = _key_store.getPersonalHummerCertificate(_current_partner_alias);

        _current_client_access_table = _key_store.getAccessTable(_current_partner_alias);
      }
      catch(NullPointerException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not find server's " +
                                        "PublicKey associated with alias \"" + _current_partner_alias + "\"" +
                                        "(server and client using same HummerKeyStore?)",
                                        _validation_response, HummerSocketException._INVALID_LOGIN);
      }
      catch(CertificateException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " + e,
                                        _validation_response, 0);
      }
      catch(OptionalDataException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not reload key store: " + e, _validation_response, 0);
      }
      catch(ClassNotFoundException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not reload key store: " + e, _validation_response, 0);
      }
      catch(IOException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not reload key store: " + e, _validation_response, 0);
      }
    }
    else if(_key_store.contains(_signer_alias) && !_current_partner_alias.equals("admin")) {
		/*
		The key store should only be reloaded if the alias is not
		another server logging in from the same key store (partner_alias=personal_alias)
		*/
      try {
        _key_store.load(true, null);
        if(_key_store.isNewAccount(_signer_alias)) {
          throw new HummerSocketException("Invalid certificate recieved.  " +
                                          "Signer alias is for unitiailized account (no signature key exists).",
                                          _validation_response,
                                          HummerSocketException._HUMMER_CERTIFICATE_SIGNER_ACCOUNT_UNITIALIZED);
        }
        if(!_key_store.isCertificateHonored(_signer_alias)) {
          throw new HummerSocketException("Invalid certificate recieved.  " +
                                          "Certifiates for this alias are not honored",
                                          _validation_response, HummerSocketException._NON_TRUSTED_HUMMER_CERTIFICATE);
        }
        signer_verify_signature = _key_store.getVerifySignature(_signer_alias);
        _server_certificate = _key_store.getHummerCertificate(_signer_alias);
        _signing_signature_object = _key_store.getPersonalSignSignature(_signer_alias);

        _current_client_access_table = _client_certificate.getAccessTable();
      }
      catch(OptionalDataException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not reload key store: " + e, _validation_response, 0);
      }
      catch(ClassNotFoundException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not reload key store: " + e, _validation_response, 0);
      }
      catch(IOException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " +
                                        "Could not reload key store: " + e, _validation_response, 0);
      }
    }
    else {
      throw new HummerSocketException("Unknown signer alias (" + _signer_alias + ")",
                                      _validation_response, HummerSocketException._UNKNOWN_HUMMER_CERTIFICATE_SIGNER);
    }
  
		/*
		The correct PublicKey and HummerCertificate have been found (is they exist), so
		verify the HummerCertificate (client_certificate) that the client sent
		*/

    boolean client_certificate_valid = false, client_certificate_expired = true;
    if(signer_verify_signature != null) {
      try {
        client_certificate_expired = _client_certificate.isExpired();
        client_certificate_valid = _client_certificate.isValid(signer_verify_signature);
      }
      catch(CertificateException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " + e,
                                        _validation_response, 0);
      }
      if(_server_certificate == null) {
        _server_certificate = _key_store.getHummerCertificate(_signer_alias);
      }
    }
    if(client_certificate_expired) {
      throw new HummerSocketException("DefaultServerProtocol:validation: HummerCertificate is expired " +
                                      "(signer alias: " + _signer_alias + ")", _validation_response,
                                      HummerSocketException._EXPIRED_HUMMER_CERTIFICATE);
    }
    if(!client_certificate_valid) {
      throw new HummerSocketException("DefaultServerProtocol:validation: HummerCertificate is not valid " +
                                      "(signer alias: " + _signer_alias + ")", _validation_response,
                                      HummerSocketException._INVALID_LOGIN);
    }
  }

  public Object initializedAccountPrepareResponse()
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException {
    HummerSignedObject signed_output = null;
    Response output = new Response();
    output._type = Response._OK;
    _input_message_id = StaticMethods.getUniqueTimestamp();
    output._id = _input_message_id;
    output._message = new Long(_output_message_id);
    output._data = new Vector();
    output._data.add(this._name);
    output._data.add(_server_certificate);
    if(_new_certificate_needed) {
      HummerCertificate new_certificate = null;
      try {
        new_certificate = _key_store.generateHummerCertificate(_current_partner_alias);
      }
      catch(CertificateException e) {
        throw new HummerSocketException("DefaultServerProtocol:initializedAccountPrepareResponse: " + e,
                                        null, 0);
      }
      _current_client_access_table = new_certificate.getAccessTable();
      output._data.add(new_certificate);
    }
    try {
      signed_output = new HummerSignedObject(output, _signing_signature_object);
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountPrepareResponse: " + e, null, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountPrepareResponse: " + e, null, 0);
    }
    return signed_output;
  }

  public void newAccountCheckInputDataFormat() throws HummerSocketException {
    if(_validation_response._data.size() != 6) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountCheckInputDataFormat: " +
                                      "Size of Response._data Vector must be 6.",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }

    if(_validation_response._data.get(3) == null) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountCheckInputDataFormat: " +
                                      "Null PublicKey recieved.", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.get(4) == null) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountCheckInputDataFormat: " +
                                      "Null message digest recieved.", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.get(5) == null) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountCheckInputDataFormat: " +
                                      "Null random value and timestamp recieved.",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
  }

  public void newAccountVerifyInformation()
         throws HummerSocketException, KeyStoreException, UnknownAliasException, UnloadedKeyStoreException {
    try {
      _current_partner_alias = (String)(_validation_response._data.get(1));
    }
    catch(ClassCastException e) {
      throw new HummerSocketException("DefaultServerProtocol:initializedAccountVerifyInformation: " + e,
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
    _output_message_id = _validation_response._id;
    if(!_key_store.contains(_current_partner_alias)) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountVerifyInformation: " +
                                      "Alias (" + _current_partner_alias + ") is not known by this system.",
                                      _validation_response, HummerSocketException._INVALID_LOGIN);
    }

    _output_message_id = _validation_response._id;
    PublicKey client_public_key = (PublicKey)(_validation_response._data.get(3));
    byte[] message_digest = null,
           random_timestamp = null;
    HummerCertificate hummer_certificate = null;
    try {
      message_digest = (byte[])(_validation_response._data.get(4));
      random_timestamp = (byte[])(_validation_response._data.get(5));
      _new_random_timestamp = StaticMethods.getRandomTimestampBytes();
      _new_message_digest = _key_store.getDigest(_current_partner_alias, _new_random_timestamp,
                                                _key_store.getPersonalKeyPair().getPublic());

      hummer_certificate = _key_store.initializeAccount(_current_partner_alias,
                                                        message_digest,
                                                        random_timestamp,
                                                        client_public_key);
    }
    catch(NoSuchAlgorithmException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountVerifyInformation: " +
                                      "Could not initialize account.  " + e,
                                      _validation_response, 0);
    }
    catch(CertificateException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountVerifyInformation: " +
                                      "Could not initialize account.  " + e,
                                      _validation_response, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountVerifyInformation: " +
                                      "Could not initialize account.  " + e,
                                      _validation_response, 0);
    }
    try {
      if(_verifying_signature_object == null) {
        _verifying_signature_object = StaticMethods.getVerifyingSignatureObject(client_public_key);
      }
      _signing_signature_object = _key_store.getPersonalSignSignature();
    }
    catch(NoSuchAlgorithmException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountVerifyInformation: " +
                     "This system does support the algorithm for signatures: " + e, _validation_response, 0);
    }
    catch(InvalidKeyException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountVerifyInformation: " + e,
                                      _validation_response, 0);
    }
    _current_client_access_table = _key_store.getAccessTable(_current_partner_alias);
    _input_message_id = StaticMethods.getUniqueTimestamp();
  }

  public Object newAccountPrepareResponse()
         throws HummerSocketException, UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
    Response output = new Response();
    output._id = _input_message_id;
    output._type = Response._OK;
    output._message = new Long(_output_message_id);
    output._data.add(this._name);
    try {
      output._data.add(_key_store.getPersonalKeyPair().getPublic());
      output._data.add(_key_store.generateHummerCertificate(_current_partner_alias));
    }
    catch(CertificateException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountPrepareResponse: " + e,
                                      null, 0);
    }
    output._data.add(_new_message_digest);
    output._data.add(_new_random_timestamp);
    HummerSignedObject signed_output = null;
    try {
      signed_output = new HummerSignedObject(output, _signing_signature_object);
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountPrepareResponse: " + e, null, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultServerProtocol:newAccountPrepareResponse: " + e, null, 0);
    }
    return signed_output;
  }

	/**
	This method is empty, because there are no functions required after sending response to client.
	*/
  public Object initializedAccountFinalize() { return null; }

	/**
	This method is empty, because there are no functions required after sending response to client.
	*/
  public Object newAccountFinalize() {  return null; }

  public Response verifyInput(Object input_object) throws HummerSocketException {
    Response input = null;
    if(input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.Response")) {
      input = (Response)input_object;
      if(input._message == null) {
        throw new HummerSocketException("DefaultServerProtocol:verifyInput: " +
                                        "Read Response object of type _EXCEPTION, but _message is null.", null,
                                        HummerSocketException._INVALID_DATA_FORMAT);
      } else if(!input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException")) {
        throw new HummerSocketException("DefaultServerProtocol:verifyInput: " +
                                        "Read Response object of type _EXCEPTION, but " +
                                        "_message is not of type HummerSocketException.", null,
                                        HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)input._message;
    }
    else if(!input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject")) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " +
                                 "Invalid type of object recieved from input stream.  Should be HummerSignedObject.",
                                 null, HummerSocketException._INVALID_DATA_FORMAT);
    }
    HummerSignedObject signed_object = (HummerSignedObject)input_object;
    boolean signed_object_valid = false;
    try {
      signed_object_valid = signed_object.verify(_verifying_signature_object);
      input = (Response)(signed_object.getObject());
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " + e, input, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " + e, input, 0);
    }

    if(!signed_object_valid) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " +
                                      "Invalid HummerSignedObject (signature was not valid) recieved.", null, 0);
    }
    _input_message_id++;
    if(input._id != _input_message_id) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " +
                                      "Incorrect message ID recieved", input,
                                      HummerSocketException._INVALID_MESSAGE_ID);
    }
    if((input._type & Response._EXCEPTION) == Response._EXCEPTION) {
      if(input._message.getClass().getName().equals("java.lang.String")) {
        input._message = new HummerSocketException((String)input._message,
                                                   null, HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)(input._message);
    }
    if((input._type & Response._OK) == 0) {
      throw new HummerSocketException("DefaultServerProtocol:readResponseObject: " +
                                      "Invalid _type recieved on Response object (" + input._type + ").",
                                      input, HummerSocketException._INVALID_DATA_FORMAT);
    }
    try {
      if(!signed_object.verify(_verifying_signature_object)) {
        throw new HummerSocketException("DefualtServerProtocol:verifyInput: " +
                                        "Signed object recieved could not be verified with client PublicKey.",
                                        _validation_response, HummerSocketException._INVALID_SIGNATURE);
      }
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " + e, _validation_response, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultServerProtocol:verifyInput: " + e, _validation_response, 0);
    }
    return input;
  }

  public HummerProtocol updateProtocol() throws HummerSocketException {
    String requested_protocol = _validation_response._data.get(0).toString();
    if(!requested_protocol.equals("default")) {
      throw new HummerSocketException("DefaultServerProtocol:updateProtocol: " +
                                      "Cannot connect with protocol: " + requested_protocol,
                                      null, HummerSocketException._PROTOCOL_NOT_KNOWN);
    }
    else {
      return this;
    }
  }

  public Object prepareOutput(Response write_object) throws HummerSocketException {
    if(write_object == null) {
      write_object = new Response();
      write_object._message = null;
      write_object._data = null;
    }

    _output_message_id++;
    write_object._id = _output_message_id;
    HummerSignedObject signed_object = null;
    try {
      signed_object = new HummerSignedObject(write_object, _signing_signature_object);
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultServerprotocol:prepareOutput: " +
                                      "System error when generating HummerSignedObject.  " + e, null, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultServerprotocol:prepareOutput: " +
                                      "System error when generating HummerSignedObject.  " + e, null, 0);
    }
    return signed_object;
  }
}