/*
Java Hummer Project
File: HummerClientSocket.java
Date: 4-5-99
Author: Chad Ward
Description:  Socket interface for a client-side connection of communication.
              This "default" version does not use any of the JCE classes, therefore
              it does do any two-way encryption/decryption.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;
import java.util.Hashtable;
import java.security.KeyStoreException;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;


/**
Socket interface for a client-side connection of communication.
The type of protocol is determined at runtime by looking at the
Hummer.propties file for the 'protocol' value. <br>
<h4>Class Usage and Flow:</h4>
<ul>
  <li>Constructor - Initializes variables in socket to given parameters, and attempts to get
      the PrivateKey for signing objects sent to the server.
  <li>validation - Method used to communicate to validate this HummerClientSocket with the server.
  <li>readResponseObject - Reads and validates a Response object sent by the server.
  <li>#addManager - Method used by a Hummer agent when connected to a new Manager, to
      correctly initialize the HummerKeyStore to account for the new manager.
      This method should be used when adding a manager rather than addServer.
  <li>#addPeerModerator - Method used by a Hummer agent when connected to a new Peer moderator, to
      correctly initialize the HummerKeyStore to account for the new peer moderator.
      This method should be used when adding a peer moderator rather than addServer.
  <li>#addServerAlias/addUserAlias/addPeerAlias methods.  Make calls to server socket to
      add a given alias to the server's HummerKeyStore
  <li>#deleteAlias - Makes call to server to delete an alias from the server's HummerKeyStore.
  <li>#getAccessTable - Makes call to server to return the server's access table for a given alias
  <li>#getAliasList - Makes a call to server to return a Vector representing all of
      the aliases in the server's HummerKeyStore
  <li>#resetAlias - Makes a call to the server to reset the account for the given alias, replacing with
      a new password.
  <li>setHummerCertificate - Makes call to server to replace the HummerCertificate for this client with
      a new HummerCertificate
</ul>
# - Must access the HummerServerSocket as "admin"
<b> Following any command that sends a request to the server, there must be a Response
object sent to the server.  Failure to do this will cause errors </b>
<h4>Usage Example</h4>
Create a PersonalConfig object that will be used to log into a server account that has
never been logged into before (new account).
	<blockquote><b>
	PersonalConfig personal_config = new PersonalConfig("admin", "password".getBytes());
	</b></blockquote>

Open a socket the server.  The initialization of address and port are not shown here.
	<blockquote><b>
	Socket socket = new Socket(address, port);
	</b></blockquote>

Create a HummerKeyStore object and load it.  Here the HummerKeyStore is assumed to have already
been written to a file called "key_store.hum"
	<blockquote><b>
	HummerKeyStore key_store = new HummerKeyStore(); <br>
	key_store.load(false, "jwilson");
	</b></blockquote>

Create a HummerClientSocket object with the values create above.
	<blockquote><b>
	HummerClientSocket hummer_socket = new HummerClientSocket(personal_config, socket, key_store);
	</b></blockquote>

Validate communication with the server. (The socket will automatically recognize that this
validation is for a new account because personal_config.isNewAccount() returns true).  Null
is used because the client is logging into a local account on the server.  The client could also
log in using a HummerCertificate given by another trusted server, so the address of this trusted server must
be given.
	<blockquote><b>
	hummer_socket.validate(null);
	</b></blockquote>


Now the client has been validated to the server, so normal communication can occur.
The following code makes a request to the server to add a new user to the system.
	<blockquote><b>
	hummer_socket.addUserAlias("jwilson", "jpass");
      hummer_socket.writeResponseObject(null);
	</b></blockquote>

<br>
If the user's account had already been initialized on the server (had previously logged into the server),
the following code would be used to accomplish the same above tasks.
	<blockquote><b>
	PersonalConfig personal_config = new PersonalConfig("admin", null); <br>
      personal_config.initializeAccount();	// Redundant, but should be used for forward-compatibility <br>
	Socket socket = new Socket(address, port); <br>
	HummerKeyStore key_store = new HummerKeyStore(); <br>
	key_store.load(false, "jwilson"); <br>
	HummerClientSocket hummer_socket = new HummerClientSocket(personal_config, socket, key_store); <br>
	hummer_socket.validate(null); <br>
	hummer_socket.addUserAlias("jwilson", "jpass"); <br>
      hummer_socket.writeResponseObject(null);
	</b></blockquote>


@author Chad Ward
*/
public final class HummerClientSocket extends HummerSocket {

	/**
	Default constructor, calls the super class constructor with the given
	parameters.
	*/
  public HummerClientSocket(PersonalConfig personal_config, Socket socket, HummerKeyStore key_store)
         throws NoSuchAlgorithmException, UnknownHostException,
                HummerSocketException, IOException, UnloadedKeyStoreException, KeyStoreException {
    super(false, personal_config, socket, key_store);
  }

	/**
	@return True if the current personal alias is "admin", or if the current personal
	alias equals the partner alias.
	*/
  public boolean hasAdminAccess() {
    return (_current_protocol.getPersonalAlias().equals("admin") ||
            _current_protocol.getPersonalAlias().equals(_current_protocol.getCurrentPartnerAlias()));
  }

	/**
	Does all handshaking and validation with the server that is being connected to.
	This also includes updating the PublicKey's for either the server or the client if either
	of them is outdated.
	@param signer_alias Alias of Hummer that signed a HummerCertificate recognized by the server that
		a connection is being made to. A null value or empty String ("") will cause the signer 
		alias to be the IP address that the socket is connected to. A HummerCertificate is
		required to login to any Hummer (unless a new account is being initialized).  This signer
		of this HummerCertificate must be a Hummer that is trusted by the server that is being logged
		in to.  This variable, <i>signer_alias</i>, is the alias (IP address) of the mutually
		trusted server. (For Hummer's where a local account exists, the IP of the Hummer can be	used)
	@throws HummerSocketException Thrown when there has been a problem with validating to the server.
	@see HummerSocket
	@see HummerServerSocket
	*/
  public void validation(Object protocol_information)
         throws HummerSocketException, KeyStoreException, UnrecoverableKeyException,
                UnloadedKeyStoreException, UnknownAliasException, CertificateException,
                IOException {
    long start_time = System.currentTimeMillis(), time;
    Object output = null;
    Object input = null;
    if(_has_validated) {
      throw new HummerSocketException("Have already called validation().", null, 0);
    }
    try {
		/*
		Initializes protocol specific information (default protocol will set partner and signer aliases)
		*/
      _current_protocol.initializeInformation(protocol_information, _socket.getInetAddress().getHostAddress());
      if(_current_protocol.isNewAccountValidation()) {

		/**************************************
		The following is for new account validation
		**************************************/

		/*
		Prepare a Reponse object to validate to the server.
		*/
        _object_output_stream = new ObjectOutputStream(_socket.getOutputStream());
        output = _current_protocol.newAccountPrepareResponse();
		/*
		Send the Response to the server to verify the client.
		*/
        _object_output_stream.writeObject(output);

        _object_input_stream = new ObjectInputStream(_socket.getInputStream());
        try {
		/*
		Read in the response from the server
		and check the validity of the data recieved.
		*/
          input = _object_input_stream.readObject();
          _current_protocol.checkBasicInputDataFormat(input);
          /*
          The server may request another protocol, and it should be determined
          whether a new protocol should be tried, disconnect, or try to continue
          with a different trusted protocol.
          */
          _current_protocol = _current_protocol.updateProtocol();
        }
        catch(ClassNotFoundException e) {
          throwException("HummerServerSocket:validation: " +
                         "Invalid data recieved over socket.  " + e, null,
                         HummerSocketException._INVALID_DATA_FORMAT);
        }
		/*
		Check that the data recieved is valid for new account login.	
		*/
        _current_protocol.newAccountCheckInputDataFormat();
		/*
		Verify that the login of the client is valid.
		*/
        _current_protocol.newAccountVerifyInformation();

		/*
		Perform any final things that may be needed for the protocol.
		If these require data to be sent back, then a non-null
		value will be returned, and sent back to the client.
		*/
        if((output = _current_protocol.newAccountFinalize()) != null) {
          _object_output_stream.writeObject(output);
        }
      }
      else {
		/*
		Prepare a Reponse object to validate to the client.
		*/
        output = _current_protocol.initializedAccountPrepareResponse();

        _object_output_stream = new ObjectOutputStream(_socket.getOutputStream());
		/*
		Send the Response to the client to verify the server.
		*/
        _object_output_stream.writeObject(output);

        _object_input_stream = new ObjectInputStream(_socket.getInputStream());
        try {

		/*
		Read in the response from the server
		*/
          input = _object_input_stream.readObject();
          _current_protocol.checkBasicInputDataFormat(input);
          /*
          The server may request another protocol, and it should be determined
          whether a new protocol should be tried, disconnect, or try to continue
          with a different trusted protocol.
          */
          _current_protocol = _current_protocol.updateProtocol();
        }
        catch(ClassNotFoundException e) {
          throwException("HummerServerSocket:validation: " +
                         "Invalid data recieved over socket.  " + e, null, HummerSocketException._INVALID_DATA_FORMAT);
        }
		/*
		Check that the data recieved is valid for new account login.	
		*/
        _current_protocol.initializedAccountCheckInputDataFormat();
		/*
		Verify that the login of the client is valid.
		*/
        _current_protocol.initializedAccountVerifyInformation();

		/*
		Perform any final things that may be needed for the protocol.
		If these require data to be sent back, then a non-null
		value will be returned, and sent back to the client.
		*/
        if((output = _current_protocol.initializedAccountFinalize()) != null) {
          _object_output_stream.writeObject(output);
        }
      }
    }
    catch(HummerSocketException e) {
      throwException(e);
    }
    catch(KeyStoreException e) {
      throwException(e.toString(), null,
                     HummerSocketException._INVALID_LOGIN);
    }
    _has_validated = true;
  }

	/**
	Reads a Response object from the socket and checks the signature before
	returning it
	@throws HummerSocketException When validation() has not been called or the signature is invalid
	*/
  public Response readResponseObject()
         throws HummerSocketException, IOException {
    Response input = null;
    if(!_has_validated) {
      throw new HummerSocketException("HummerClientSocket:readResponseObject: " +
                                      "Have not called validation() yet.", null, 0);
    }
    if(_object_input_stream == null) {
      _object_input_stream = new ObjectInputStream(_socket.getInputStream());
    }
    Object input_object = null;
    try {
      input_object = _object_input_stream.readObject();
      input = _current_protocol.verifyInput(input_object);
    }
    catch(java.io.EOFException e) {
       throw new HummerSocketException("HummerClientSocket:readResponseObject: " +
                                       "Server has disconnected prematurely from the client.", null, 0);
    }
    catch(ClassNotFoundException e) {
      throw new HummerSocketException("HummerClientSocket:readResponseObject: " +
                                      "Invalid stream input (could not generate a valid object from stream)." + e,
                                      null, 0);
    }
    catch(HummerSocketException e) {
      throwException(e);
    }
    catch(NullPointerException e) {
    	 throwException("HummerClientSocket:readResponseObject: Null value recieved from client",
    	                (Response)input_object, HummerSocketException._INVALID_DATA_FORMAT);
    }

    if((input._type & Response._EXCEPTION) == Response._EXCEPTION) {
      if(input._message.getClass().getName().equals("java.lang.String")) {
        input._message = new HummerSocketException((String)input._message, null,
                                                   HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)(input._message);
    }
    return input;
  }





	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li> Creates a Response object with _type equal to _ADD_MANAGER
	  <li> Adds the personal config object needed to login to the manager server
	  <li> Adds the address of the manager to add
	  <li> Send the Response object to the server
	</ul>
	@param account_config The PersonalConfig object needed for the ConfigServer to login into
	the manager server.
	*/
  public void addManager(PersonalConfig account_config, String manager_address) 
         throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:addManager: " +
                                      "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._ADD_MANAGER;
    output._data.add(account_config);
    output._data.add(manager_address);
    writeResponseObject(output);
  }

	/**
	<h4>Method Usage and Flow:</h4> <br>
	<b> This is called by the server - Do not call this,
	it is used by HummerServerSocket</b>
	<ul>
	  <li> Sets the account for the current server to  <b>trusted (HummerCertificates
	       signed by that server are trusted)</b>
	  <li> Changes the access table so that the server can log into this <i>server</i>
	       (must be server, or it doesn't make sense to add a mangager)
	  <li>Generates a HummerCertificate for the manager and sends it to the manager.
	</ul>
	*/
  public void addManager()
         throws UnknownAliasException, KeyStoreException, UnloadedKeyStoreException, IOException,
                HummerSocketException, CertificateException {
    _key_store.setServerTrust(_current_protocol.getCurrentPartnerAlias(), true);
    _key_store.setAccessTable(_current_protocol.getCurrentPartnerAlias(), StaticMethods.getDefaultServerAccessTable());
    setHummerCertificate();
    _key_store.store(true, null);
  }

	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li> Creates a Response object with _type equal to _ADD_PEER_MODERATOR
	  <li> Adds the personal config object needed to login to the moderator
	  <li> Adds the address of the moderator to add
	  <li> Send the Response object to the server
	</ul>
	@param account_config The PersonalConfig object needed for the ConfigServer to login into
	the manager server.
	*/
  public void addPeerModerator(PersonalConfig account_config, String moderator_address) 
         throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:addPeerModerator: " +
                                      "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._ADD_PEER_MODERATOR;
    output._data.add(account_config);
    output._data.add(moderator_address);
    writeResponseObject(output);
  }

	/**
	<h4>Method Usage and Flow:</h4> <br>
	<b> This is called by the server - Do not call this,
	it is used by HummerServerSocket</b>
	<ul>
	  <li> Sets the account for the current server to  <b>trusted (HummerCertificates signed 
	       by that server are trusted)</b>
	  <li> Changes the access table so that the server can log into this <i>server</i>
	       (must be server, or it doesn't make sense to add a peer moderator)
	  <li>Generates a HummerCertificate for the peer moderator and sends it to the peer moderator.
	</ul>
	*/
  public void addPeerModerator()
             throws UnknownAliasException, KeyStoreException, UnloadedKeyStoreException,
                    IOException, HummerSocketException, CertificateException {
    _key_store.setAliasType(_current_protocol.getCurrentPartnerAlias(),
                            AliasInfo._PEER_ALIAS_TYPE);
    _key_store.setAccessTable(_current_protocol.getCurrentPartnerAlias(),
                              StaticMethods.getDefaultPeerAccessTable());
    setHummerCertificate();
    _key_store.store(true, null);
  }

	/**
	<h4>Method Usage and Flow:</h4> <br>
	Makes call to server to add given user alias (Will throw exception if not for admin account).
	@param alias The alias to add to the system. If this alias already exists, then a HummerSocketException
		will be thrown.
	@param password The password to use for the account
	*/
  public void addPeerAlias(String alias, byte[] password) throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:addPeerAlias: " +
                                      "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._ADD_ALIAS;
    try {
      alias = InetAddress.getByName(alias).getHostAddress();
    }
    catch(UnknownHostException e) {
      throw new HummerSocketException("HummerClientSocket:addPeerAlias: " +
                                      "Server alias \"" + alias +
                                      "\" is not a valid host name/IP.", null,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    output._data.add(alias);
    output._data.add(new Integer(AliasInfo._PEER_ALIAS_TYPE));
    output._data.add(null);
    output._data.add(password);
    writeResponseObject(output);
  }


	/**
	<h4>Method Usage and Flow:</h4>
	Makes call to server to add given user alias (Will throw exception if not for admin account).
	@param alias The alias to add to the system. If this alias already exists, then a HummerSocketException
		will be thrown.
	@param password The password to use for the account
	*/
  public void addUserAlias(String alias, byte[] password) throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:addUserAlias: " +
                                      "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._ADD_ALIAS;
    output._data.add(alias);
    output._data.add(new Integer(AliasInfo._USER_ALIAS_TYPE));
    output._data.add(null);
    output._data.add(password);
    writeResponseObject(output);
  }

	/**
	<h4>Method Usage and Flow:</h4>
	Makes call to server to add given server alias (Will throw exception if not for admin account).
	@param alias The alias to add to the system. If this alias already exists, then a HummerSocketException
		will be thrown.
	@param certificates_trusted True if the certificates given by this server will be honored (given
		access to the system as if the user held an account on this system.  admin accounts witheld)
	@param password The password to use for the account
	*/
  public void addServerAlias(String alias, boolean certificates_trusted, byte[] password)
         throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:addUserAlias: " +
                                      "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._ADD_ALIAS;
    try {
      alias = InetAddress.getByName(alias).getHostAddress();
    }
    catch(UnknownHostException e) {
      throw new HummerSocketException("HummerClientSocket:addServerAlias: " +
                                      "Server alias \"" + alias +
                                      "\" is not a valid host name/IP.", null,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    output._data.add(alias);
    output._data.add(new Integer(AliasInfo._SERVER_ALIAS_TYPE));
    output._data.add(Boolean.FALSE);
    output._data.add(password);
    writeResponseObject(output);
  }

	/**
	Makes call to server to reset given alias (Will throw exception if not for admin account).
	@param alias The alias to add to the system. If this alias already exists, then a HummerSocketException
		will be thrown.
	@param password The password to use for the account
	*/
  public void resetAlias(String alias, byte[] password)
         throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:addUserAlias: " +
                                      "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }

    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._RESET_ALIAS;
    output._data.add(alias);
    output._data.add(password);
    writeResponseObject(output);
  }


	/**
	Makes call to server to delete given alias (Will throw exception if not for admin account).
	*/
  public void deleteAlias(String alias) throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:deleteAlias: " +
                                  "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._DELETE_ALIAS;
    output._data.add(alias);
    writeResponseObject(output);
  }



	/**
	Makes call to server to retrieve list aliases (Will throw exception if not for admin account).
	Each of the objects in the Vector is an array of objects of size=3
	Object[0]=String alias
	Object[1]=Integer alias_type
	Object[2]=Boolean initialized_account
	*/
  public Vector getAliasList() throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:getAliasList: " +
                                  "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._GET_ALIASES;
    writeResponseObject(output);
    return (Vector)(readResponseObject()._data);
  }


	/**
	Makes call to server to retrieve the access values for a given alias
	(Will throw exception if not for admin account).
	*/
  public Hashtable getAccessTable(String alias) throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:getAccessTable: " +
                                  "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._GET_ACCESS_VALUES;
    output._data.add(alias);
    writeResponseObject(output);
    return (Hashtable)(readResponseObject()._message);
  }

	/**
	Makes call to server requesting it to re-load its key store from
	file.
	(Will throw exception if not for admin account).
	*/
  public void reloadKeyStore() throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:reloadKeyStore: " +
                                      "Server commands can only be performed by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._RELOAD_KEY_STORE;
    writeResponseObject(output);
  }


	/**
	Makes call to server to set the access values for a given alias
	(Will throw exception if not for admin account).
	*/
  public void setAccessTable(String alias, Hashtable access_table)
         throws HummerSocketException, IOException {
    if(!hasAdminAccess()) {
      throw new HummerSocketException("HummerClientSocket:setAccessTable: " +
                                  "Server accounts can only be modified by admin.", null,
                                      HummerSocketException._INVALID_COMMAND);
    }
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._SET_ACCESS_VALUES;
    output._data.add(alias);
    output._data.add(access_table);
    writeResponseObject(output);
  }


	/**
	Makes call to server to reset the HummerCertificate for this client
	*/
  public void setHummerCertificate()
         throws HummerSocketException, IOException, UnloadedKeyStoreException, UnknownAliasException,
                CertificateException, KeyStoreException {
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._SET_HUMMER_CERTIFICATE;
    output._data.add(_key_store.generateHummerCertificate(_current_protocol.getCurrentPartnerAlias()));
    writeResponseObject(output);
  }

	/**
	Makes a call to server to update the PublicKey for this client.  This should
	not be required to be called manually, because it automatically updated
	upon login (when the PublicKey is outdated).
	*/
  protected void updatePublicKey()
         throws HummerSocketException, IOException, UnloadedKeyStoreException, CertificateException,
                UnknownAliasException, KeyStoreException {
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._UPDATE_PUBLIC_KEY;
    output._data.add(_key_store.getPersonalKeyPair().getPublic());
    writeResponseObject(output);
    Response input = readResponseObject();
    _key_store.setHummerCertificate(_current_protocol.getCurrentPartnerAlias(), (HummerCertificate)(input._message));
  }

	/**
	Makes a call to server to get a HummerCertificate signed by server for this client.  This should
	not be required to be called manually, because it automatically updated
	upon login (when the HummerCertificate is outdated).
	*/
  public void getHummerCertificate()
         throws HummerSocketException, IOException, CertificateException, UnknownAliasException,
                UnloadedKeyStoreException, KeyStoreException {
    Response output = new Response();
    output._type = Response._OK;
    output._command = Response._GET_HUMMER_CERTIFICATE;
    writeResponseObject(output);
    Response input = readResponseObject();
    _key_store.setHummerCertificate(_current_protocol.getCurrentPartnerAlias(), (HummerCertificate)(input._message));
  }

	/**
	Method used only for measuring the amount of differnece between the current time
	with the given time value (System.currentTimeMillis()).
	*/
  private long difference(long time) {
    return System.currentTimeMillis() - time;
  }
}
