/*
Java Hummer Project
File: PersonalInfo.java
Date: 1-30-99
Author: Chad Ward
Description:  General interface for Hummer communication on an internet socket.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.OptionalDataException;
import java.io.FileNotFoundException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.InvalidKeyException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;

import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.dataobject.PersonalConfig;


/**
Abstract class defining commands for communication using Hummer protocols.
Defines writeResponseObject, and some methods for retrieving information
about the current communication partner. <br><br>

@author Chad Ward
*/
public abstract class HummerSocket {

	/**
	Current protocol being used by the socket.  Could be changed during
	transmission if allowed by the protocol.
	*/
  protected HummerProtocol _current_protocol;

	/**
	False if validate() method has been called, true if it has been called
	*/
  protected boolean _has_validated;


	/**
	Underlying socket used for communication
	*/
  protected Socket _socket;

	/**
	Stream used for reading in object from the socket InputStream
	*/
  protected ObjectInputStream _object_input_stream;

	/**
	Stream used for writing out object to the socket OutputStream
	*/
  protected ObjectOutputStream _object_output_stream;

	/**
	HummerKeyStore object used for storing keys, dates keys created, and NewUserValidator objects for each user
	*/
  protected HummerKeyStore _key_store;

	/**
	Default constructor for all types of HummerSocket objects
	@param personal_config PersonalConfig object representing the owner of this HummerSocket object
	@param socket Socket used for communication with another Hummer socket
	@param key_store HummerKeyStore object holding all key and user information about any other HummerSocket
                   objects that may connect with this one
	*/
  protected HummerSocket(boolean is_server, PersonalConfig personal_config, Socket socket, HummerKeyStore key_store) 
            throws KeyStoreException, UnloadedKeyStoreException,
                   FileNotFoundException, IOException, HummerSocketException {
    Hashtable hummer_properties = StaticMethods.getHummerPropertyTable(false);
    if(!hummer_properties.containsKey("const.PROTOCOL")) {
      throw new HummerSocketException("HummerServerSocket:constructor: " +
                                      "Could not find protocol property in Hummer.properties file", null, 0);
    }
    String protocol_name = hummer_properties.get("const.PROTOCOL").toString();
    try {
      _current_protocol = HummerProtocol.getProtocol(is_server, protocol_name, personal_config, key_store);
    }
    catch(NoSuchAlgorithmException e) {
      throw new HummerSocketException("HummerSocket:constructor: Could not get protocol: " + e, null, 0);
    }
    if(key_store == null) {
      throw new NullPointerException("HummerSocket:constructor: HummerKeyStore reference is null.");
    }
    this._key_store = key_store;
    if(socket == null) {
      throw new NullPointerException("HummerSocket:constructor: Null socket reference.");
    }
    this._socket = socket;
  }


	/**
	Closes the socket connection
	*/
  public void close() throws IOException{
    if(_object_output_stream != null) {
      this._object_output_stream.close();
    }
    if(_object_input_stream != null) {
      this._object_input_stream.close();
    }
    if(_socket != null) {
      this._socket.close();
    }
    this._socket = null;
    this._object_input_stream = null; 
    this._object_output_stream = null;
  }        

	/**
	Writes an end-of-tranmission message to the current partner. (output._type=Response._EOT)
	*/
  public void endOfTransmission(Response output) throws HummerSocketException, IOException {
    if(output == null) {
      output = new Response();
      output._data = null;
      output._message = null;
    }
    output._type = Response._EOT;
    this.writeResponseObject(output);
  }

	/**
	Returns _has_validated value (true is validation method has been called, false otherwise).
	*/
  public boolean hasValidated() { return _has_validated; }

	/**
	True if the account has access to view and change user accounts, as well
	as make requests for servers to reload their key stores.  See
	the implementations of the HummerSocket for when this is true or false.
	*/
  public abstract boolean hasAdminAccess();

	/**
	Returns the alias that communication is currently being done with.
	*/
  public String getCurrentPartnerAlias() { return _current_protocol.getCurrentPartnerAlias(); }

	/**
	Returns the name of the current protocol that is being used.
	Most likely "default"
	*/
  public String getCurrentProtocol() { return _current_protocol.toString(); }

	/**
	Reads a Response object from the socket and checks the signature before
	returning it. Parsing of the Response object is method implementation dependant (servers will have special
	commands that can be sent).

	@throws HummerSocketException When validation() has not been called or the signature is invalid
	*/
  public abstract Response readResponseObject()
         throws HummerSocketException, IOException, OptionalDataException,
                KeyStoreException, UnloadedKeyStoreException, CertificateException, UnknownAliasException;

	/**
	Creates a HummerSocketException using the given message, recieved Response object and the type of
	exception to create.  The exception is sent to the partner (over socket) and then thrown.
	*/
  public void throwException(String exception_message, Response response,
                                int addition_type_information)
            throws HummerSocketException {
    HummerSocketException exception = new HummerSocketException(exception_message,
                                                                response, addition_type_information);
    throwException(exception);
  }

	/**
	Creates a HummerSocketException using the given message, recieved Response object and the type of
	exception to create.  The exception is sent to the partner (over socket) and then thrown.
	*/
  protected void throwException(HummerSocketException exception)
                   throws HummerSocketException {
    writeException(exception);
    throw exception;
  }

	/**
	Places the given HummerSocketException in a Response object, sets the Response._type
	to _EXCEPTION, and sends thre Response object back to the current partner.
	*/
  public final void writeException(HummerSocketException exception) throws HummerSocketException {
    try {
      if(_object_output_stream == null) {
        if(_socket == null) {
          throw new NullPointerException("HummerSocket:throwException: " +
                                         "OutputStream and socket are null.");
        }
        _object_output_stream = new ObjectOutputStream(_socket.getOutputStream());
      }
      Response output = new Response();
      output._type = Response._EXCEPTION;
      output._id = _current_protocol.getOutputMessageID();
      output._message = exception;
      if(_object_output_stream != null) {
        _object_output_stream.writeObject(output);
        _object_output_stream.flush();
        this.close();
      }
    }
    catch(IOException e) {
      throw new HummerSocketException("HummerSocket:throwException: Error while writing exception: " +
                                      exception + "\n\tIOError: " + e, null, 0);
    }
  }    

	/**
	Writes a signed Response object to the socket.

	@param write_object The config object that is being written
	@throws HummerSocketException When validation() has not been called or the signature is invalid
	*/
  public void writeResponseObject(edu.uidaho.hummer.dataobject.Response write_response)
         throws HummerSocketException, IOException {
    if(!_has_validated) {
      throwException("HummerSocket:writeResponseObject: " +
                     "Have not called validation() yet.", new Response(), 0);
    }
    if(_object_output_stream == null) {
      _object_output_stream = new ObjectOutputStream(_socket.getOutputStream());
    }
    Object write_object = null;
    write_object = _current_protocol.prepareOutput((Response)write_response);
    _object_output_stream.writeObject(write_object);
  }

	/**
	Calls protocol for validating the identity of other HummerSocket connection.  Protocol
	varies for each type of HummerSocket
	@param protocol_value String value that may or may not be used by the implementation of HummerSocket
		(See documentation on concrete class for meaning of this parameter)
	*/
  public abstract void validation(Object protocol_value)
         throws HummerSocketException, KeyStoreException, UnrecoverableKeyException,
                UnloadedKeyStoreException, UnknownAliasException, CertificateException,
                IOException, OptionalDataException;
}